/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "GUI_functions.h"
#include "sensitivity.h"
#include "main_window.h"
#include "../GUI_db_objects/database.h"
#include "../GUI_db_objects/schema.h"
#include "../GUI_db_objects/language.h"
#include "../GUI_db_objects/cast.h"
#include "../GUI_db_objects/table.h"
#include "../GUI_db_objects/view.h"
#include "../GUI_db_objects/function.h"
#include "../GUI_db_objects/domain.h"
#include "../GUI_db_objects/trigger.h"
#include "../GUI_db_objects/rule.h"
#include "../GUI_db_objects/sequence.h"
#include "../GUI_db_objects/aggregate.h"
#include "../GUI_db_objects/type.h"
#include "../GUI_db_objects/operator.h"
#include "../GUI_db_objects/other_objects.h"
#include "../functions.h"
#include "../database/db_connection.h"
#include "../database/get_db_objects.h"
#include "../otree/show_otree.h"


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Working with database profiles                                                                                     */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

gboolean
gui_connect_db(T_Server * server, T_Profile * profile){
	if(get_db_objects(server, profile)){
	  profile->connected = TRUE;
		some_db_connected = TRUE;
		//show_object_tree();
	}
	else return FALSE;
	
	return TRUE;
}

gboolean
gui_disconnect_all_dbs(void){
	for(int i = 0; i < o_tree.n_servers; i++){
		for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
			if(o_tree.server[i]->profile[j]->connected){
			 if(!gui_disconnect_db(o_tree.server[i], o_tree.server[i]->profile[j])) return FALSE;
			}
		}
	}
	return TRUE;
}

gboolean
gui_disconnect_db(T_Server * server, T_Profile * profile){
	// close tabs
	if(!close_pages_of_profile(&notebook, profile)) return FALSE;
	
	disconnect_db(profile->conn);
	profile->conn = NULL;
	profile->connected = FALSE;
	
	// check if some database connected, if yes, set database info label and current profile at first connected database
	some_db_connected = FALSE;
	for(int i = 0; i < o_tree.n_servers; i++){
		for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
			if(o_tree.server[i]->profile[j]->connected){
				some_profile_connected();
				some_db_connected = TRUE;
				current_profile = o_tree.server[i]->profile[j];
				gchar *text = bold_text(o_tree.server[i]->profile[j]->alias, "#FFFFFF");
			  gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"info_database")), text);
			  free(text);		
				break;
			}
		}
	}
	
	if(!some_db_connected){
		current_profile = NULL;
		no_profile_connected();
		normal_sensitivity();
		gchar *text = normal_text_with_color("no database connected", "#FFFFFF");
	  gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"info_database")), text);
	  free(text);		
	}
	
	
	// free profile
	free_profile(profile);
	//show_object_tree();
	return TRUE;
}

gboolean
gui_remove_profile(T_Server * server, T_Profile * profile){
	//find profile on server
	gint index = -1;
	for(int i = 0; i < server->n_profiles; i++){
		if(profile == server->profile[i]){
			index = i;
			break;
		}
	}
	if(index == -1) return FALSE; // not found
	
	// run dialog to confirm delete
	gchar * label = append_string(NULL, "Do you really want to remove profile \n'");
	label = append_string(label, profile->alias);	
	label = append_string(label, "'?");
	gchar * l = normal_text(label);
	free(label);
	gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
	free(l);
	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
	if(res == GTK_RESPONSE_YES){
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));	
		return FALSE;
	}
	
	if(profile->connected){
	 if(!gui_disconnect_db(server, profile)) return FALSE;
	}
	free(profile->alias);
	free(profile->dbname);
	free(profile);
	
	// move field of profiles
	for(int i = index+1; i < server->n_profiles; i++){
		server->profile[i-1] = server->profile[i];
	}
	
	server->n_profiles--;
	
	// if no profile then remove server
	index = -1;
	if(server->n_profiles == 0){
		for(int i=0; i < o_tree.n_servers; i++){
				if(server == o_tree.server[i]){
					index = i;
					break;
				}
		}
		if(index == -1) return FALSE; // not found
		
		free(server->profile);
		free(server);
	
	  // move field of servers
		for(int i = index+1; i < o_tree.n_servers; i++){
			o_tree.server[i-1] = o_tree.server[i];
		}
		o_tree.n_servers--;
	}
	
	show_object_tree();
	return TRUE;
}

/*
 * Refresh selected profile
 */
void
gui_refresh(T_Server * server, T_Profile * profile){
	typedef struct open_page{
		gchar * object_name;
		gchar * server;
		gchar * dbname;
		gchar * schema;
		gchar * table;
		gint object_type;
		gint page_num;
		gboolean modified;
	} T_OPage;
	enum types{DATABASE = 0, SCHEMA, LANGUAGE, CAST, TABLE, VIEW, FUNCTION, DOMAIN, AGGREGATE, SEQUENCE, TYPE, OPERATOR, TRIGGER, RULE};
	gint active_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")));
	// get active page
	
	
	// save info about opened pages
	GList *list = NULL;
	for(int i=0; i < notebook.n_pages; i++){
		T_Page * page = notebook.page[i];
		GtkTreePath * p = gtk_tree_path_new_from_string(page->path);
		gint depth = gtk_tree_path_get_depth(p);
		gint * intpath = gtk_tree_path_get_indices(p);
		if(depth >=3 && profile == o_tree.server[intpath[0]]->profile[intpath[2]]){ // refreshed profile
			T_OPage * op = NULL;
			if((op = (T_OPage *) malloc (sizeof(T_OPage)))==NULL){
				fprintf(stderr, "Memory allocation error.");
	 			exit(0);
			}
			switch(depth){
			case 3: op->object_type = intpath[1]; break;
			case 5: op->object_type = intpath[3]+1; break;
			case 7:	op->object_type = intpath[5]+4; break;
			case 9: op->object_type = intpath[7]+8; break;
			}
					
			switch(op->object_type){
				case DATABASE : 
					op->object_name = copy_string(((T_Profile *)(page->object))->dbname);
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = NULL;
					op->schema = NULL;
					op->table = NULL;
					break;
				case SCHEMA : 
					op->object_name = copy_string(((T_Schema *)(page->object))->name);
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = NULL;
					op->table = NULL;
					break;
				case LANGUAGE : 
					op->object_name = copy_string(((T_Language *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = NULL;
					op->table = NULL;
					break;
				case CAST : 
					op->object_name = copy_string(((T_Cast *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = NULL;
					op->table = NULL;
					break;
				case TABLE : 
					op->object_name = copy_string(((T_Table *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;
					break;
				case VIEW : 
					op->object_name = copy_string(((T_View *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case FUNCTION : 
					op->object_name = copy_string(((T_Function *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case DOMAIN : 
					op->object_name = copy_string(((T_Domain *)(page->object))->name);  
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case AGGREGATE : 
					op->object_name = copy_string(((T_Aggregate *)(page->object))->name);  
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case SEQUENCE : 
					op->object_name = copy_string(((T_Sequence *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case TYPE : 
					op->object_name = copy_string(((T_Type *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case OPERATOR : 
					op->object_name = copy_string(((T_Operator *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = NULL;					
					break;
				case TRIGGER : 
					op->object_name = copy_string(((T_Trigger *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = copy_string(((T_Trigger *)(page->object))->table_name); 
					break;
				case RULE : 
					op->object_name = copy_string(((T_Rule *)(page->object))->name); 
					op->server = copy_string(o_tree.server[intpath[0]]->name);
					op->dbname = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->dbname);
					op->schema = copy_string(o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
					op->table = copy_string(((T_Rule *)(page->object))->parent_name); 
					break;
				default: op->object_name = copy_string(""); break;
			}
			op->page_num = gtk_notebook_page_num(GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")), page->page);
			op->modified = page->modified;
			if(page->modified) page->modified = FALSE;
			list = g_list_append (list, op);
		}
	}
	
	// disconect from db
	gui_disconnect_db(server, profile);
	
	// connect to db
	gui_connect_db(server, profile);
	
	// show o_tree
	show_object_tree();
	
	//restore pages
	for(; list != NULL; list = list->next){
		T_OPage * op = (T_OPage *)list->data;
		gchar * path = NULL;
		switch(op->object_type){
				case DATABASE :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->object_name, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									current_profile = o_tree.server[i]->profile[j];
									create_db_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j], path);
									// set position
									GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
									gint n = gtk_notebook_get_n_pages(nb);
									if(op->page_num >= n) op->page_num = n-1;
									for(int g = 0; g < notebook.n_pages; g++) 
										if(notebook.page[g]->object == o_tree.server[i]->profile[j]){ 
											gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
											break;
										}
								}
							}
						}
					}
				 	break;
				case SCHEMA :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											current_profile = o_tree.server[i]->profile[j];
											create_schema_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k], path);
											// set position
											GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
											gint n = gtk_notebook_get_n_pages(nb);
											if(op->page_num >= n) op->page_num = n-1;
											for(int g = 0; g < notebook.n_pages; g++) 
												if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]){ 
													gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
													break;
												}
										}
									}
								}
							}
						}
					}
				  break;
				case LANGUAGE :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_languages; k++){
										if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.language[k]->name) == 0){
											path = append_string(path, ":1:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											current_profile = o_tree.server[i]->profile[j];
											create_language_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.language[k], path);
											// set position
											GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
											gint n = gtk_notebook_get_n_pages(nb);
											if(op->page_num >= n) op->page_num = n-1;
											for(int g = 0; g < notebook.n_pages; g++) 
												if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.language[k]){ 
													gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
													break;
												}
										}
									}
								}
							}
						}
					}
				  break;
				case CAST :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_casts; k++){
										if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.cast[k]->name) == 0){
											path = append_string(path, ":2:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											current_profile = o_tree.server[i]->profile[j];
											create_cast_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.cast[k], path);
											// set position
											GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
											gint n = gtk_notebook_get_n_pages(nb);
											if(op->page_num >= n) op->page_num = n-1;
											for(int g = 0; g < notebook.n_pages; g++) 
												if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.cast[k]){ 
													gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
													break;
												}
										}
									}
								}
							}
						}
					}
				  break;
				case TABLE :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_tables; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->table[l]->name) == 0){
											  	path = append_string(path, ":0:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_table_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->table[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->table[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
				  break;
				case VIEW :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_views; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->view[l]->name) == 0){
											  	path = append_string(path, ":1:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_view_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->view[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->view[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
				  break;
				case FUNCTION :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_functions; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->function[l]->name) == 0){
											  	path = append_string(path, ":2:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_function_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->function[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->function[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
				  break;
				case DOMAIN : 
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_domains; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->domain[l]->name) == 0){
											  	path = append_string(path, ":3:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_domain_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->domain[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->domain[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case AGGREGATE : 
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_aggregates; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->aggregate[l]->name) == 0){
											  	path = append_string(path, ":4:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_aggregate_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->aggregate[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->aggregate[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case SEQUENCE : 
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_sequences; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->sequence[l]->name) == 0){
											  	path = append_string(path, ":5:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_sequence_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->sequence[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->sequence[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case TYPE : 
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_types; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->type[l]->name) == 0){
											  	path = append_string(path, ":6:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_type_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->type[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->type[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case OPERATOR : 
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_operators; l++){
											  if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->operator[l]->name) == 0){
											  	path = append_string(path, ":7:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
												 	current_profile = o_tree.server[i]->profile[j];
													create_operator_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->operator[l], path);
													// set position
													GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
													gint n = gtk_notebook_get_n_pages(nb);
													if(op->page_num >= n) op->page_num = n-1;
													for(int g = 0; g < notebook.n_pages; g++) 
														if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->operator[l]){ 
															gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
															break;
														}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case TRIGGER :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_tables; l++){
											  if(strcmp(op->table, o_tree.server[i]->profile[j]->database.schema[k]->table[l]->name) == 0){
											  	path = append_string(path, ":0:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
													for(int m = 0; m < o_tree.server[i]->profile[j]->database.schema[k]->table[l]->n_triggers; m++){
												 		if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->table[l]->trigger[m]->name) == 0){
												 			path = append_string(path, ":4:");
															gchar * num = itoa_n(m,0);
															path = append_string(path, num);
															free(num);
												 			current_profile = o_tree.server[i]->profile[j];
															create_trigger_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->table[l]->trigger[m], path);
															// set position
															GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
															gint n = gtk_notebook_get_n_pages(nb);
															if(op->page_num >= n) op->page_num = n-1;
															for(int g = 0; g < notebook.n_pages; g++) 
																if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->table[l]->trigger[m]){ 
																	gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
																	break;
																}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				case RULE :
					for(int i = 0; i <o_tree.n_servers; i++){
						if(strcmp(o_tree.server[i]->name, op->server) == 0){
							gchar * num = itoa_n(i,0);
							path = append_string(path, num);
							free(num);
							for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
								if(strcmp(op->dbname, o_tree.server[i]->profile[j]->dbname) == 0){
									path = append_string(path, ":0:");
									gchar * num = itoa_n(j,0);
									path = append_string(path, num);
									free(num);
									for(int k = 0; k < o_tree.server[i]->profile[j]->database.n_schemas; k++){
										if(strcmp(op->schema, o_tree.server[i]->profile[j]->database.schema[k]->name) == 0){
											path = append_string(path, ":0:");
											gchar * num = itoa_n(k,0);
											path = append_string(path, num);
											free(num);
											for(int l = 0; l < o_tree.server[i]->profile[j]->database.schema[k]->n_tables; l++){
											  if(strcmp(op->table, o_tree.server[i]->profile[j]->database.schema[k]->table[l]->name) == 0){
											  	path = append_string(path, ":0:");
													gchar * num = itoa_n(l,0);
													path = append_string(path, num);
													free(num);
													for(int m = 0; m < o_tree.server[i]->profile[j]->database.schema[k]->table[l]->n_rules; m++){
												 		if(strcmp(op->object_name, o_tree.server[i]->profile[j]->database.schema[k]->table[l]->rule[m]->name) == 0){
												 			path = append_string(path, ":5:");
															gchar * num = itoa_n(m,0);
															path = append_string(path, num);
															free(num);
												 			current_profile = o_tree.server[i]->profile[j];
															create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[i]->profile[j]->database.schema[k]->table[l]->rule[m], path);
															GtkNotebook * nb = GTK_NOTEBOOK(glade_xml_get_widget(xml,"main_notebook"));
															gint n = gtk_notebook_get_n_pages(nb);
															if(op->page_num >= n) op->page_num = n-1;
															for(int g = 0; g < notebook.n_pages; g++) 
																if(notebook.page[g]->object == o_tree.server[i]->profile[j]->database.schema[k]->table[l]->rule[m]){ 
																	gtk_notebook_reorder_child(nb,  notebook.page[g]->page , op->page_num);
																	break;
																}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
					break;
				default:  break;
		}
	}
	gint n = gtk_notebook_get_n_pages(GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")));
	if(active_page >= n) active_page = n-1;
	gtk_notebook_set_current_page(GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")) , active_page);
	
	for(list = g_list_first(list); list != NULL; list=list->next){
		T_OPage * op = (T_OPage *)list->data;
		if(op->object_name != NULL) free(op->object_name);
		if(op->server != NULL) free(op->server);
		if(op->dbname != NULL) free(op->dbname);
		if(op->schema != NULL) free(op->schema);
		if(op->table != NULL) free(op->table);
		if(op != NULL) free(op);
	}
	g_list_free(list);
} // gui_refresh()

void
free_profile(T_Profile * profile){
	// free all profile
	for(int i=0; i < profile->database.n_schemas; i++){
		T_Schema * schema = profile->database.schema[i];

		for(int j=0; j < schema->n_tables; j++){
			T_Table * table = schema->table[j];
			
			  for(int k=0; k < table->n_columns; k++){
				  T_Column * column = table->column[k];
					  free(column->name);
					  free(column->comment);
			 		  free(column->type);
					  free(column->size);
						free(column->precision);
					  free(column->default_value);			
					  free(column);	  
			  }
			  if(table->column != NULL) free(table->column);
			  for(int k=0; k < table->n_indexes; k++){
				  T_Index * index = table->index[k];
					  free(index->name);
					  free(index->comment);
	          free(index->column);
					  free(index->predicate);
					  free(index->tablespace);
					  free(index->access_method);      				  
			  }
				if(table->index != NULL) free(table->index);
				
				for(int k=0; k < table->n_fkeys; k++){
					T_FKey * fkey = table->fkey[k];

					  free(fkey->name);
					  free(fkey->comment);
					  free(fkey->column);
					  free(fkey->ref_schema);
					  free(fkey->ref_table);
					  free(fkey->ref_column);
					  free(fkey->on_update_rule);
					  free(fkey->on_delete_rule);
					  free(fkey->match_type);
					  free(fkey->check_time);					
				}
				if(table->fkey != NULL) free(table->fkey);
				
				
				for(int k=0; k < table->n_checks; k++){
					T_Check * check = table->check[k];
			  		free(check->name);
			  		free(check->comment);
			 			free(check->condition);					
			 			free(check);
				}
				if(table->check != NULL) free(table->check);
				
			
				for(int k=0; k < table->n_triggers; k++){
					 T_Trigger * trigger = table->trigger[k];
					  free(trigger->table_name);
					  free(trigger->name);
					  free(trigger->comment);
					  free(trigger->type);
					  free(trigger->for_each);
					  free(trigger->events);
					  free(trigger->function);
					  free(trigger->arguments);
						for(int l = 0; l < trigger->dep_on.n_deps; l++) free(trigger->dep_on.dep[l].type);
						free(trigger->dep_on.dep);
						for(int l = 0; l < trigger->ref_by.n_deps; l++) free(trigger->ref_by.dep[l].type);
						free(trigger->ref_by.dep);
						free(trigger);
				}
				if(table->trigger != NULL) free(table->trigger);
				
				
				for(int k=0; k < table->n_rules; k++){
					T_Rule * rule = table->rule[k];
			 		  free(rule->parent_name);
			 	  	free(rule->name);
					  free(rule->comment);
					  free(rule->event);
					  free(rule->condition);
			 		  free(rule->definition);
						for(int l = 0; l < rule->dep_on.n_deps; l++) free(rule->dep_on.dep[l].type);
						free(rule->dep_on.dep);
						for(int l = 0; l < rule->ref_by.n_deps; l++) free(rule->ref_by.dep[l].type);
						free(rule->ref_by.dep);				
						free(rule);		
				}
				if(table->rule != NULL) free(table->rule);
				for(int k=0; k < table->n_references; k++){
					T_Reference * reference = table->reference[k];
					  free(reference->name);
            free(reference->column);
					  free(reference->ref_schema);
					  free(reference->ref_table);
            free(reference->ref_column);					
				}
				if(table->reference != NULL) free(table->reference);
				
 			  free(table->name);
 			  free(table->owner);
 			  free(table->comment);
 			  free(table->inherited_from);
 			  free(table->tablespace);  
				// Dependencies
				for(int k = 0; k < table->dep_on.n_deps; k++) free(table->dep_on.dep[k].type);
				free(table->dep_on.dep);
				for(int k = 0; k < table->ref_by.n_deps; k++) free(table->ref_by.dep[k].type);
				free(table->ref_by.dep);	
			  free(table->acl);						
			  free(table);
		}
		if(schema->table != NULL) free(schema->table);
		
		for(int j=0; j < schema->n_views; j++){
			T_View * view = schema->view[j];
			
			  for(int k=0; k < view->n_columns; k++){
				  T_Column * column = view->column[k];
				  
					  free(column->name);
					  free(column->comment);
			 		  free(column->type);
					  free(column->size);
						free(column->precision);
					  free(column->default_value);
					  free(column);
			  }
			  free(view->column);
			  
			  for(int k=0; k < view->n_rules; k++){
					T_Rule * rule = view->rule[k];
			 		  free(rule->parent_name);
			 	  	free(rule->name);
					  free(rule->comment);
					  free(rule->event);
					  free(rule->condition);
			 		  free(rule->definition);
						for(int l = 0; l < rule->dep_on.n_deps; l++) free(rule->dep_on.dep[l].type);
						free(rule->dep_on.dep);
						for(int l = 0; l < rule->ref_by.n_deps; l++) free(rule->ref_by.dep[l].type);
						free(rule->ref_by.dep);	
						free(rule);
				}
				free(view->rule);
				
			  free(view->name);
			  free(view->owner);
			  free(view->comment);
			  free(view->body);
				// Dependencies
				for(int k = 0; k < view->dep_on.n_deps; k++) free(view->dep_on.dep[k].type);
				free(view->dep_on.dep);
				for(int k = 0; k < view->ref_by.n_deps; k++) free(view->ref_by.dep[k].type);
				free(view->ref_by.dep);	
			  free(view->acl);	
			  free(view);
		  
		}
		if(schema->view != NULL) free(schema->view);

		for(int j=0; j < schema->n_functions; j++){
			T_Function * function = schema->function[j];
				
				for(int k=0; k < function->n_parameters; k++){
					T_Parameter * parameter = function->parameter[k];
            free(parameter->name);
            free(parameter->type);
            free(parameter->scope);
            free(parameter);
				}
				free(function->parameter);
				
			  free(function->name);
			  free(function->owner);
			  free(function->comment);
			  free(function->parameters);
			  free(function->return_type);
			  free(function->language);
			  free(function->execution_privilegies);
			  free(function->stability);
			  free(function->definition);
				// Dependencies
				for(int k = 0; k < function->dep_on.n_deps; k++) free(function->dep_on.dep[k].type);
				free(function->dep_on.dep);
				for(int k = 0; k < function->ref_by.n_deps; k++) free(function->ref_by.dep[k].type);
				free(function->ref_by.dep);	
			  free(function->acl);	
			  free(function);
		}
		if(schema->function != NULL) free(schema->function);

		for(int j=0; j < schema->n_domains; j++){
			T_Domain * domain = schema->domain[j];
				for(int k=0; k < domain->n_checks; k++){
					T_Check * check = domain->check[k];
						
			  		free(check->name);
			  		free(check->comment);
			 			free(check->condition);
			 			free(check);
				}	
				free(domain->check);
				
			  free(domain->name);
			  free(domain->owner);
			  free(domain->comment);
			  free(domain->data_type);
			  free(domain->default_value);
				// Dependencies
				for(int k = 0; k < domain->dep_on.n_deps; k++) free(domain->dep_on.dep[k].type);
				free(domain->dep_on.dep);
				for(int k = 0; k < domain->ref_by.n_deps; k++) free(domain->ref_by.dep[k].type);
				free(domain->ref_by.dep);
				free(domain);
		}
		if(schema->domain != NULL) free(schema->domain);

		for(int j=0; j < schema->n_aggregates; j++){
			T_Aggregate * aggregate = schema->aggregate[j];
			
			  free(aggregate->name);
			  free(aggregate->owner);
			  free(aggregate->comment);
			  free(aggregate->base_type);
			  free(aggregate->state_type);
			  free(aggregate->state_function);
			  free(aggregate->final_type);
			  free(aggregate->final_function);
			  free(aggregate->initial_condition);
				// Dependencies
				for(int k = 0; k < aggregate->dep_on.n_deps; k++) free(aggregate->dep_on.dep[k].type);
				free(aggregate->dep_on.dep);
				for(int k = 0; k < aggregate->ref_by.n_deps; k++) free(aggregate->ref_by.dep[k].type);
				free(aggregate->ref_by.dep);	
				free(aggregate);
		}
		if(schema->aggregate != NULL) free(schema->aggregate);

		for(int j=0; j < schema->n_sequences; j++){
			T_Sequence * sequence = schema->sequence[j];
				
			  free(sequence->name);
			  free(sequence->owner);
			  free(sequence->comment);
			  free(sequence->value);
			  free(sequence->increment);
			  free(sequence->max_value);
			  free(sequence->min_value);
			  free(sequence->cache);
				// Dependencies
				for(int k = 0; k < sequence->dep_on.n_deps; k++) free(sequence->dep_on.dep[k].type);
				free(sequence->dep_on.dep);
				for(int k = 0; k < sequence->ref_by.n_deps; k++) free(sequence->ref_by.dep[k].type);
				free(sequence->ref_by.dep);
				// Permissions
			  free(sequence->acl);	
			  free(sequence);				
		}
		if(schema->sequence != NULL)free(schema->sequence);

		for(int j=0; j < schema->n_types; j++){
			T_Type * type = schema->type[j];
			
			  free(type->name);
			  free(type->owner);
			  free(type->comment);
			  free(type->input_function);
			  free(type->output_function);  
			  free(type->receive_function);  
			  free(type->send_function);
			  free(type->analyze_function);  
			  free(type->internal_length);
			  free(type->Default);  
			  free(type->element);
			  free(type->delimiter);
			  free(type->alignment);
			  free(type->storage);
				// Dependencies
				for(int k = 0; k < type->dep_on.n_deps; k++) free(type->dep_on.dep[k].type);
				free(type->dep_on.dep);
				for(int k = 0; k < type->ref_by.n_deps; k++) free(type->ref_by.dep[k].type);
				free(type->ref_by.dep);
				free(type);
		}
		if(schema->type != NULL) free(schema->type);
		
		for(int j=0; j < schema->n_operators; j++){
			T_Operator * operator = schema->operator[j];
			
			free(operator->name);
			free(operator->owner);
			free(operator->comment);
			free(operator->kind);
			free(operator->function);  
			free(operator->left_operand_type);  
			free(operator->right_operand_type);
			free(operator->commutator);  
			free(operator->negator);
			free(operator->restrict_function);  
			free(operator->join_function);
			free(operator->left_sort_operator);
			free(operator->right_sort_operator);
			free(operator->less_than_operator);
			free(operator->grater_than_operator);

			// Dependencies
			for(int k = 0; k < operator->dep_on.n_deps; k++) free(operator->dep_on.dep[k].type);
			free(operator->dep_on.dep);
			for(int k = 0; k < operator->ref_by.n_deps; k++) free(operator->ref_by.dep[k].type);
			free(operator->ref_by.dep);
			free(operator);
		}
		if(schema->operator != NULL) free(schema->operator);
		if(schema->trigger != NULL) free(schema->trigger);
		if(schema->rule != NULL) free(schema->rule);
		
	  free(schema->name);
	  free(schema->owner);
	  free(schema->comment);
		for(int j = 0; j < schema->dep_on.n_deps; j++) free(schema->dep_on.dep[j].type);
		free(schema->dep_on.dep);
		for(int j = 0; j < schema->ref_by.n_deps; j++) free(schema->ref_by.dep[j].type);
		free(schema->ref_by.dep);
	  free(schema->acl);		
	}
	if(profile->database.schema != NULL) free(profile->database.schema);
	
	for(int i=0; i < profile->database.n_languages; i++){
		T_Language * language = profile->database.language[i];
	  free(language->name);
	  free(language->comment);
	  free(language->handler_func);
	  free(language->validator_func);
		for(int j = 0; j < language->dep_on.n_deps; j++) free(language->dep_on.dep[j].type);
		free(language->dep_on.dep);
		for(int j = 0; j < language->ref_by.n_deps; j++) free(language->ref_by.dep[j].type);
		free(language->ref_by.dep);
	  free(language->acl);		
	}
	if(profile->database.language != NULL) free(profile->database.language);
	
	for(int i=0; i < profile->database.n_casts; i++){
		T_Cast * cast = profile->database.cast[i];
	  free(cast->name);
	  free(cast->comment);
	  free(cast->source_type);
	  free(cast->target_type);
	  free(cast->function);
	  free(cast->context);		
	}
	if(profile->database.cast != NULL) free(profile->database.cast);	
	
	// init counts and pointers
	profile->database.n_schemas = 0;
	profile->database.schema = NULL;
	profile->database.n_languages = 0;
	profile->database.language = NULL;
	profile->database.n_casts = 0;
	profile->database.cast = NULL;

	  free(profile->database.owner);
	  free(profile->database.comment);
	  free(profile->database.encoding);
	  free(profile->database.tablespace);
	  free(profile->database.acl);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Function for finding objects                                                                                           */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
GtkWidget*
get_widget_child_by_name(GtkWidget *widget, const gchar *widget_name){
	GtkWidget * found_widget = NULL;
	zanoreni++;
	
	if(GTK_IS_CONTAINER(widget)){
			GList * list = gtk_container_get_children(GTK_CONTAINER(widget));
			for(; list != NULL; list = list->next){
				
				if(strcmp(widget_name, gtk_widget_get_name(GTK_WIDGET(list->data))) == 0){ // found
					found_widget = GTK_WIDGET(list->data);
					zanoreni--;
					return found_widget;
				}
				else{
					
					found_widget = get_widget_child_by_name(GTK_WIDGET(list->data), widget_name);
					if(found_widget != NULL){
						zanoreni--;
						return found_widget;
					}
				}
  		}
	}
	else found_widget = NULL;
	zanoreni--;
  return found_widget;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Properties panel close callback                                                                                    */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

gboolean
on_tab_close_clicked(GtkButton  *button, gpointer         user_data)
{
	// find if page modified
	T_Page * page;
	if( (page = find_widget_in_page_list(&notebook, GTK_WIDGET(user_data))) == NULL){ // widget not found
		gtk_widget_destroy(GTK_WIDGET(user_data));
	}
	else{ // widget found
		 remove_page_from_list( &notebook, page);
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Objects dependencies                                                                                                  */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void
show_dependencies(GtkTreeView * dep_treeview, GtkTreeView * ref_treeview,T_Dependencies *dep_on,	T_Dependencies *ref_by){
	T_Profile * profile = current_profile;

	create_dependencies_tree_view (dep_treeview, "Depends On");
	create_dependencies_tree_view (ref_treeview, "Referenced By");
	GtkTreeModel * dep_model = gtk_tree_view_get_model(dep_treeview);
	GtkTreeModel * ref_model = gtk_tree_view_get_model(ref_treeview);
	T_Dependencies * dep;
	GtkTreeModel * model;
	GtkTreeView * treeview;
	
	for(int k = 0; k < 2; k++){	
		GtkTreeIter it1, it2, it3, it4,it5;
		// deps
		if(k == 0){
			dep = dep_on;		
			model = dep_model;
			treeview = dep_treeview;
		}
    // refs
		else{
			dep = ref_by;
			model = ref_model;
			treeview = ref_treeview;
		}
		
		gboolean show_schemas=FALSE;		
		for(int i=0; i < profile->database.n_schemas; i++){
			it1 = add_depend( GTK_TREE_STORE(model), NULL, get_icon(PIXMAP_DIRECTORY "/Schema.png"), profile->database.schema[i]->name, "");
			for(int d=0; d < dep->n_deps; d++){
				if(dep->dep[d].OID == profile->database.schema[i]->OID){
					gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it1, 2, text, -1);
					free(text);
					show_schemas=TRUE;
			  }
			}
			// tables
			gint n_tabs=0;
			gboolean show_tabs=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Tables.png"), "Tables", "");
			for(int j=0; j < profile->database.schema[i]->n_tables; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Table.png"), profile->database.schema[i]->table[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->table[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_tabs = TRUE;
			  	}
				}
								
			  // Indexes
			  gint n_ind = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Indexes.png"), "Indexes", "");
			  for(int l=0; l < profile->database.schema[i]->table[j]->n_indexes; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->table[j]->index[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Index.png"), profile->database.schema[i]->table[j]->index[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_ind++;
			  		}
			  	}
				}
				if(n_ind == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Indexes", n_ind);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_tabs = TRUE;
				}
					
				// FKeys
			  gint n_fkeys = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/ForeignKeys.png"), "Foreign Keys", "");
			  for(int l=0; l < profile->database.schema[i]->table[j]->n_fkeys; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->table[j]->fkey[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/ForeignKey.png"), profile->database.schema[i]->table[j]->fkey[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_fkeys++;
			  		}
			  	}
				}
				if(n_fkeys == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Foreign Keys", n_fkeys);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_tabs = TRUE;
				}
					
				// Checks
			  gint n_checks = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Checks.png"), "Checks", "");
			  for(int l=0; l < profile->database.schema[i]->table[j]->n_checks; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->table[j]->check[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Check.png"), profile->database.schema[i]->table[j]->check[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_checks++;
			  		}
			  	}
				}
				if(n_checks == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Checks", n_checks);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_tabs = TRUE;
				}
					
				// Triggers
			  gint n_triggs = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Triggers.png"), "Triggers", "");
			  for(int l=0; l < profile->database.schema[i]->table[j]->n_triggers; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->table[j]->trigger[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Trigger.png"), profile->database.schema[i]->table[j]->trigger[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_triggs++;
			  		}
			  	}
				}
				if(n_triggs == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Triggers", n_triggs);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_tabs = TRUE;
				}
					
				// Rules
			  gint n_rules = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Rules.png"), "Rules", "");
			  for(int l=0; l < profile->database.schema[i]->table[j]->n_rules; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->table[j]->rule[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Rule.png"), profile->database.schema[i]->table[j]->rule[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_rules++;
			  		}
			  	}
				}
				if(n_rules == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Rules", n_rules);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_tabs = TRUE;
				}
				
				if(show_tabs) n_tabs++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_tabs = FALSE;
			}
			if(n_tabs == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Tables", n_tabs);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			
			// views
			gint n_views=0;
			gboolean show_views=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Views.png"), "View", "");
			for(int j=0; j < profile->database.schema[i]->n_views; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/View.png"), profile->database.schema[i]->view[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->view[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_views = TRUE;
			  	}
			  }
			 			  
			  // Rules
			  gint n_rules = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Rules.png"), "Rules", "");
			  for(int l=0; l < profile->database.schema[i]->view[j]->n_rules; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->view[j]->rule[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Rule.png"), profile->database.schema[i]->view[j]->rule[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_rules++;
			  		}
			  	}
				}
				if(n_rules == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Rules", n_rules);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_views = TRUE;
				}
				
				if(show_views) n_views++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_views = FALSE;
			}
			if(n_views == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Views", n_views);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			
			// functions
			gint n_funcs=0;
			gboolean show_funcs=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Functions.png"), "Functions", "");
			for(int j=0; j < profile->database.schema[i]->n_functions; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Function.png"), profile->database.schema[i]->function[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->function[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_funcs = TRUE;
			  	}
				}
				if(show_funcs) n_funcs++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_funcs = FALSE;
			}
			if(n_funcs == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Functions", n_funcs);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			// domains
			gint n_doms=0;
			gboolean show_doms=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Domains.png"), "Domains", "");
			for(int j=0; j < profile->database.schema[i]->n_domains; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Domain.png"), profile->database.schema[i]->domain[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->domain[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_doms = TRUE;
			  	}
				}
				// Checks
			  gint n_checks = 0;
			  it4 = add_depend( GTK_TREE_STORE(model), &it3, get_icon(PIXMAP_DIRECTORY "/Checks.png"), "Checks", "");
			  for(int l=0; l < profile->database.schema[i]->domain[j]->n_checks; l++){
			  	for(int d=0; d < dep->n_deps; d++){
						if(dep->dep[d].OID == profile->database.schema[i]->domain[j]->check[l]->OID){
			  			gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
							it5 = add_depend( GTK_TREE_STORE(model), &it4, get_icon(PIXMAP_DIRECTORY "/Check.png"), profile->database.schema[i]->domain[j]->check[l]->name, "");
							gtk_tree_store_set(  GTK_TREE_STORE(model), &it5, 2, text, -1);
							free(text);
							n_checks++;
			  		}
			  	}
				}
				if(n_checks == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it4);
				else{
					gchar * text = bold_text_with_count("Checks", n_checks);
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it4, 1, text, -1);
					free(text);
					show_doms = TRUE;
				}
				
				if(show_doms) n_doms++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_doms = FALSE;
			}
			if(n_doms == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Domains", n_doms);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			// aggregates
			gint n_aggs=0;
			gboolean show_aggs=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Aggregates.png"), "Aggregates", "");
			for(int j=0; j < profile->database.schema[i]->n_aggregates; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Aggregate.png"), profile->database.schema[i]->aggregate[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->aggregate[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_aggs = TRUE;
			  	}
				}
				if(show_aggs) n_aggs++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_aggs = FALSE;
			}
			if(n_aggs == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Aggregates", n_aggs);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			// sequences
			gint n_seqs=0;
			gboolean show_seqs=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Sequences.png"), "Sequences", "");
			for(int j=0; j < profile->database.schema[i]->n_sequences; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Sequence.png"), profile->database.schema[i]->sequence[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->sequence[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_seqs = TRUE;
			  	}
				}
				if(show_seqs) n_seqs++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_seqs = FALSE;
			}
			if(n_seqs == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Sequences", n_seqs);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			// types
			gint n_types=0;
			gboolean show_types=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Types.png"), "Types", "");
			for(int j=0; j < profile->database.schema[i]->n_types; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Type.png"), profile->database.schema[i]->type[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->type[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_types = TRUE;
			  	}
				}
				if(show_types) n_types++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_types = FALSE;
			}
			if(n_types == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Types", n_types);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			// operators
			gint n_opers=0;
			gboolean show_opers=FALSE;
			it2 = add_depend( GTK_TREE_STORE(model), &it1, get_icon(PIXMAP_DIRECTORY "/Operators.png"), "Operators", "");
			for(int j=0; j < profile->database.schema[i]->n_operators; j++){
				it3 = add_depend( GTK_TREE_STORE(model), &it2, get_icon(PIXMAP_DIRECTORY "/Operator.png"), profile->database.schema[i]->operator[j]->name, "");
				for(int d=0; d < dep->n_deps; d++){
					if(dep->dep[d].OID == profile->database.schema[i]->operator[j]->OID){
			  		gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
						gtk_tree_store_set(  GTK_TREE_STORE(model), &it3, 2, text, -1);
						free(text);
						show_opers = TRUE;
			  	}
				}
				if(show_opers) n_opers++;
				else gtk_tree_store_remove(GTK_TREE_STORE(model), &it3);
				show_opers = FALSE;
			}
			if(n_opers == 0) gtk_tree_store_remove(GTK_TREE_STORE(model), &it2);
			else{
				gchar * text = bold_text_with_count("Operators", n_opers);
				gtk_tree_store_set(  GTK_TREE_STORE(model), &it2, 1, text, -1);
				free(text);
				show_schemas=TRUE;
			}
			
		}
		if(!show_schemas){
  		if(it1.user_data != NULL) gtk_tree_store_remove(GTK_TREE_STORE(model), &it1);
		}
		
		gboolean show_langs=FALSE;		
		for(int i=0; i < profile->database.n_languages; i++){
			it1 = add_depend( GTK_TREE_STORE(model), NULL, get_icon(PIXMAP_DIRECTORY "/Language.png"), profile->database.language[i]->name, "");
			for(int d=0; d < dep->n_deps; d++){
				if(dep->dep[d].OID == profile->database.language[i]->OID){
					gchar * text = normal_text_with_color(dep->dep[d].type, "#980F0F");
					gtk_tree_store_set(  GTK_TREE_STORE(model), &it1, 2, text, -1);
					free(text);
					show_langs=TRUE;
			  }
			}
		}
		if(!show_langs){
			if(it1.user_data != NULL) gtk_tree_store_remove(GTK_TREE_STORE(model), &it1);
		}
			
		gtk_tree_view_expand_all(treeview);
	}
}

void
create_dependencies_tree_view (GtkTreeView * view, gchar * title){
    GtkTreeModel      *model;
    GtkTreeViewColumn *col;
    GtkCellRenderer   *renderer;
 
    model = GTK_TREE_MODEL(gtk_tree_store_new(3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING));
    
    gtk_tree_view_set_model(view, model);
				
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, title);
    
    renderer = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_start(col, renderer, FALSE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "pixbuf", 0,
                                        NULL);
                                        
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "markup", 1,
                                        NULL);
                                            
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
    
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, "Type");                                       
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, FALSE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "markup", 2,
                                        NULL);
                                            
    gtk_tree_view_append_column(view, col);
    gtk_widget_show_all(GTK_WIDGET(view));
		gtk_tree_view_set_enable_tree_lines(view, TRUE);
}

GtkTreeIter
add_depend(GtkTreeStore * store, GtkTreeIter * iter, GdkPixbuf *icon, gchar * object, gchar * type){
		GtkTreeIter child;
		gchar * text1;
		gchar * text2;
 		gtk_tree_store_append(store, &child, iter);
 		
 		text1 = normal_text(object);
 		text2 = normal_text_with_color(type, "#980F0F");
 		
		gtk_tree_store_set(store, &child, 0, icon, 1, text1, 2, text2, -1);
		free(text1); free(text2);
		return child;
}



