/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "main_window_callbacks.h"
#include "GUI_functions.h"
#include "main_window.h"

/*
 * Connects callbacks for main window
 */
void
connect_main_window_signals(GladeXML * source)
{
	GtkWidget * widget;

	// When closing the window
	widget = glade_xml_get_widget(source, "main_window");
	g_signal_connect ((gpointer) widget, "delete-event",
                    G_CALLBACK (on_main_window_destroy),
                    NULL);
   
   widget = glade_xml_get_widget(source, "main_notebook");
	 g_signal_connect ((gpointer) widget, "switch-page",
                    G_CALLBACK (on_main_notebook_change_page),
                    NULL);
   
} // connect_main_window_signals ()

/*
 * Confirm closing aplication
 */
gboolean
on_main_window_destroy (GtkWindow *object, GdkEvent * event, gpointer user_data)
{
	// run dialog to confirm delete
	gchar * l = normal_text("Do you really want to exit from Postgre SQL Manager?");
	gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
	free(l);
	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
	if(res == GTK_RESPONSE_YES){
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
		if(!gui_disconnect_all_dbs()) return TRUE;
		gtk_main_quit();
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
		return TRUE;
	}
	return FALSE;
} // on_main_window_destroy ()

/*
 * Main notebook page changed
 */
void
on_main_notebook_change_page (GtkNotebook *nb, GtkNotebookPage *page, guint page_num, gpointer user_data){
	GtkWidget * widget = gtk_notebook_get_nth_page(nb, page_num);
	T_Page * p = find_widget_in_page_list(&notebook, widget);
	if(p != NULL){
		gchar * text = normal_text((p->modified)? "Modified": "");
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "statusbar_label2")), text);
		free(text);
	}
	else{
		gchar * text = normal_text("");
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "statusbar_label2")), text);
		free(text);
	}
} // on_main_notebook_change_page ()
                                                        
