/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "toolbar_callbacks.h"
#include "GUI_functions.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/drop_db_objects.h"
#include "../GUI_wizards/db_profile_wizard.h"
#include "../GUI_wizards/db_wizard.h"
#include "../otree/show_otree.h"
#include "../tools/db_profile_manager.h"
#include "../tools/sql_editor.h"

/*
 * Connects toolbar callbacks
 */
void
connect_toolbar_signals (GladeXML * source)
{
	GtkWidget * widget;
	
	// Refresh button
	widget = glade_xml_get_widget(source, "tb_refresh");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_refresh_clicked),
                    NULL);
                    
  // Add database profiles
  widget = glade_xml_get_widget(source, "tb_create_db_profile");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_add_db_profiles_clicked),
                    NULL);
                    
  // Edit database profile
  widget = glade_xml_get_widget(source, "tb_edit_db_profile");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_edit_db_profile_clicked),
                    NULL);

  // Remove database profile 
  widget = glade_xml_get_widget(source, "tb_remove_db_profile");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_remove_db_profile_clicked),
                    NULL);
	
	// Database profile manager
  widget = glade_xml_get_widget(source, "tb_db_profile_man");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_db_profile_manager_clicked),
                    NULL);

	// Connect to database
  widget = glade_xml_get_widget(source, "tb_connect_to_db");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_connect_to_db_clicked),
                    NULL);

	// Disconnect from database
  widget = glade_xml_get_widget(source, "tb_disconnect_from_db");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_disconnect_from_db_clicked),
                    NULL);
                    
                                                                                            
  // Create new database
  widget = glade_xml_get_widget(source, "tb_create_db");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_create_db_clicked),
                    NULL);
                    
  // Execute SQL
  widget = glade_xml_get_widget(source, "tb_sql_editor");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_execute_sql_clicked),
                    NULL);
                    
  // Help
  widget = glade_xml_get_widget(source, "tb_help");
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_toolbar_help_clicked),
                    NULL);
}

void
on_toolbar_refresh_clicked(GtkButton  *button, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_refresh(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
}

void
on_toolbar_add_db_profiles_clicked(GtkButton  *button, gpointer user_data)
{
	run_c_db_profile_wizard();
}

void
on_toolbar_edit_db_profile_clicked(GtkButton  *button, gpointer user_data)
{
	run_db_profile_manager();
}

void
on_toolbar_remove_db_profile_clicked(GtkButton  *button, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_remove_profile(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path);
  gtk_tree_selection_select_path(selection, path);
}

void
on_toolbar_db_profile_manager_clicked(GtkButton  *button, gpointer user_data)
{
	run_db_profile_manager();
}

void
on_toolbar_connect_to_db_clicked(GtkButton  *button, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_connect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path);
}

void
on_toolbar_disconnect_from_db_clicked(GtkButton  *button, gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_disconnect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
}


void
on_toolbar_create_db_clicked(GtkButton  *button, gpointer user_data)
{
	run_c_db_wizard();
}

void
on_toolbar_execute_sql_clicked(GtkButton  *button, gpointer user_data)
{
	run_sql_editor();
}

void
on_toolbar_help_clicked(GtkButton  *button, gpointer user_data)
{
	gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "help_dialog")));	
	gtk_widget_hide(glade_xml_get_widget(xml, "help_dialog"));
}
