/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "other_objects.h"
#include "../functions.h"
#include "../database/create_db_objects.h"
#include "../database/edit_db_objects.h"
#include "../database/get_db_objects.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/properties_widget.h"
#include "../GUI/textview_functions.h"
#include "../GUI/main_window.h"
#include "../otree/show_otree.h"

/*
 * Inits object properties dialog
 */
void
init_op_dialog(void){
	// Color settings
	GdkColor color;
	color.red = 148*257;
  color.green = 23*257;
  color.blue = 23*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "opd_img_bg"), GTK_STATE_NORMAL, &color);
  
	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"opd_properties_hr"), GTK_STATE_NORMAL, &color);
	
	color.red = 236*257;
  color.green = 232*257;
  color.blue = 226*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "opd_properties_columns_bg"), GTK_STATE_NORMAL, &color);
  
  color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "opd_properties_bg"), GTK_STATE_NORMAL, &color);
  
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "opd_properties")), "properties_column1");
	if(wid != NULL) gtk_widget_destroy(wid);
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "opd_properties")), "properties_column2");
	if(wid != NULL) gtk_widget_destroy(wid);
	
} // init_op_dialog()

/////////////////////////////////////////////// Field dialog /////////////////////////////////////////////////////////////////////////

/*
 * Function that runs field dialog
 */
void
run_field_dialog(T_Profile * profile, T_Schema * schema, T_Table * table, T_Column * column, gboolean new){
	init_op_dialog();
	
	GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
	// set title
	gtk_window_set_title(GTK_WINDOW(dialog), "Field properties");
	
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "opd_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);

  gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	if(!new){
		if(strcmp(column->name, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	
	add_entry_property("property_name", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Name",  (new) ? "" : column->name);
	height += 21;
	GtkWidget * wid = get_widget_child_by_name(dialog, "property_name");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_field_entries_changed),
                    wid);  
	}	
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", (new) ? "" : column->comment);
	height += 41;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Field",  PIXMAP_DIRECTORY "/Field.png");
	height += 21;
	T_Data_types * dtypes = get_data_types(profile->conn);
	if(dtypes != NULL){
		gint id=0;
		if(!new){
		 for(int i = 0; i < dtypes->n_data_types; i++){
			 if(strcmp(dtypes->name[i], column->type) == 0){
				 id = i;
				 break;
			 }
		 }
		}
		add_combo_property("property_type", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Type", dtypes->name, dtypes->n_data_types, id);
		for(int i = 0; i < dtypes->n_data_types; i++){
			free(dtypes->name[i]);
		}
		free(dtypes->name);
	  free(dtypes->varlen);
	  free(dtypes);
	}
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_type");
	if(wid != NULL){
		g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_field_type_changed),
                    dtypes); 
   }
	add_spin_property("property_dimensions", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Array dimensions", (new) ? 0 : column->dimensions, 1, 1000);
	height += 21;
	add_spin_property("property_size", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Size", (new) ? -1 : atoi(column->size), -1, 1000);
	height += 21;
	add_noneditable_property(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Precision", (new) ? "" : column->precision);
	height += 21;
	wid = get_widget_child_by_name(glade_xml_get_widget(xml, "object_properties_dialog"), "property_type");
	if(wid != NULL){
		gint i = gtk_combo_box_get_active(GTK_COMBO_BOX(wid));
		if(i > 0) gtk_combo_box_set_active(GTK_COMBO_BOX(wid), i-1);
		else gtk_combo_box_set_active(GTK_COMBO_BOX(wid), i+1);
		gtk_combo_box_set_active(GTK_COMBO_BOX(wid), i);
	}
	
	add_toggle_property("property_pkey", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Primary Key",  (new) ? FALSE : column->pkey, TRUE);
	height += 21;
	add_toggle_property("property_unique", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Unique",  (new) ? FALSE : column->unique, TRUE);
	height += 21;
	add_toggle_property("property_not_null", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Not Null",  (new) ? FALSE : column->not_null, TRUE);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_pkey");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "toggled",
                    G_CALLBACK (on_field_pkey_toggled),
                    wid);  
    gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
    if(active){
    	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), !active);
    	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), active);    
    }
	}
	
	
	add_entry_property("property_default", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Default Value",  (new) ? "" : column->default_value);
	height += 21;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "scrolledwindow11"), -1, height);

  // run dialog
  
  gboolean end = FALSE;
  while(!end){
  	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "object_properties_dialog")));
  	if(res == GTK_RESPONSE_OK){
			// get properties
			T_Column * new_field = NULL;
			if(( new_field = (T_Column *) malloc (sizeof(T_Column)) ) == NULL){	
				fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
			}
			// name
			wid = get_widget_child_by_name(dialog, "property_name");
			if(wid != NULL) new_field->name = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_field->name = copy_string("");
			
			// comment
			wid = get_widget_child_by_name(dialog, "property_comment");
			if(wid != NULL){
				GtkTextBuffer * buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
				GtkTextIter start, end;
  			gtk_text_buffer_get_start_iter(buffer, &start);
  			gtk_text_buffer_get_end_iter(buffer, &end);
  			new_field->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
  		}
  		else new_field->comment = copy_string("");
  		
  		// type
  		wid = get_widget_child_by_name(dialog, "property_type");
			if(wid != NULL){
				 new_field->type = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			
				T_Data_types * dtypes = get_data_types(profile->conn);
				if(dtypes != NULL){
					for(int i = 0; i < dtypes->n_data_types; i++){
						 if(strcmp(dtypes->name[i], new_field->type) == 0){
							 if(dtypes->varlen[i]) new_field->resizeable = TRUE;
							 break;
						 }
		 			}
				  for(int i = 0; i < dtypes->n_data_types; i++) free(dtypes->name[i]);
					free(dtypes->name);
					free(dtypes->varlen);
  				free(dtypes);
				}
			}
			else{
			 new_field->type = copy_string("");
			 new_field->resizeable = FALSE;
			}
			
  		// dimensions
  		wid = get_widget_child_by_name(dialog, "property_dimensions");
			if(wid != NULL) new_field->dimensions = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wid));
			else new_field->dimensions = 0;
			
  		// size
  		wid = get_widget_child_by_name(dialog, "property_size");
			if(wid != NULL) new_field->size = itoa_n( gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wid)), 0);
			else new_field->size = copy_string("");
			
   		// primary key
  		wid = get_widget_child_by_name(dialog, "property_pkey");
			if(wid != NULL) new_field->pkey = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_field->pkey = FALSE;
			
  		// unique 
  		wid = get_widget_child_by_name(dialog, "property_unique");
			if(wid != NULL) new_field->unique = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_field->unique = FALSE;
  		
  		// not null
  		wid = get_widget_child_by_name(dialog, "property_not_null");
			if(wid != NULL) new_field->not_null = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_field->not_null = FALSE;
			
  		// deafault value
  		wid = get_widget_child_by_name(dialog, "property_default");
			if(wid != NULL) new_field->default_value = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_field->default_value = copy_string("");
			
  		// make sql code
			gchar *sql = NULL;
			gchar * parent = append_string(NULL, schema->name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, table->name);
			if(new){
				sql =  create_new_field_SQL_query(new_field, parent, FALSE);
			}
			else{
				sql =  create_edit_field_SQL_query(table, column, new_field, parent);
			}
			//execute code
			if(sql != NULL){
				if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
				 end = TRUE;
					// get server
					gboolean e = FALSE;
					gint i_server = 0;
					for(int k = 0; k < o_tree.n_servers; k++){
						for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
							if(o_tree.server[k]->profile[l] == profile){
								i_server = k;
								e = TRUE;
								break;
							}
						}
						if(e) break;
					}
					gui_refresh( o_tree.server[i_server], profile);
				}
				free(sql);
			}
			else end = TRUE;
			
  		// free field structure
  		free(new_field->name);
  		free(new_field->comment);
  		free(new_field->type);
  		free(new_field->size);
  		free(new_field->default_value);
  		free(new_field);
	  	new_field = NULL;
	  	
  		free(parent);
  	}
  	else{
  		end = TRUE;
  	}  	
  }
  
 	gtk_widget_hide(glade_xml_get_widget(xml, "object_properties_dialog"));
}

void
on_field_entries_changed (GtkEditable *editable, gpointer user_data){
	GtkEntry * entry = GTK_ENTRY(user_data);
	gchar * text = copy_string(gtk_entry_get_text(entry));
	if(strcmp(text, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else{
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_name");
		if(wid1 != NULL){
			if(strcmp(gtk_entry_get_text(GTK_ENTRY(wid1)), "") != 0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), TRUE);
		}
	}
}

void
on_field_type_changed (GtkComboBox *combobox, gpointer user_data){
	gint i = gtk_combo_box_get_active(combobox);
	T_Data_types * t = (T_Data_types *) user_data;
	
	GtkWidget * wid = get_widget_child_by_name(glade_xml_get_widget(xml, "object_properties_dialog"), "property_size");
	if(wid != NULL){
		if(t->varlen[i]) gtk_widget_set_sensitive(wid,TRUE);	
	  else gtk_widget_set_sensitive(wid, FALSE);	
	  
	}
	
	gchar * type = NULL;
	type = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(combobox)->child)));
	gint len = strlen(type);
	if(len>2){
		GtkWidget * wid = get_widget_child_by_name(glade_xml_get_widget(xml, "object_properties_dialog"), "property_dimensions");
		if(wid != NULL){
			if(type[len-2] == '[' && type[len-1] == ']') gtk_widget_set_sensitive(wid,TRUE);	
	  	else gtk_widget_set_sensitive(wid, FALSE);	
	  }
	}
	free(type);	
	
}

void
on_field_pkey_toggled (GtkToggleButton *togglebutton, gpointer user_data){
	GtkWidget * wid = GTK_WIDGET(user_data);
	if(wid != NULL){
		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid))){
			GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
			GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_unique");
			if(wid1 != NULL){
			  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), TRUE);
			  gtk_widget_set_sensitive( wid1, FALSE);
			}
			
			wid1 = get_widget_child_by_name(dialog, "property_not_null");
			if(wid1 != NULL){
				 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), TRUE);
				gtk_widget_set_sensitive( wid1, FALSE);
			}
		}
		else{
			GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
			GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_unique");
			if(wid1 != NULL) gtk_widget_set_sensitive( wid1, TRUE);
			wid1 = get_widget_child_by_name(dialog, "property_not_null");
			if(wid1 != NULL) gtk_widget_set_sensitive( wid1, TRUE);
		}
	}
}

/////////////////////////////////////////////// Index dialog ////////////////////////////////////////////////////////////////////////

/*
 * Function that runs index dialog
 */
void
run_index_dialog(T_Profile * profile, T_Schema * schema, T_Table * table, T_Index * index, gboolean new){
	init_op_dialog();

	GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
	// set title
	gtk_window_set_title(GTK_WINDOW(dialog), "Index properties");
	
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "opd_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);

  gint height = 0;
	// header property Common
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;

	// name
	if(!new){
		if(strcmp(index->name, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	
	add_entry_property("property_name", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Name", (new) ? "" :  index->name);
	height += 21;
	GtkWidget * wid = get_widget_child_by_name(dialog, "property_name");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_index_entries_changed),
                    wid);  
	}	

	// comment
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment",(new) ? "" :  index->comment);
	height += 41;
	if(!new){
	gchar * text = itoa_n(index->OID, 0);
	add_noneditable_property(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "OID", text);
	free(text);
	height += 21;
	}

	// header property Index
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Index",  PIXMAP_DIRECTORY "/Index.png");
	height += 21;
	
	// columns
	gint i_schema = 0;
	gint i_table = 0;
	// find table and schema
	for(int j = 0; j < profile->database.n_schemas; j++) if(profile->database.schema[j] == schema) i_schema = j;
  for(int k = 0; k < schema->n_tables; k++) if(schema->table[k] == table) i_table = k;
	
	gchar * cols = NULL;
	cols = append_string(cols, "");
	if(!new){	
		for(int i = 0; i < index->n_columns; i++){
			cols = append_string(cols, index->column[i]->name);
			if(i < index->n_columns-1){
				cols = append_string(cols, ", ");
			}
		}
	}

	if(strcmp(cols, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_column_property("property_columns", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Columns", cols, profile, i_schema, i_table, (new) ? NULL : index->column, (new) ? 0 : index->n_columns);
	free(cols);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_columns");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_index_entries_changed),
                    wid);  
	}

	// primary key constriant
	add_toggle_property("property_pkey_constraint", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Primary Key Constraint", (new) ? FALSE : index->pkey_constraint, TRUE);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_pkey_constraint");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "toggled",
                    G_CALLBACK (on_index_pkeycon_toggled),
                    wid);  
	}

	// unique constraint
	add_toggle_property("property_unique_constraint", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Unique Constraint",  (new) ? FALSE : index->unique_constraint, TRUE);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_unique_constraint");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "toggled",
                    G_CALLBACK (on_index_uniqcon_toggled),
                    wid);  
	}

	// unique index
	add_toggle_property("property_unique_index", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Unique Index",  (new) ? FALSE : index->unique_index, TRUE);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_unique_index");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "toggled",
                    G_CALLBACK (on_index_uniqind_toggled),
                    wid);  
	}

	// clustered
	add_toggle_property("property_clustered", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Clustered",  (new) ? FALSE : index->clustered, TRUE);
	height += 21;

	// predicate
	add_entry_property("property_predicate", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Predicate", (new) ? "" :  index->predicate);
	height += 21;

	// tablespace
	T_Tablespaces * tspces = get_Tablespaces(profile);
	if(tspces != NULL){
		gint tspc_id=-1;
		if(!new){
			for(int i = 0; i < tspces->n_tablespaces; i++){
				if(strcmp(tspces->name[i], index->tablespace) == 0){
					tspc_id = i;
					break;
				}
			}
		}
		add_combo_property("property_tablespace", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Tablespace", tspces->name,  tspces->n_tablespaces, tspc_id);
		height += 21;
		for(int i = 0; i < tspces->n_tablespaces; i++){
			free(tspces->name[i]);
		}
		free(tspces->name);
		free(tspces);
	}

	// acces method
	gchar * access_methods[] = {"","btree","hash","rtree","gist"};
	gint am_id=-1;
	if(!new){
		for(int i = 0; i < 5; i++){
			if(strcmp(access_methods[i], index->access_method) == 0){
				am_id = i;
				break;
			}
		}
	}
	add_combo_property("property_access_method", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Access Method", access_methods,  5, am_id);
	height += 21;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "scrolledwindow11"), -1, height);

  // run dialog
  
  gboolean end = FALSE;
  while(!end){
  	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "object_properties_dialog")));
  	if(res == GTK_RESPONSE_OK){
			// get properties
			T_Index * new_index = NULL;
			if(( new_index = (T_Index *) malloc (sizeof(T_Index)) ) == NULL){	
				fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
			}

			// name
			wid = get_widget_child_by_name(dialog, "property_name");
			if(wid != NULL) new_index->name = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_index->name = copy_string("");

			// comment
			wid = get_widget_child_by_name(dialog, "property_comment");
			if(wid != NULL){
				GtkTextBuffer * buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
				GtkTextIter start, end;
  			gtk_text_buffer_get_start_iter(buffer, &start);
  			gtk_text_buffer_get_end_iter(buffer, &end);
  			new_index->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
  		}
  		else new_index->comment = copy_string("");

  		// columns
  		new_index->n_columns = 0;
  		new_index->column = NULL;
  		wid = get_widget_child_by_name(dialog, "property_columns");
			if(wid != NULL){
				gchar * columns = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
				columns = append_string(columns, ",");
			  gint s=0, e = 0;
			  gint len = strlen(columns);
			  
			  for(; e < len; e++){
			  	if(columns[e] == ','){
			  		columns[e] = '\0';
			  		gchar * col = copy_string(columns+s);
			  		gint m = new_index->n_columns;
			  		// add column	
			  		if(new_index->column == NULL){
							if((new_index->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
						}
						else{
		 					if((new_index->column = (T_Column **) realloc((new_index->column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
						}
						if((new_index->column[m] = (T_Column *) malloc (sizeof(T_Column))) == NULL){
							fprintf(stderr, "Memory allocation error.");
	  					exit(0);
						}
						
						new_index->column[m]->name = col;
						new_index->n_columns++;
								  	
			  		columns[e] = ',';
			  		s = e+2;
			  		e+=2;
			  	}
			  }
			  free(columns);	
  		}

  		// primary key constriant
  		wid = get_widget_child_by_name(dialog, "property_pkey_constraint");
			if(wid != NULL) new_index->pkey_constraint = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_index->pkey_constraint = FALSE;

  		// unique constriant
  		wid = get_widget_child_by_name(dialog, "property_unique_constraint");
			if(wid != NULL) new_index->unique_constraint = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_index->unique_constraint = FALSE;

  		// unique index
  		wid = get_widget_child_by_name(dialog, "property_unique_index");
			if(wid != NULL) new_index->unique_index = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_index->unique_index = FALSE;

  		// clustered
  		wid = get_widget_child_by_name(dialog, "property_clustered");
			if(wid != NULL) new_index->clustered = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_index->clustered = FALSE;

  		// predicate
  		wid = get_widget_child_by_name(dialog, "property_predicate");
			if(wid != NULL) new_index->predicate = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_index->predicate = copy_string("");

  		// tablespace
  		wid = get_widget_child_by_name(dialog, "property_tablespace");
			if(wid != NULL) new_index->tablespace = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_index->tablespace = copy_string("");

  		// access method
  		wid = get_widget_child_by_name(dialog, "property_access_method");
			if(wid != NULL) new_index->access_method = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_index->access_method = copy_string("");
			
			// make sql code
			gchar *sql = NULL;
			gchar * parent = append_string(NULL, schema->name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, table->name);
			if(new){
				sql =  create_new_index_SQL_query(new_index, parent, FALSE);
			}
			else{
				sql = create_edit_index_SQL_query(index, new_index, parent);
			}
			//execute code
			if(sql != NULL){
				if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
				 end = TRUE;
					// get server
					gboolean e = FALSE;
					gint i_server = 0;
					for(int k = 0; k < o_tree.n_servers; k++){
						for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
							if(o_tree.server[k]->profile[l] == profile){
								i_server = k;
								e = TRUE;
								break;
							}
						}
						if(e) break;
					}
					gui_refresh( o_tree.server[i_server], profile);
				}
				free(sql);
			}
			else end = TRUE;
			
			// free index structure
  		free(new_index->name);
  		free(new_index->comment);
  		for(int i = 0; i < new_index->n_columns; i++){
  			free(new_index->column[i]->name);
  			free(new_index->column[i]);
  		}
  		free(new_index->column);
  		free(new_index->predicate);
  		free(new_index->tablespace);
  		free(new_index->access_method);
  		free(new_index);
	  	new_index = NULL;
	  	
  		free(parent);
  	}
  	else{
  		end = TRUE;
  	}
  }
  
  gtk_widget_hide(glade_xml_get_widget(xml, "object_properties_dialog"));
  
}

void
on_index_entries_changed (GtkEditable *editable, gpointer user_data){
	GtkEntry * entry = GTK_ENTRY(user_data);
	gchar * text = copy_string(gtk_entry_get_text(entry));
	if(strcmp(text, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else{
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_name");
		GtkWidget * wid2 = get_widget_child_by_name(dialog, "property_columns");
		if(wid1 != NULL && wid2 != NULL){
			if(strcmp(gtk_entry_get_text(GTK_ENTRY(wid1)), "") != 0 && strcmp(gtk_entry_get_text(GTK_ENTRY(wid2)), "") != 0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), TRUE);
		}
		
	}
}

void
on_index_pkeycon_toggled (GtkToggleButton *togglebutton, gpointer user_data){
	GtkWidget * wid = GTK_WIDGET(user_data);
	if(wid != NULL){
		gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_unique_constraint");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		wid1 = get_widget_child_by_name(dialog, "property_unique_index");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), active);
	}
}

void
on_index_uniqcon_toggled (GtkToggleButton *togglebutton, gpointer user_data){
	GtkWidget * wid = GTK_WIDGET(user_data);
	if(wid != NULL){
		gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_pkey_constraint");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		wid1 = get_widget_child_by_name(dialog, "property_unique_index");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), active);
	}
}

void
on_index_uniqind_toggled (GtkToggleButton *togglebutton, gpointer user_data){
	GtkWidget * wid = GTK_WIDGET(user_data);
	if(wid != NULL){
		gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_pkey_constraint");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		wid1 = get_widget_child_by_name(dialog, "property_unique_constraint");
		if(wid1 != NULL) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), active);
	}
}

/////////////////////////////////////////////// Foreign key dialog //////////////////////////////////////////////////////////////

/*
 * Function that runs foreign key dialog
 */
void
run_fkey_dialog(T_Profile * profile, T_Schema * schema, T_Table * table, T_FKey * fkey, gboolean new){
	init_op_dialog();
	
	GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
	// set title
	gtk_window_set_title(GTK_WINDOW(dialog), "Foreign key properties");
	
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "opd_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);
	
	// header Common
  gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;

	// property name
	if(!new){ if(strcmp(fkey->name, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);}
	else gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_entry_property("property_name", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Name", (new) ? "" : fkey->name);
	height += 21;
	GtkWidget * wid = get_widget_child_by_name(dialog, "property_name");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_fkey_columns_changed),
                    wid);  
	}

	// property comment
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", (new) ? "" : fkey->comment);
	height += 41;
	
	// property OID
	if(!new){
		gchar * text = itoa_n(fkey->OID, 0);
		add_noneditable_property(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "OID", text);
		free(text);
		height += 21;
	}
	
	// header property Foreign Key
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Foreign Key",  PIXMAP_DIRECTORY "/ForeignKey.png");
	height += 21;
	
	// property columns
	gint i_schema = 0;
	gint i_table = 0;
	// find table and schema
	for(int j = 0; j < profile->database.n_schemas; j++) if(profile->database.schema[j] == schema) i_schema = j;
  for(int k = 0; k < schema->n_tables; k++) if(schema->table[k] == table) i_table = k;
		
	gchar * cols = NULL;
	cols = append_string(cols, "");
	
	if(!new){
		for(int i = 0; i < fkey->n_columns; i++){
			cols = append_string(cols, fkey->column[i]->name);
			if(i < fkey->n_columns-1){
				cols = append_string(cols, ", ");
			}
		}
	}
	if(strcmp(cols, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_column_property("property_columns", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Columns", cols, profile, i_schema, i_table, (new) ? NULL : fkey->column, (new) ? 0 : fkey->n_columns);
	free(cols);
	height += 21;
  wid = get_widget_child_by_name(dialog, "property_columns");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_fkey_columns_changed),
                    wid);  
	}

	// property foreign tables
	gint count = 0;
	gint id= 0;
	for(int i = 0; i < profile->database.n_schemas; i++){
		count += profile->database.schema[i]->n_tables; 
	}
	gchar * list1[count];
	gint index= 0;
	for(int i = 0; i < profile->database.n_schemas; i++){
		for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
			gchar * table = NULL;
			
			if(!new){
				if(strcmp(profile->database.schema[i]->table[j]->name, fkey->ref_table) == 0 && strcmp(profile->database.schema[i]->name, fkey->ref_schema) == 0){
				  id = index;
				  i_schema = i;
				  i_table = j;
				}
			}
			if(new && index == 0){
				i_schema = i;
				i_table = j;
			}
			table = append_string(table, profile->database.schema[i]->name);
			table = append_string(table, ".");
			table = append_string(table, profile->database.schema[i]->table[j]->name);
			list1[index++] = table;
		}
	}

	if(id == -1) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_combo_property("property_foreign_table", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Foreign Table", list1,  count, id);
	for(int i = 0; i < count; i++){
		free(list1[i]);
	}
	height += 21;


	wid = get_widget_child_by_name(dialog, "property_foreign_table");
	if(wid != NULL){
		g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_fkey_ftables_changed),
                    dialog); 
   }

  // property foreign columns 
	cols = NULL;
	cols = append_string(cols, "");
	if(!new) for(int i = 0; i < fkey->n_ref_columns; i++){
		cols = append_string(cols, fkey->ref_column[i]->name);
		if(i < fkey->n_ref_columns-1){
			cols = append_string(cols, ", ");
		}
	}
	if(strcmp(cols, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_column_property("property_foreign_columns", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Foreign Columns", cols, profile, i_schema, i_table, (new) ? NULL : fkey->ref_column, (new) ? 0 : fkey->n_ref_columns);
	free(cols);
	height += 21;
	
	wid = get_widget_child_by_name(dialog, "property_foreign_columns");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_fkey_columns_changed),
                    wid);  
	}
	
	// property on update rule
	gchar * list2[] = {"Cascade", "No Action", "Restrict", "Set Null", "Set Default"};
	id=1;
	if(!new)for(int i = 0; i < 5; i++){
		if(strcmp(list2[i], fkey->on_update_rule) == 0){
			id = i;
			break;
		}
	}
	add_combo_property("property_on_update_rule", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "On Update Rule", list2,  5, id);
	height += 21;

	// property on delete rule	
	id=1;
	if(!new) for(int i = 0; i < 5; i++){
		if(strcmp(list2[i], fkey->on_delete_rule) == 0){
			id = i;
			break;
		}
	}
	add_combo_property("property_on_delete_rule", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "On Delete Rule", list2,  5, id);
	height += 21;

	// property on match type		
	gchar * list3[] = {"Simple","Full"};
	id=0;
	if(!new) for(int i = 0; i < 2; i++){
		if(strcmp(list3[i], fkey->match_type) == 0){
			id = i;
			break;
		}
	}
	add_combo_property("property_match_type", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Match Type", list3,  2, id);
	height += 21;
	
	// property on deferrable	
	add_toggle_property("property_deferrable", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Deferrable",  (new) ? FALSE : fkey->deferrable, TRUE);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_deferrable");
	if(wid != NULL){
	 g_signal_connect ((gpointer) wid, "toggled",
                    G_CALLBACK (on_fkey_deferrable_toggled),
                    dialog);  
	}

	// property on check time
	gchar * list4[] = {"At the end of the transaction","After each statement"};
	id=0;
	if(!new) for(int i = 0; i < 2; i++){
		if(strcmp(list4[i], fkey->check_time) == 0){
			id = i;
			break;
		}
	}
	add_combo_property("property_check_time", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Check Time", list4,  2, id);
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_check_time");
	if(wid != NULL){
		if(!new){
			if(!fkey->deferrable) gtk_widget_set_sensitive(wid, FALSE);
		}
		else gtk_widget_set_sensitive(wid, FALSE);
	}
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "scrolledwindow11"), -1, height);
  	
  	
  // run dialog
  
  gboolean end = FALSE;
  while(!end){
  	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "object_properties_dialog")));
  	if(res == GTK_RESPONSE_OK){
			// get properties
			T_FKey * new_fkey = NULL;
			if(( new_fkey = (T_FKey *) malloc (sizeof(T_FKey)) ) == NULL){	
				fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
			}
			// name
			wid = get_widget_child_by_name(dialog, "property_name");
			if(wid != NULL) new_fkey->name = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_fkey->name = copy_string("");
			
			// comment
			wid = get_widget_child_by_name(dialog, "property_comment");
			if(wid != NULL){
				GtkTextBuffer * buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
				GtkTextIter start, end;
  			gtk_text_buffer_get_start_iter(buffer, &start);
  			gtk_text_buffer_get_end_iter(buffer, &end);
  			new_fkey->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
  		}
  		else new_fkey->comment = copy_string("");
  		
  		// columns
  		new_fkey->n_columns = 0;
  		new_fkey->column = NULL;
  		wid = get_widget_child_by_name(dialog, "property_columns");
			if(wid != NULL){
				gchar * columns = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
				columns = append_string(columns, ",");
			  gint s=0, e = 0;
			  gint len = strlen(columns);
			  
			  for(; e < len; e++){
			  	if(columns[e] == ','){
			  		columns[e] = '\0';
			  		gchar * col = copy_string(columns+s);
			  		gint m = new_fkey->n_columns;
			  		// add column	
			  		if(new_fkey->column == NULL){
							if((new_fkey->column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
						}
						else{
		 					if((new_fkey->column = (T_Column **) realloc((new_fkey->column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
						}
						if((new_fkey->column[m] = (T_Column *) malloc (sizeof(T_Column))) == NULL){
							fprintf(stderr, "Memory allocation error.");
	  					exit(0);
						}
						
						new_fkey->column[m]->name = col;
						new_fkey->n_columns++;
								  	
			  		columns[e] = ',';
			  		s = e+2;
			  		e+=2;
			  	}
			  }
			  free(columns);	
  		}
  		
  		// foreign table
  		wid = get_widget_child_by_name(dialog, "property_foreign_table");
			if(wid != NULL){
				 gchar * table = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
				 gint i = 0;
				 while(table[i] != '.'){
				 		i++;
				 }
				 table[i]='\0';
				 new_fkey->ref_schema = copy_string(table);
				 new_fkey->ref_table = copy_string(table+i+1);
			}
			else{
				new_fkey->ref_schema = copy_string("");
			 	new_fkey->ref_table = copy_string("");
			}
			
  		// foreign columns
  		new_fkey->n_ref_columns = 0;
  		new_fkey->ref_column = NULL;
  		wid = get_widget_child_by_name(dialog, "property_foreign_columns");
			if(wid != NULL){
				gchar * columns = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
				columns = append_string(columns, ",");
			  gint s=0, e = 0;
			  gint len = strlen(columns);
			  
			  for(; e < len; e++){
			  	if(columns[e] == ','){
			  		columns[e] = '\0';
			  		gchar * col = copy_string(columns+s);
			  		gint m = new_fkey->n_ref_columns;
			  		// add column
			  		if(new_fkey->ref_column == NULL){
							if((new_fkey->ref_column = (T_Column **) malloc(sizeof(T_Column*))) == NULL){
	  					 fprintf(stderr, "Memory allocation error.");
	   					 exit(0);
	   					}
						}
						else{
		 					if((new_fkey->ref_column = (T_Column **) realloc((new_fkey->ref_column), sizeof(T_Column*)*(m+1))) == NULL){
		  					fprintf(stderr, "Memory allocation error.");
	  						exit(0);
	 						}
						}
						if((new_fkey->ref_column[m] = (T_Column *) malloc (sizeof(T_Column))) == NULL){
							fprintf(stderr, "Memory allocation error.");
	  					exit(0);
						}
						
						new_fkey->ref_column[m]->name = col;
						new_fkey->n_ref_columns++;
								  	
			  		columns[e] = ',';
			  		s = e+2;
			  		e+=2;
			  	}
			  }
			  free(columns);	
  		} 		
  		
  		// on update rule
  		wid = get_widget_child_by_name(dialog, "property_on_update_rule");
			if(wid != NULL) new_fkey->on_update_rule = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_fkey->on_update_rule = copy_string("");
			
  		// on delete rule
  		wid = get_widget_child_by_name(dialog, "property_on_delete_rule");
			if(wid != NULL) new_fkey->on_delete_rule = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_fkey->on_delete_rule = copy_string("");
			
  		// match type
  		wid = get_widget_child_by_name(dialog, "property_match_type");
			if(wid != NULL) new_fkey->match_type = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_fkey->match_type = copy_string("");
			
  		// deferrable
  		wid = get_widget_child_by_name(dialog, "property_deferrable");
			if(wid != NULL) new_fkey->deferrable = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
			else new_fkey->deferrable = FALSE;
  		
  		// check time
  		wid = get_widget_child_by_name(dialog, "property_check_time");
			if(wid != NULL) new_fkey->check_time = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
			else new_fkey->check_time = copy_string("");
			
			
			// make sql code
			gchar *sql = NULL;
			gchar * parent = append_string(NULL, schema->name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, table->name);
			if(new){
				sql =  create_new_fkey_SQL_query(new_fkey, parent, FALSE);
			}
			else{
				sql = create_edit_fkey_SQL_query(fkey, new_fkey, parent);
			}
			//execute code
			if(sql != NULL){
				if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
				 end = TRUE;
					// get server
					gboolean e = FALSE;
					gint i_server = 0;
					for(int k = 0; k < o_tree.n_servers; k++){
						for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
							if(o_tree.server[k]->profile[l] == profile){
								i_server = k;
								e = TRUE;
								break;
							}
						}
						if(e) break;
					}
					gui_refresh( o_tree.server[i_server], profile);
				}
				free(sql);
			}
			else end = TRUE;
			
			// free index structure
  		free(new_fkey->name);
  		free(new_fkey->comment);
  		for(int i = 0; i < new_fkey->n_columns; i++){
  			free(new_fkey->column[i]->name);
  			free(new_fkey->column[i]);
  		}
  		free(new_fkey->column);
  		free(new_fkey->ref_schema);
  		free(new_fkey->ref_table);
  		for(int i = 0; i < new_fkey->n_ref_columns; i++){
  			free(new_fkey->ref_column[i]->name);
  			free(new_fkey->ref_column[i]);
  		}
  		free(new_fkey->ref_column);
  		free(new_fkey->on_update_rule);
  		free(new_fkey->on_delete_rule);  		
  		free(new_fkey->match_type);
  		free(new_fkey->check_time);
  		free(new_fkey);
	  	new_fkey = NULL;
	  	
  		free(parent);
  	}
  	else{
  		end = TRUE;
  	}
  }
  
  GList * list = gtk_window_list_toplevels();
	GtkWidget * window = NULL;
	for(; list != NULL; list = list->next){
		gchar * name = copy_string(gtk_widget_get_name(GTK_WIDGET(list->data)));
	
		if(strcmp(name,  "property_columns") == 0){
		 	window = GTK_WIDGET(list->data);
		 	gtk_widget_destroy(window);  
		}
		if(strcmp(name,  "property_foreign_columns") == 0){
		 	window = GTK_WIDGET(list->data);
		 	gtk_widget_destroy(window);  
		}
		free(name);
	}
	
	gtk_widget_hide(glade_xml_get_widget(xml, "object_properties_dialog"));
}

void
on_fkey_ftables_changed (GtkComboBox *combobox, gpointer user_data){
	GList * list = gtk_window_list_toplevels();
	GtkWidget * window = NULL;
	for(; list != NULL; list = list->next){
		gchar * name = copy_string(gtk_widget_get_name(GTK_WIDGET(list->data)));
		
		if(strcmp(name,  "property_foreign_columns") == 0){
		 window = GTK_WIDGET(list->data);
		 free(name);
		 break;
		}
		free(name);
	}
	if(window == NULL) return;
	
	GtkEntry * entry = GTK_ENTRY(GTK_BIN(combobox)->child);
	
	// selected table
	gchar * table = copy_string(gtk_entry_get_text(entry));
	gchar tab[strlen(table)+1];
	gint i = 0;
	while(table[i] != '.') i++;
	tab[0] = '\0';
	strcat(tab, (table+i+1));
	free(table);
	
	// find table
	gint i_server = -1, i_profile = -1, i_schema = -1, i_table = -1; 
	gboolean found = FALSE;
	for(int j = 0; j < o_tree.n_servers; j++){
		for(int k = 0; k < o_tree.server[j]->n_profiles; k++){
			for(int l = 0; l < o_tree.server[j]->profile[k]->database.n_schemas; l++){
				for(int m = 0; m < o_tree.server[j]->profile[k]->database.schema[l]->n_tables; m++){
					if(strcmp(tab, o_tree.server[j]->profile[k]->database.schema[l]->table[m]->name) == 0){
						i_server = j;
						i_profile = k;
						i_schema = l;
						i_table = m;
						found = TRUE;
						break; 
					}
				}
				if(found) break;
			}
 			if(found) break;
		}
		if(found) break;
	}
	
	// add available columns
	GtkWidget * wid = get_widget_child_by_name(window, "csw_available_cols");
	if(wid != NULL){
		GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW(wid));
		GtkListStore * store = GTK_LIST_STORE(model);
		GtkTreeIter child;
		gtk_list_store_clear(store);
		for(int i = 0; i < o_tree.server[i_server]->profile[i_profile]->database.schema[i_schema]->table[i_table]->n_columns; i++){
			gchar * text1;
 			gtk_list_store_append(store, &child);
 			text1 = normal_text_with_color(o_tree.server[i_server]->profile[i_profile]->database.schema[i_schema]->table[i_table]->column[i]->name, "#000000");
			gtk_list_store_set(store, &child, 0, (o_tree.server[i_server]->profile[i_profile]->database.schema[i_schema]->table[i_table]->column[i]->pkey) ? get_icon(PIXMAP_DIRECTORY "/FieldPK.png") : get_icon(PIXMAP_DIRECTORY "/Field.png"), 1, text1, -1);
			free(text1);
		}
	}
	wid = get_widget_child_by_name(window, "csw_selected_cols");
	if(wid != NULL){
		GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW(wid));
		GtkListStore * store = GTK_LIST_STORE(model);
		gtk_list_store_clear(store);
	}
	wid = get_widget_child_by_name(GTK_WIDGET(user_data), "property_foreign_columns");
	if(wid != NULL){
		gtk_entry_set_text(GTK_ENTRY(wid), "");	
	}
}

void
on_fkey_columns_changed (GtkEditable *editable, gpointer user_data){
	GtkEntry * entry = GTK_ENTRY(user_data);
	gchar * text = copy_string(gtk_entry_get_text(entry));
	if(strcmp(text, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else{
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_name");
		GtkWidget * wid2 = get_widget_child_by_name(dialog, "property_columns");
		GtkWidget * wid3 = get_widget_child_by_name(dialog, "property_foreign_columns");
		if(wid1 != NULL && wid2 != NULL && wid3 != NULL){
			if(strcmp(gtk_entry_get_text(GTK_ENTRY(wid1)), "") != 0 && strcmp(gtk_entry_get_text(GTK_ENTRY(wid2)), "") != 0 && strcmp(gtk_entry_get_text(GTK_ENTRY(wid3)), "") != 0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), TRUE);
					
		}
		
	}
}

void
on_fkey_deferrable_toggled (GtkToggleButton *togglebutton, gpointer user_data){
	GtkWidget * wid = get_widget_child_by_name(GTK_WIDGET(user_data), "property_check_time");
	if(wid != NULL){
		if(gtk_toggle_button_get_active(togglebutton)){
			gtk_widget_set_sensitive(wid, TRUE);
		}
		else{
			gtk_widget_set_sensitive(wid, FALSE);		
		}
	}
}

/////////////////////////////////////////////// Check dialog /////////////////////////////////////////////////////////////////////////
/*
 * Function that runs check dialog
 */
void
run_check_dialog(T_Profile * profile, T_Schema * schema, T_Table * table, T_Check * check, gboolean new){
	init_op_dialog();
	
	if(new) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
  else gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), TRUE);
	
	GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
	// set title
	gtk_window_set_title(GTK_WINDOW(dialog), "Check properties");
	
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "opd_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);

  gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	if(!new) if(strcmp(check->name, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_entry_property("property_name", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Name",  (!new) ? check->name : "");
	height += 21;	
	GtkWidget * wid = get_widget_child_by_name(dialog, "property_name");
	if(wid != NULL){
		g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_check_entries_changed),
                    wid);  
	}	
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", (!new) ? check->comment : "");
	height += 41;
	if(!new){
	gchar * text = itoa_n(check->OID, 0);
	add_noneditable_property(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "OID", text);
	free(text);
	height += 21;
	}
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Check",  PIXMAP_DIRECTORY "/Check.png");
	height += 21;
	if(!new) if(strcmp(check->condition, "")==0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	add_entry_property("property_condition", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Condition",  (!new) ? check->condition : "");
	height += 21;
	wid = get_widget_child_by_name(dialog, "property_condition");
	if(wid != NULL){
		g_signal_connect ((gpointer) wid, "changed",
                    G_CALLBACK (on_check_entries_changed),
                    wid);  
	}	
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "scrolledwindow11"), -1, height);
  
  
  // run dialog
  
  gboolean end = FALSE;
  while(!end){
  	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "object_properties_dialog")));
  	if(res == GTK_RESPONSE_OK){
			// get properties
			T_Check * new_check = NULL;
			if(( new_check = (T_Check *) malloc (sizeof(T_Check)) ) == NULL){	
				fprintf(stderr, "Memory allocation error: callbacks.c");
	    	exit(0);
			}
			// name
			wid = get_widget_child_by_name(dialog, "property_name");
			if(wid != NULL) new_check->name = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_check->name = copy_string("");
			
			// comment
			wid = get_widget_child_by_name(dialog, "property_comment");
			if(wid != NULL){
				GtkTextBuffer * buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
				GtkTextIter start, end;
  			gtk_text_buffer_get_start_iter(buffer, &start);
  			gtk_text_buffer_get_end_iter(buffer, &end);
  			new_check->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
  		}
  		else new_check->comment = copy_string("");
  		
			// condition
			wid = get_widget_child_by_name(dialog, "property_condition");
			if(wid != NULL) new_check->condition = copy_string(gtk_entry_get_text(GTK_ENTRY(wid)));
			else new_check->condition = copy_string("");
			
			// make sql code
			gchar *sql = NULL;
			gchar * parent = append_string(NULL, schema->name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, table->name);
			if(new){
				sql =  create_new_check_SQL_query(new_check, parent, FALSE);
			}
			else{
				sql = create_edit_check_SQL_query(check, new_check, parent);
			}
			//execute code
			if(sql != NULL){
				if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
				 end = TRUE;
					// get server
					gboolean e = FALSE;
					gint i_server = 0;
					for(int k = 0; k < o_tree.n_servers; k++){
						for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
							if(o_tree.server[k]->profile[l] == profile){
								i_server = k;
								e = TRUE;
								break;
							}
						}
						if(e) break;
					}
					gui_refresh( o_tree.server[i_server], profile);
				}
				free(sql);
			}
			else end = TRUE;
			
			// free check structure
			free(new_check->name);
			free(new_check->comment);
			free(new_check->condition);
						
  		free(parent);
  	}
  	else{
  		end = TRUE;
  	}
  }
  gtk_widget_hide(glade_xml_get_widget(xml, "object_properties_dialog"));
  
}

void
on_check_entries_changed (GtkEditable *editable, gpointer user_data){
	GtkEntry * entry = GTK_ENTRY(user_data);
	gchar * text = copy_string(gtk_entry_get_text(entry));
	if(strcmp(text, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), FALSE);
	}
	else{
		GtkWidget * dialog = glade_xml_get_widget(xml, "object_properties_dialog");
		GtkWidget * wid1 = get_widget_child_by_name(dialog, "property_name");
		GtkWidget * wid2 = get_widget_child_by_name(dialog, "property_condition");
		if(wid1 != NULL && wid2 != NULL){
			if(strcmp(gtk_entry_get_text(GTK_ENTRY(wid1)), "") != 0 && strcmp(gtk_entry_get_text(GTK_ENTRY(wid2)), "") != 0) gtk_widget_set_sensitive(glade_xml_get_widget(xml, "opd_ok"), TRUE);
		}
	}
}
