/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "schema_wizard.h"
#include "../functions.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/get_db_objects.h"
#include "../otree/show_otree.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/properties_widget.h"

// Structure for new database
T_Schema * csw_schema;
gint new_schema_count = 0;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*						Create database profile wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
init_c_schema_wizard(T_Profile * profile){
	// Color settings
	GdkColor color;
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw1_hr_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw2_hr_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw2_properties_bg2"), GTK_STATE_NORMAL, &color);
	
	color.red = 204*257;
  color.green = 0*257;
  color.blue = 0*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw1_hr"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw2_hr"), GTK_STATE_NORMAL, &color);

	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"csw2_properties_hr_bg"), GTK_STATE_NORMAL, &color);
	
	csw_schema = NULL;
	
	// init schema name
	GtkWidget * name = glade_xml_get_widget(xml, "csw1_name");
	gchar * new_schema = append_string(NULL, "Schema");
	gchar * count = itoa_n(new_schema_count, 0);
	new_schema = append_string(new_schema, count);
	free(count);
	gtk_entry_set_text(GTK_ENTRY(name), new_schema);
	free(new_schema);
	
	// add properties
	GtkWidget * dialog = glade_xml_get_widget(xml, "c_schema_wiz2");
		
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "csw2_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);
	
	gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	T_Roles * roles = get_roles(profile, FALSE);
	if(roles != NULL){
	gint role_id=-1;
	add_combo_property("property_owner", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Owner", roles->name,  roles->n_roles, role_id);
	for(int i = 0; i < roles->n_roles; i++){
		free(roles->name[i]);
	}
	free(roles->name);
	free(roles);
	}
	height += 21;
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", "");
	height += 41;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "csw2_scrolledwindow"), -1, height);
}

void 
run_c_schema_wizard(T_Profile * profile){

	GtkWidget * widget = NULL;
	gint page = 1;
	gboolean not_end = TRUE;
	
	init_c_schema_wizard(profile);

	not_end = TRUE;
	while(not_end){
		switch(page){
			case 1:	widget = glade_xml_get_widget(xml,"c_schema_wiz1"); break;
			case 2: widget = glade_xml_get_widget(xml,"c_schema_wiz2"); break;
			default:;
		};
		gint res = gtk_dialog_run(GTK_DIALOG(widget));
		
		if(res == GTK_RESPONSE_YES){
			if(page < 2){
				if(page == 1){ // get new db name
					page++;
					gtk_widget_hide(widget);	
				}
				else {
					 page++;
  				 gtk_widget_hide(widget);
				}
			}
			else{ // end the wizard
					not_end = FALSE;
					// get properties
					T_Schema * schema = csw_get_properties();
					gchar * sql = NULL;
					
					// create sql query
					if(schema != NULL){					
						sql = create_new_schema_SQL_query(schema);
					}
					
					// execute sql
					if(!execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
						// get server
						gboolean e = FALSE;
						gint i_server = 0;
						for(int k = 0; k < o_tree.n_servers; k++){
							for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
								if(o_tree.server[k]->profile[l] == profile){
									i_server = k;
									e = TRUE;
									break;
								}
							}
							if(e) break;
						}
						gui_refresh( o_tree.server[i_server], profile);
						not_end = TRUE;
					}
					else{
					  new_schema_count++;
					  gtk_widget_hide(widget);
					}
					
					if(schema != NULL){
						free(sql);
						free(schema->name);
						free(schema->owner);
						free(schema->comment);
						free(schema);
					}
			}
		}
		else if(res == GTK_RESPONSE_NO){
			if(page > 1){
				page--;
				gtk_widget_hide(widget);
			}
		}
		else if(res == GTK_RESPONSE_CANCEL || res == GTK_RESPONSE_DELETE_EVENT){
				not_end = FALSE;
				gtk_widget_hide(widget);
		}
	}
	
	// clear properties
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_schema_wiz2")), "properties_column1");
	if(wid != NULL) gtk_widget_destroy(wid);
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_schema_wiz2")), "properties_column2");
	if(wid != NULL) gtk_widget_destroy(wid);
}

T_Schema *
csw_get_properties(void){
	T_Schema * schema = NULL;
		
	schema = (T_Schema *) malloc (sizeof(T_Schema));
	if(schema == NULL){
		fprintf(stderr, "Memory allocation error.");
	 	exit(0);
	}
	
	GtkWidget * wid = NULL;
	wid = glade_xml_get_widget(xml, "csw1_name");

	schema->name = append_string(NULL, (gchar *)gtk_entry_get_text(GTK_ENTRY(wid)));
	
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_schema_wiz2")), "property_owner");
	if(wid == NULL) return NULL;
	schema->owner = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_schema_wiz2")), "property_comment");
	if(wid == NULL) return NULL;
	GtkTextBuffer * buffer;
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
	GtkTextIter start, end;
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
	schema->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
		
	return schema;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for create database wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_csw_signals(GladeXML * source){
	GtkWidget * widget;
  
	// Check if database name not null
	widget = glade_xml_get_widget(source, "csw1_name");	
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_csw1_schema_name_editing_done),
                    NULL);
                                     
}

void
on_csw1_schema_name_editing_done(GtkEditable *editable, gpointer user_data)
{
	const gchar * name;
	name = gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"csw1_name")));

	if(strcmp(name, "") == 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"csw1_bt_next"), FALSE);
	}
	else{
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"csw1_bt_next"), TRUE);
	}
}
