/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "table_wizard.h"
#include "../functions.h"
#include "../database/db_connection.h"
#include "../database/create_db_objects.h"
#include "../database/get_db_objects.h"
#include "../otree/show_otree.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/properties_widget.h"

// Structure for new table
T_Table * ctw_table;
gint new_table_count = 0;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*						Create database profile wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
init_c_table_wizard(T_Profile * profile){
	// Color settings
	GdkColor color;
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw1_hr_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw2_hr_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw3_hr_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw3_properties_bg2"), GTK_STATE_NORMAL, &color);
	
	color.red = 204*257;
  color.green = 0*257;
  color.blue = 0*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw1_hr"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw2_hr"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw3_hr"), GTK_STATE_NORMAL, &color);
	
	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
	gtk_widget_modify_bg (glade_xml_get_widget(xml,"ctw3_properties_hr_bg"), GTK_STATE_NORMAL, &color);
	
	ctw_table = NULL;
	
	// init schemas
	GtkTreeModel      *model;
  if((model = gtk_combo_box_get_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "ctw1_schema")))) != NULL){
		gtk_list_store_clear(GTK_LIST_STORE(model));
	}
	else{
	  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
		gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "ctw1_schema")), model);
		gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(glade_xml_get_widget(xml, "ctw1_schema")), 0);
	}
	
	gtk_entry_set_editable(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "ctw1_schema"))->child), FALSE);
	for(int i = 0; i < profile->database.n_schemas; i++){
		gtk_combo_box_append_text (GTK_COMBO_BOX (glade_xml_get_widget(xml, "ctw1_schema")), profile->database.schema[i]->name);
	}
	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "ctw1_schema"))->child), "");
	gtk_combo_box_set_active( GTK_COMBO_BOX (glade_xml_get_widget(xml, "ctw1_schema")), -1);
	
	// init table name
	GtkWidget * name = glade_xml_get_widget(xml, "ctw1_name");
	gchar * new_table = append_string(NULL, "Table");
	gchar * count = itoa_n(new_table_count, 0);
	new_table = append_string(new_table, count);
	free(count);
	gtk_entry_set_text(GTK_ENTRY(name), new_table);
	free(new_table);
	
	// init inherit toggle button
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "ctw1_inherit")), FALSE);
	// init tables to inherit from
	if((model = gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables")))) != NULL){
		gtk_list_store_clear(GTK_LIST_STORE(model));
	}
	else{
		create_column_select_property_list_view (glade_xml_get_widget(xml, "ctw2_available_tables"), "Available tables");
	}
	
	if((model = gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables")))) != NULL){
		gtk_list_store_clear(GTK_LIST_STORE(model));
	}
	else{
		create_column_select_property_list_view (glade_xml_get_widget(xml, "ctw2_selected_tables"), "Selected tables");
	}	
	
	GtkTreeModel * model2 = gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables")));
	GtkListStore * store = GTK_LIST_STORE(model2);
	GtkTreeIter child;
	
	for(int i = 0; i < profile->database.n_schemas; i++){
		for(int j = 0; j < profile->database.schema[i]->n_tables; j++){
			gchar * text1 = NULL;
 			gtk_list_store_append(store, &child);
 			
 			text1 = append_string(text1, profile->database.schema[i]->name);
 			text1 = append_string(text1, ".");
 			text1 = append_string(text1, profile->database.schema[i]->table[j]->name);
 			
 			gchar *text2 = normal_text_with_color(text1, "#000000");
			gtk_list_store_set(store, &child, 0, get_icon(PIXMAP_DIRECTORY "/Table.png"), 1, text2, -1);
			free(text2);
			free(text1);
		}
	}
	
	// add properties
	GtkWidget * dialog = glade_xml_get_widget(xml, "c_table_wiz2");
		
	// add properties
	GtkWidget * properties_columns = glade_xml_get_widget(xml, "ctw3_properties");
  GtkWidget * properties_column1 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column1, "properties_column1");
  gtk_widget_show (properties_column1);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column1, FALSE, TRUE, 0);
 
  GtkWidget * properties_column2 = gtk_vbox_new (FALSE, 1);
  gtk_widget_set_name(properties_column2, "properties_column2");
  gtk_widget_show (properties_column2);
  gtk_box_pack_start (GTK_BOX (properties_columns), properties_column2, TRUE, TRUE, 0);
	
	gint height = 0;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Common",  PIXMAP_DIRECTORY "/Common.png");
	height += 21;
	T_Roles * roles = get_roles(profile, FALSE);
	if(roles != NULL){
	gint role_id=-1;
	roles->name = (gchar ** )realloc(roles->name, sizeof(gchar *)*(roles->n_roles+1));
	if(roles->name == NULL){
		fprintf(stderr, "Memory allocation error.");
	 	exit(0);
	}
	roles->name[roles->n_roles] = copy_string("");
	roles->n_roles++;
	add_combo_property("property_owner", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Owner", roles->name,  roles->n_roles, role_id);
	for(int i = 0; i < roles->n_roles; i++){
		free(roles->name[i]);
	}
	free(roles->name);
	free(roles);
	}
	height += 21;
	add_comment_entry_property("property_comment", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Comment", "");
	height += 41;
	add_header_proper(GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Table",  PIXMAP_DIRECTORY "/Table.png");
	height += 21;
	add_toggle_property("property_with_OIDs", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "With OIDs",  FALSE, TRUE);
	height += 21;
	T_Tablespaces * tspces = get_Tablespaces(profile);
	if(tspces != NULL){
		gint tspc_id=-1;
		tspces->name = (gchar ** )realloc(tspces->name, sizeof(gchar *)*(tspces->n_tablespaces+1));
		if(tspces->name == NULL){
			fprintf(stderr, "Memory allocation error.");
	 		exit(0);
		}
		tspces->name[tspces->n_tablespaces] = copy_string("");
		tspces->n_tablespaces++;
		add_combo_property("property_tablespace", dialog, GTK_BOX(properties_column1), GTK_BOX(properties_column2), "Tablespace", tspces->name,  tspces->n_tablespaces, tspc_id);
		for(int i = 0; i < tspces->n_tablespaces; i++){
			free(tspces->name[i]);
		}
		free(tspces->name);
		free(tspces);
	}
	height += 21;
	gtk_widget_set_size_request (glade_xml_get_widget(xml, "ctw3_scrolledwindow"), -1, height);
}

void 
run_c_table_wizard(T_Profile * profile){

	GtkWidget * widget = NULL;
	gint page = 1;
	gboolean not_end = TRUE;
	
	init_c_table_wizard(profile);

	not_end = TRUE;
	while(not_end){
		switch(page){
			case 1:	widget = glade_xml_get_widget(xml,"c_table_wiz1"); break;
			case 2: widget = glade_xml_get_widget(xml,"c_table_wiz2"); break;
			case 3: widget = glade_xml_get_widget(xml,"c_table_wiz3"); break;
			default:;
		};
		gint res = gtk_dialog_run(GTK_DIALOG(widget));
		
		if(res == GTK_RESPONSE_YES){
			if(page < 3){
				if(page == 1){ // get new db name
					if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "ctw1_inherit")))) page++;
					else page += 2;
					gtk_widget_hide(widget);	
				}
				else {
					 page++;
  				 gtk_widget_hide(widget);
				}
			}
			else{ // end the wizard
					not_end = FALSE;
					
					// get properties
					T_Table * table = ctw_get_properties();
					gchar * sql = NULL;
					table->n_columns = 0;
					table->n_indexes = 0;
					table->n_fkeys = 0;
					table->n_checks = 0;
					table->n_triggers = 0;
					table->n_rules = 0;
					
					// create sql query
					if(table != NULL){					
						sql = create_new_table_SQL_query(table, NULL);
					}
					
					// execute sql
					if(sql != NULL){
						if(!execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)){
							not_end = TRUE;
						}
						else{
							// get server
							gboolean e = FALSE;
							gint i_server = 0;
							for(int k = 0; k < o_tree.n_servers; k++){
								for(int l = 0; l < o_tree.server[k]->n_profiles; l++){
									if(o_tree.server[k]->profile[l] == profile){
										i_server = k;
										e = TRUE;
										break;
									}
								}
								if(e) break;
							}
							gui_refresh( o_tree.server[i_server], profile);
					  	new_table_count++;
					  	gtk_widget_hide(widget);
						}
						free(sql);
					}
					if(table != NULL){
						free(table->name);
						free(table->owner);
						free(table->comment);
						free(table->tablespace);
						free(table->inherited_from);
						free(table);
					}
			}
		}
		else if(res == GTK_RESPONSE_NO){
			if(page > 1){
				if(page == 3 && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "ctw1_inherit"))))	page -= 2;
				else page--;
				gtk_widget_hide(widget);
			}
		}
		else if(res == GTK_RESPONSE_CANCEL || res == GTK_RESPONSE_DELETE_EVENT){
				not_end = FALSE;
				gtk_widget_hide(widget);
		}
	}
	
	// clear properties
	GtkWidget * wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "properties_column1");
	if(wid != NULL) gtk_widget_destroy(wid);
	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "properties_column2");
	if(wid != NULL) gtk_widget_destroy(wid);
}

T_Table *
ctw_get_properties(void){
	T_Table * table = NULL;
		
	table = (T_Table *) malloc (sizeof(T_Table));
	if(table == NULL){
		fprintf(stderr, "Memory allocation error.");
	 	exit(0);
	}
	
	GtkWidget * wid = NULL;
	wid = glade_xml_get_widget(xml, "ctw1_schema");

	table->name = append_string(NULL, (gchar *)gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));
	table->name = append_string(table->name, "\".\"");
	wid = glade_xml_get_widget(xml, "ctw1_name");
	table->name = append_string(table->name, (gchar *)gtk_entry_get_text(GTK_ENTRY(wid)));

	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "property_owner");
	if(wid == NULL) return NULL;
	table->owner = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "property_comment");
	if(wid == NULL) return NULL;
	GtkTextBuffer * buffer;
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (wid));
	GtkTextIter start, end;
  gtk_text_buffer_get_start_iter(buffer, &start);
  gtk_text_buffer_get_end_iter(buffer, &end);
	table->comment = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "property_with_OIDs");
	if(wid == NULL) return NULL;
	table->WithOIDs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));

	wid = NULL;
	wid = get_widget_child_by_name(GTK_WIDGET(glade_xml_get_widget(xml, "c_table_wiz3")), "property_tablespace");
	if(wid == NULL) return NULL;
	table->tablespace = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(wid)->child)));

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "ctw1_inherit")))){
		GtkTreeIter iter;
		GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables"));
		GtkTreeModel * model = gtk_tree_view_get_model(view);
		gchar * value;
		gboolean next = FALSE;
		gchar * cols = NULL;
	
		if(gtk_tree_model_get_iter_first(model, &iter)){
			do{
				gtk_tree_model_get(model, &iter, 1, &value,-1);
				
				gchar * s = get_text_from_markup(value);
				cols = append_string(cols, s);
				free(value);
				free(s);	
				if(gtk_tree_model_iter_next(model,&iter)){
					cols = append_string(cols, ", ");
				 	next =TRUE;
				}
				else next = FALSE;
	 		}while(next);
		}
		if(cols != NULL)	table->inherited_from = cols;
		else table->inherited_from = copy_string("");
	}
	else{
		table->inherited_from = copy_string("");
	}

	return table;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for create database wizard                                                                          */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_ctw_signals(GladeXML * source){
	GtkWidget * widget;
  
	// Check if database name not null
	widget = glade_xml_get_widget(source, "ctw1_name");	
	g_signal_connect ((gpointer) widget, "changed",
                    G_CALLBACK (on_ctw1_table_name_editing_done),
                    NULL);
  
  widget = glade_xml_get_widget(source, "ctw1_schema");	
	g_signal_connect ((gpointer) GTK_BIN(widget)->child, "changed",
                    G_CALLBACK (on_ctw1_table_name_editing_done),
                    NULL);
   
	g_signal_connect((gpointer)glade_xml_get_widget(xml, "ctw2_select_all"), "clicked",
                    G_CALLBACK (ctw_select_all),
                    NULL);
  g_signal_connect((gpointer)glade_xml_get_widget(xml, "ctw2_select"), "clicked",
                    G_CALLBACK (ctw_select),
                    NULL);
	g_signal_connect((gpointer)glade_xml_get_widget(xml, "ctw2_remove"), "clicked",
                    G_CALLBACK (ctw_remove),
                    NULL);
  g_signal_connect((gpointer)glade_xml_get_widget(xml, "ctw2_remove_all"), "clicked",
                    G_CALLBACK (ctw_remove_all),
                    NULL);                   
}

void
on_ctw1_table_name_editing_done(GtkEditable *editable, gpointer user_data)
{
	const gchar * name, *schema;
	name = gtk_entry_get_text( GTK_ENTRY( glade_xml_get_widget(xml,"ctw1_name")));
	schema = gtk_entry_get_text( GTK_ENTRY( GTK_BIN(glade_xml_get_widget(xml,"ctw1_schema"))->child ));
	
	if(strcmp(name, "") != 0 && strcmp(schema, "") != 0){
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"ctw1_bt_next"), TRUE);
	}
	else{
		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"ctw1_bt_next"), FALSE);
	}
}

void
ctw_select_all(GtkButton * button, gpointer user_data){
	GtkTreeIter iter, new;
	GtkTreeView * view1 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables"));
	GtkTreeView * view2 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables"));
	GtkTreeModel * model1 = gtk_tree_view_get_model(view1);
	GtkTreeModel * model2 = gtk_tree_view_get_model(view2);
	gchar * value;
	
	while(gtk_tree_model_get_iter_first(model1,&iter)){
		GdkPixbuf       *icon;
		
		gtk_tree_model_get(model1, &iter, 0, &icon, 1, &value,-1);
		
		GtkListStore   *store;
  	
		store= GTK_LIST_STORE(model2);
			
	 gtk_list_store_append(store, &new);
	 
    gtk_list_store_set(store, &new,
                       COL_ICON, icon,
                       COL_TEXT, value,
                       -1);
		
		gtk_list_store_remove(GTK_LIST_STORE(model1), &iter);
	}
	
}

void
ctw_select(GtkButton * button, gpointer user_data){
	GtkTreeIter iter, new;
	GtkTreeView * view1 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables"));
	GtkTreeView * view2 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables"));
	GtkTreeModel * model1 = gtk_tree_view_get_model(view1);
	GtkTreeModel * model2 = gtk_tree_view_get_model(view2);
	gchar * value;	
	GdkPixbuf       *icon;
	
	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(view1), &model1, &iter)){
		gtk_tree_model_get(model1, &iter, 0,&icon,1, &value,-1);
		
		GtkListStore   *store;
		
		store= GTK_LIST_STORE(model2);
        
		gtk_list_store_append(store, &new);

    gtk_list_store_set(store, &new,
                       COL_ICON, icon,
                       COL_TEXT, value,
                       -1);
		gtk_list_store_remove(GTK_LIST_STORE(model1), &iter);
		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(view2), &new);
	}
}

void
ctw_remove(GtkButton * button, gpointer user_data){
	GtkTreeIter iter, new;
	GtkTreeView * view1 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables"));
	GtkTreeView * view2 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables"));
	GtkTreeModel * model1 = gtk_tree_view_get_model(view1);
	GtkTreeModel * model2 = gtk_tree_view_get_model(view2);
	gchar * value;	
	GdkPixbuf       *icon;
	
	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(view2), &model2, &iter)){
		gtk_tree_model_get(model2, &iter, 0,&icon,1, &value,-1);
		
		GtkListStore   *store;
		
		store= GTK_LIST_STORE(model1);
        
		gtk_list_store_append(store, &new);

    gtk_list_store_set(store, &new,
                       COL_ICON, icon,
                       COL_TEXT, value,
                       -1);
		gtk_list_store_remove(GTK_LIST_STORE(model2), &iter);
		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(view1), &new);
	}
}

void
ctw_remove_all(GtkButton * button, gpointer user_data){
	GtkTreeIter iter, new;
	GtkTreeView * view1 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_available_tables"));
	GtkTreeView * view2 = GTK_TREE_VIEW(glade_xml_get_widget(xml, "ctw2_selected_tables"));
	GtkTreeModel * model1 = gtk_tree_view_get_model(view1);
	GtkTreeModel * model2 = gtk_tree_view_get_model(view2);
	gchar * value;
	
	while(gtk_tree_model_get_iter_first(model2,&iter)){
		GdkPixbuf       *icon;
		
		gtk_tree_model_get(model2, &iter, 0, &icon, 1, &value,-1);
		
		GtkListStore   *store;
  	
		store= GTK_LIST_STORE(model1);
			
	 gtk_list_store_append(store, &new);
	 
    gtk_list_store_set(store, &new,
                       COL_ICON, icon,
                       COL_TEXT, value,
                       -1);
		
		gtk_list_store_remove(GTK_LIST_STORE(model2), &iter);
	}
		
}
