/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "create_db_objects.h"
#include "get_db_objects.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/treeview_functions.h"
#include "../GUI/textview_functions.h"

/*
 * Function that executes given sql code
 */
gboolean
execute_sql(gchar * sql, GtkTreeView * data_view, GtkTextView * mess_view, T_Profile * profile, gboolean errors_only, gboolean show_code){
	if(sql == NULL) return FALSE;
		
	if(show_code){
		// init messages textview
		GtkTextBuffer * buffer;
		GtkTextIter start, end;
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "ssd_messages")));
		gtk_text_buffer_get_start_iter(buffer, &start);
 		gtk_text_buffer_get_end_iter(buffer, &end);
		gtk_text_buffer_delete(buffer, &start, &end);
		init_message_view(glade_xml_get_widget(xml, "ssd_messages"), FALSE, "");
	}
	
	gchar * SQL = NULL;
	SQL = copy_string(sql);
	gboolean end;
	do{
		end = TRUE;
		if(show_code){	// show dialog that shows sql code
			// init sql textview
			GtkTextBuffer * buffer;
			buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "ssd_SQL_code")));
			GtkTextIter start, end;
 			gtk_text_buffer_get_start_iter(buffer, &start);
 			gtk_text_buffer_get_end_iter(buffer, &end);
			gtk_text_buffer_delete(buffer, &start, &end);
			init_text_view(glade_xml_get_widget(xml, "ssd_SQL_code"), TRUE, SQL);
					
			gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "show_SQL_dialog")));
  		if(res == GTK_RESPONSE_YES){
  			mess_view = GTK_TEXT_VIEW (glade_xml_get_widget(xml, "ssd_messages"));
  			//get new sql code
  			buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "ssd_SQL_code")));
				GtkTextIter start, end;
  			gtk_text_buffer_get_start_iter(buffer, &start);
  			gtk_text_buffer_get_end_iter(buffer, &end);
			
				if(SQL != NULL) free(SQL);
				SQL = copy_string(gtk_text_buffer_get_text(buffer, &start, &end, TRUE));
  		}
  		else{
  			gtk_widget_hide(glade_xml_get_widget(xml, "show_SQL_dialog"));
  		 return FALSE;
  		}
		}
		
		// Execution
		PGresult *res = PQexec(profile->conn, SQL);
		
		// Get result type
		ExecStatusType stat = PQresultStatus(res);
		gchar * message = NULL;
		gboolean data = FALSE;
		gboolean command = FALSE;
		gboolean information = FALSE;
		gboolean warning = FALSE;
		gboolean error = FALSE;

		switch(stat){
			case PGRES_EMPTY_QUERY:
				message = append_string(NULL, "Warning: Empty query.\n");
				warning = TRUE;
			break;
			case PGRES_COMMAND_OK:
				message = PQresultErrorMessage(res);
				command = TRUE;
				information = TRUE;
			break;
			case PGRES_TUPLES_OK:
				message = PQresultErrorMessage(res);
				data = TRUE;
				information = TRUE;
			break;
			case PGRES_COPY_OUT:
				message = PQresultErrorMessage(res);
				information = TRUE;
			break;
			case PGRES_COPY_IN:
				message = PQresultErrorMessage(res);
				information = TRUE;
			break;
			case PGRES_BAD_RESPONSE:
				message = PQresultErrorMessage(res);
				error = TRUE;
			break;
			case PGRES_NONFATAL_ERROR:
				message = PQresultErrorMessage(res);
				error = TRUE;
			break;
			case PGRES_FATAL_ERROR:
				message = PQresultErrorMessage(res);
				error = TRUE;
			break;
		}

		// show data
		if(data){
			if(data_view != NULL){
				// add columns
				gint cols = PQnfields(res);
			
				GType types[cols+1];
				types[0] = G_TYPE_STRING;
				add_row_number_column(data_view, 0);
			
				for(int i = 0; i < cols; i++){
				
					if(is_bool( itoa_n(PQftype(res, i), 0), profile) ){
						types[i+1] = G_TYPE_BOOLEAN;
						gchar * name = parse_underline(PQfname(res, i));
						add_bool_column(data_view, name, i+1);
						free(name);
					}
					else{
						types[i+1] = G_TYPE_STRING;
						gchar * name = parse_underline(PQfname(res, i));
						add_text_column(data_view, name, i+1);
						free(name);
					}
				}
				// add data
				GtkTreeModel * model = GTK_TREE_MODEL(gtk_list_store_newv(cols+1, types));
			
				gint tuples = PQntuples(res);
				message = append_string(NULL, "Information: ");
				message = append_string(message, itoa_n(tuples, 0));
				message = append_string(message, " rows fetched.\n");
			
				for(int j = 0; j < tuples; j++){
		 			GtkTreeIter iter;
					gtk_list_store_append(GTK_LIST_STORE(model), &iter);
					gchar * t = itoa_n( j+1 ,0);
					gchar * text = normal_text_with_color(t, "#000000");
					free(t);
					gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, text, -1);
					free(text);
					for(int i = 0; i < cols; i++){
						if(types[i+1] == G_TYPE_BOOLEAN){
							gtk_list_store_set(GTK_LIST_STORE(model), &iter, i+1, (strcmp(PQgetvalue(res, j, i), "t") == 0) ? TRUE : FALSE, -1);
						}
						else{
							gchar * text = normal_text_with_color(PQgetvalue(res, j, i), "#980F0F");
 							gtk_list_store_set(GTK_LIST_STORE(model), &iter, i+1, text, -1);
							free(text);
						}
					}
				}
				gtk_tree_view_set_model(data_view, model);
			}
		}
 
		if(command){ // if command then create information
	 		message = append_string(NULL, "Information: Query OK, ");
	 		gchar * affrows = copy_string(PQcmdTuples(res));
	 	
	 		if(strcmp(affrows, "") == 0){
	 		 affrows = append_string(affrows, "0");
	 		}
			message = append_string(message, affrows);
			message = append_string(message, " rows affected.\n");
			free(affrows);
		}

		//show message
		if(mess_view == NULL){	// show alert
			if(information && !errors_only){ // informatin alert
			  message_dialog("Information: Executing SQL code", message, GTK_MESSAGE_INFO, NULL);
				PQclear(res);
				free(message);
				return TRUE;
			}
			else if(warning){ // warning alert
			  message_dialog("Warning: Executing SQL code", message, GTK_MESSAGE_WARNING, NULL);
				PQclear(res);
				free(message);
				return FALSE;
			}
			else if(error){ // error alert
			  message_dialog("Error: Executing SQL code", message, GTK_MESSAGE_ERROR, NULL);
				PQclear(res);
				return FALSE;
			}
			else return TRUE;
		}
		else{	// show message into given widget
			GtkTextBuffer * buffer;
			buffer = gtk_text_view_get_buffer (mess_view);
			GtkTextIter start;
 			gtk_text_buffer_get_start_iter(buffer, &start);
 			gtk_text_buffer_insert (buffer, &start, message, -1);
 			gtk_text_view_scroll_to_iter(mess_view, &start, 0.4, FALSE, 0, 0);
 			if(error || warning) end = FALSE;
		}

		PQclear(res);
	}while(!end && show_code);
	
	if(show_code){ // hide show sql dialog
		gtk_widget_hide(glade_xml_get_widget(xml, "show_SQL_dialog"));
	}
	
	if(SQL != NULL) free(SQL);
	
	return TRUE;
} // execute_sql ()

/*
 * Function that finds out whether the object with given OID is bool or not
 */
gboolean
is_bool(gchar * OID, T_Profile * profile){
	
	// currently not used function so it returns false
	return FALSE;
	
	// SQL query 
	gchar * sql=NULL;
	sql = append_string(sql, "select case when format_type(");
	sql = append_string(sql, OID);
	sql = append_string(sql, ", NULL) = 'boolean' then 'true' else 'false' end as bool;");
						           
	// Execution
	PGresult *res = PQexec(profile->conn, sql);
	free(sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  message_dialog("Error: SQL", "Error during retrieving data", GTK_MESSAGE_ERROR, NULL);
		PQclear(res);
		return FALSE;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "bool");
	guint row=0;
	gint rows = PQntuples(res);
	if(rows ==0 || rows >1){
	 PQclear(res);
	 return FALSE;
	}
	
	gboolean ret = (strcmp(PQgetvalue(res, row, col1), "true") == 0) ? TRUE : FALSE;
	PQclear(res);
	return ret;
} // is_bool ()

/*
 * Funciton that returns number of records in given table
 */
gint
get_ntuples(T_Table * table, T_Schema * schema, T_Profile * profile){
	gint n = 0;
	
	gchar * sql = append_string(NULL, "SELECT count(*) FROM \"");
	sql = append_string(sql, schema->name);
	sql = append_string(sql, "\".\"");
	sql = append_string(sql, table->name);
	sql = append_string(sql, "\"");
	if(table->data.condition != NULL){
		sql = append_string(sql, " WHERE ");
		sql = append_string(sql, table->data.condition);
	}	
	sql = append_string(sql, ";");
	
	// Execution
	PGresult *res = PQexec(profile->conn, sql);
	
	if (PQresultStatus(res) != PGRES_TUPLES_OK){
	  message_dialog("Error: SQL", "Error during retrieving data", GTK_MESSAGE_ERROR, NULL);
		PQclear(res);
		return 0;	                                                           
	}
	
	// Determine column nubers
	guint col1 = PQfnumber(res, "count");
	gint row = 0;
	gint rows = PQntuples(res);
	if(rows ==0 || rows >1) return 0;
	
	n = atoi(PQgetvalue(res, row, col1));
	
	free(sql);
	PQclear(res);
	
	return n;
} // get_ntuples ()

/*
 * Creates SQL query to create new database
 */
gchar *
create_new_database_SQL_query(T_Profile * profile){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	if(profile->dbname == NULL) return NULL;
	
	// Make header
	sql = append_string(sql, "-- Database: ");
	sql = append_string(sql, profile->dbname);
	
	sql = append_string(sql, "\n\n-- DROP DATABASE \"");
	sql = append_string(sql, profile->dbname);
	sql = append_string(sql, "\";\n\n");
	
	// CREATE DATABASE "dbname"
	sql = append_string(sql, "CREATE DATABASE \"");
	sql = append_string(sql, profile->dbname);
	sql = append_string(sql, "\"");
	
	// WITH\n  OWNER = "owner"
	if(strcmp(profile->database.owner, "") != 0){
		if(!pre) sql = append_string(sql, " WITH");
		sql = append_string(sql, "\n  OWNER = \"");
		sql = append_string(sql, profile->database.owner);
		sql = append_string(sql, "\"");
		pre = TRUE;
	}
	
	// ?WITH?\n  ENCODING = 'encoding'
	if(strcmp(profile->database.encoding, "") != 0){
		if(!pre) sql = append_string(sql, " WITH");
		sql = append_string(sql, "\n  ENCODING = \'");
		sql = append_string(sql, profile->database.encoding);
		sql = append_string(sql, "\'");
		pre = TRUE;
	}
	
	// ?WITH?\n  TEMPLATE = "template"
	if(strcmp(profile->database.template, "") != 0){
		if(!pre) sql = append_string(sql, " WITH");
		sql = append_string(sql, "\n  TEMPLATE = \"");
		sql = append_string(sql, profile->database.template);
		sql = append_string(sql, "\"");
		pre = TRUE;
	}
	
	// ?WITH?\n  TABLESPACE = tablespace
	if(strcmp(profile->database.tablespace, "") != 0){
		if(!pre) sql = append_string(sql, " WITH");
		sql = append_string(sql, "\n  TABLESPACE = \"");
		sql = append_string(sql, profile->database.tablespace);
		sql = append_string(sql, "\"");
		pre = TRUE;
	}
	sql = append_string(sql, ";\n");
	
	// COMMENT ON DATABASE db\n  IS 'comment';
	if(strcmp(profile->database.comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON DATABASE \"");
		sql = append_string(sql, profile->dbname);
		sql = append_string(sql, "\"\n  IS \'");
		sql = append_string(sql, profile->database.comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_new_database_SQL_query ()

/*
 * Creates SQL query to create new schema
 */
gchar *
create_new_schema_SQL_query(T_Schema * schema){
	gchar * sql = NULL;

	if(schema == NULL) return NULL;
	
	sql = append_string(sql, "-- Schema: ");
	sql = append_string(sql, schema->name);
	
	sql = append_string(sql, "\n\n-- DROP SCHEMA \"");
	sql = append_string(sql, schema->name);
	sql = append_string(sql, "\";\n\n");
	
	sql = append_string(sql, "CREATE SCHEMA \"");
	sql = append_string(sql, schema->name);
	sql = append_string(sql, "\"");
		
	if(strcmp(schema->owner, "") != 0){
		sql = append_string(sql, " ");
		sql = append_string(sql, "AUTHORIZATION \"");
		sql = append_string(sql, schema->owner);
		sql = append_string(sql, "\"");
	}	
	sql = append_string(sql, ";\n");
	
	if(strcmp(schema->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON SCHEMA \"");
		sql = append_string(sql, schema->name);
		sql = append_string(sql, "\"\n  IS \'");
		sql = append_string(sql, schema->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_new_schema_SQL_query ()

/*
 * Creates SQL query to create new table
 */
gchar *
create_new_table_SQL_query(T_Table * table, gchar * schema){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	if(table == NULL) return NULL;
	
	sql = append_string(sql, "-- Table: ");
	if(schema != NULL){
		sql = append_string(sql, schema);
		sql = append_string(sql, ".");
	}
	sql = append_string(sql, table->name);
	
	sql = append_string(sql, "\n\n-- DROP TABLE \"");
	if(schema != NULL){
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
	}
	sql = append_string(sql, table->name);
	sql = append_string(sql, "\";\n\n");
	
	sql = append_string(sql, "CREATE TABLE \"");
	if(schema != NULL){
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
	}
	sql = append_string(sql, table->name);
	sql = append_string(sql, "\" (\n");
	
	gboolean pre2 = FALSE;
	
	/* create subobjects */
	
	//fields
	gboolean first = TRUE;
	for(int i = 0; i < table->n_columns; i++){
		if(!table->column[i]->inherited){
			T_Column * field = table->column[i];
			if(pre2) sql = append_string(sql, ",\n");
			if(first){
			 	sql = append_string(sql, "  -- Fields\n");
				 first= FALSE;
			}
			sql = append_string(sql, "  \"");
			sql = append_string(sql, field->name);
			sql = append_string(sql, "\" ");
	
			gchar * type = copy_string(field->type);
			gint len = strlen(type);
			gboolean array = FALSE;
			if(len > 1 && type[len-2]=='[' && type[len-1]==']'){
					type[len-2] = '\0';
					array = TRUE;
			}
			sql = append_string(sql, type);
	
			if(field->resizeable && atoi(field->size) > 1){
					sql = append_string(sql, "(");
					sql = append_string(sql, field->size);
					sql = append_string(sql, ")");
			}
			if(array){
				for(int j=0; j<field->dimensions; j++) sql = append_string(sql, "[]");
				type[len-2] = '[';
			}
			free(type);
		
			if(field->not_null) sql = append_string(sql, " NOT NULL");
			if(strcmp(field->default_value, "") != 0){
				sql = append_string(sql, " DEFAULT ");
				sql = append_string(sql, field->default_value);
			}
	
			pre2 = TRUE;
		}
	}
	
	//unique and primary constraints
	first = TRUE;
	for(int i = 0; i < table->n_indexes; i++){
		if(table->index[i]->pkey_constraint || table->index[i]->unique_constraint){
			T_Index * index = table->index[i];
			if(pre2) sql = append_string(sql, ",\n");
			if(first){
			 	sql = append_string(sql, "  -- Keys\n");
				 first= FALSE;
			}
			sql = append_string(sql, "  CONSTRAINT \"");
			sql = append_string(sql, index->name);
		
			if(index->pkey_constraint) sql = append_string(sql, "\"\n    PRIMARY KEY (\"");
			else sql = append_string(sql, "\"\n    UNIQUE (\"");
	
			//columns
			for(int i = 0; i < index->n_columns; i++){
				sql = append_string(sql, index->column[i]->name);
				if(i != (index->n_columns-1)) sql = append_string(sql, "\", \"");
			}
			sql = append_string(sql, "\")");
			pre2 = TRUE;
		}
	}
	
	//foreign keys
	first = TRUE;
	for(int i = 0; i < table->n_fkeys; i++){
		T_FKey * fkey = table->fkey[i];
		if(pre2) sql = append_string(sql, ",\n");
		if(first){
		 	sql = append_string(sql, "  -- Foreign Keys\n");
			first= FALSE;
		}
		sql = append_string(sql, "  CONSTRAINT \"");
		sql = append_string(sql, fkey->name);
	
		sql = append_string(sql, "\"\n    FOREIGN KEY (\"");
	
		//columns
		for(int i = 0; i < fkey->n_columns; i++){
			sql = append_string(sql, fkey->column[i]->name);
			if(i != (fkey->n_columns-1)) sql = append_string(sql, "\", \"");
		}
	
	
		sql = append_string(sql, "\")\n");
	
		sql = append_string(sql, "    REFERENCES \"");
		sql = append_string(sql, fkey->ref_schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, fkey->ref_table);
		sql = append_string(sql, "\" (\"");
		
		// referenced columns
		for(int i = 0; i < fkey->n_ref_columns; i++){
			sql = append_string(sql, fkey->ref_column[i]->name);
			if(i != (fkey->n_ref_columns-1)) sql = append_string(sql, "\", \"");
		}
	
		sql = append_string(sql, "\")\n");
		sql = append_string(sql, "    ON DELETE ");
		gchar * s = copy_string(fkey->on_delete_rule);
		gint len = strlen(s);
	
		for(int i = 0; i < len; i++){
			s[i] = toupper(s[i]);
		}
		sql = append_string(sql, s);
		free(s);
		sql = append_string(sql, "\n");
	
		sql = append_string(sql, "    ON UPDATE ");
	
		s = copy_string(fkey->on_update_rule);
		len = strlen(s);
	
		for(int i = 0; i < len; i++){
			s[i] = toupper(s[i]);
		}
		sql = append_string(sql, s);
		free(s);
	
		if(fkey->deferrable){
			sql = append_string(sql, "\n    DEFERRABLE\n");
			if(strcmp(fkey->check_time, "At the end of the transaction") == 0) sql = append_string(sql, "    INITIALLY DEFERRED");
			else sql = append_string(sql, "    INITIALLY IMMEDIATE");
		}
		pre2 = TRUE;
	}
	
	// checks
	first = TRUE;
	for(int i = 0; i < table->n_checks; i++){
		T_Check * check = table->check[i];
		if(pre2) sql = append_string(sql, ",\n");
		if(first){
		 	sql = append_string(sql, "  -- Checks\n");
		  first= FALSE;
		}
		sql = append_string(sql, "  CONSTRAINT \"");
		sql = append_string(sql, check->name);
		
		sql = append_string(sql, "\"\n    CHECK (");
		sql = append_string(sql, check->condition);
		sql = append_string(sql, ")");
		pre2 = TRUE;
	}
	
	
	sql = append_string(sql, "\n)");
	
	if(strcmp(table->inherited_from, "") != 0){
		if(pre) sql = append_string(sql, "\n ");
		sql = append_string(sql, " INHERITS (");
		sql = append_string(sql, table->inherited_from);
		sql = append_string(sql, ")");
		pre = TRUE;
	}
	
	if(pre) sql = append_string(sql, "\n ");
	if(table->WithOIDs) sql = append_string(sql, " WITH OIDS");
	else sql = append_string(sql, " WITHOUT OIDS");
	
	pre = TRUE; 
	
	if(strcmp(table->tablespace, "") != 0){
		if(pre) sql = append_string(sql, "\n ");
		sql = append_string(sql, " TABLESPACE \"");
		sql = append_string(sql, table->tablespace);
		sql = append_string(sql, "\"");
		pre = TRUE;
	}
	
	sql = append_string(sql, ";\n");
	
	// clusters of index
	for(int i = 0; i < table->n_indexes; i++){
		if((table->index[i]->pkey_constraint || table->index[i]->unique_constraint) && table->index[i]->clustered){
			T_Index * index = table->index[i];
			sql = append_string(sql, "\nALTER TABLE \"");
			if(schema != NULL){
				sql = append_string(sql, schema);
				sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\"\n  CLUSTER ON \"");
			sql = append_string(sql, index->name);
			sql = append_string(sql, "\";\n");	
		}
	}	
	
	// indexes
	for(int i = 0; i < table->n_indexes; i++){
		if(!(table->index[i]->pkey_constraint || table->index[i]->unique_constraint)){
			T_Index * index = table->index[i];
			gchar * parent = NULL;
			if(schema != NULL){
				parent = append_string(parent, "\"");			
				parent = append_string(parent, schema);
				parent = append_string(parent, "\".");
			}
			parent = append_string(parent, "\"");
			parent = append_string(parent, table->name);
			parent = append_string(parent, "\"");
			gchar * s = create_new_index_SQL_query(index, parent, FALSE);
			sql = append_string(sql, "\n");
			sql = append_string(sql, s);
			free(s); free(parent);
		}
	}	
	
	// triggers
	for(int i = 0; i < table->n_triggers; i++){
		T_Trigger * trigger = table->trigger[i];
		gchar * parent = NULL;
		if(schema != NULL){
			parent = append_string(parent, "\"");			
			parent = append_string(parent, schema);
			parent = append_string(parent, "\".");
		}
		parent = append_string(parent, "\"");			
		parent = append_string(parent, table->name);
		parent = append_string(parent, "\"");			
		gchar * s = create_new_trigger_SQL_query(trigger, parent, FALSE, FALSE);
		sql = append_string(sql, "\n");
		sql = append_string(sql, s);
		free(s); free(parent);
	}	
	
	// rules
	for(int i = 0; i < table->n_rules; i++){
		T_Rule * rule = table->rule[i];
		gchar * parent = NULL;
		if(schema != NULL){
			parent = append_string(parent, "\"");			
			parent = append_string(parent, schema);
			parent = append_string(parent, "\".");
		}
		parent = append_string(parent, "\"");			
		parent = append_string(parent, table->name);
		parent = append_string(parent, "\"");			
		gchar * s = create_new_rule_SQL_query(rule, parent, FALSE, FALSE);
		sql = append_string(sql, "\n");
		sql = append_string(sql, s);
		free(s); free(parent);
	}	
	
	// change owner
	if(strcmp(table->owner, "") != 0){
		sql = append_string(sql, "\nALTER TABLE \"");
		if(schema != NULL){
			sql = append_string(sql, schema);
			sql = append_string(sql, "\".\"");
		}
		sql = append_string(sql, table->name);
		sql = append_string(sql, "\"\n  OWNER TO \"");
		sql = append_string(sql, table->owner);
		sql = append_string(sql, "\";\n");
	}
	
	// comment on table
	if(strcmp(table->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON TABLE \"");
		if(schema != NULL){
			sql = append_string(sql, schema);
			sql = append_string(sql, "\".\"");
		}
		sql = append_string(sql, table->name);
		sql = append_string(sql, "\"\n  IS \'");
		sql = append_string(sql, table->comment);
		sql = append_string(sql, "\';\n");
	}
	
	// comments fields
	for(int i = 0; i < table->n_columns; i++){
		T_Column * field = table->column[i];
		if(strcmp(field->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON COLUMN \"");
			if(schema != NULL){
				sql = append_string(sql, schema);
				sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\".\"");
			sql = append_string(sql, field->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, field->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	// comments indexes
	for(int i = 0; i < table->n_indexes; i++){
		T_Index * index = table->index[i];
		if(strcmp(index->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON INDEX \"");
			sql = append_string(sql, index->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, index->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	// comments fkeys
	for(int i = 0; i < table->n_fkeys; i++){
		T_FKey * fkey = table->fkey[i];
		if(strcmp(fkey->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON CONSTRAINT \"");
			sql = append_string(sql, fkey->name);
			sql = append_string(sql, "\"\n  ON \"");
			if(schema != NULL){
				sql = append_string(sql, schema);
				sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, fkey->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	// comments check
	for(int i = 0; i < table->n_checks; i++){
		T_Check * check = table->check[i];
		if(strcmp(check->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON CONSTRAINT \"");
			sql = append_string(sql, check->name);
			sql = append_string(sql, "\"\n  ON \"");
			if(schema != NULL){
				sql = append_string(sql, schema);
				sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, check->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	// comments triggers
	for(int i = 0; i < table->n_triggers; i++){
		T_Trigger * trigger = table->trigger[i];
		if(strcmp(trigger->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON TRIGGER \"");
			sql = append_string(sql, trigger->name);
			sql = append_string(sql, "\"\n  ON \"");
			if(schema != NULL){
					sql = append_string(sql, schema);
					sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, trigger->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	// comments rules
	for(int i = 0; i < table->n_rules; i++){
		T_Rule * rule = table->rule[i];
		if(strcmp(rule->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON RULE \"");
			sql = append_string(sql, rule->name);
			sql = append_string(sql, "\"\n  ON \"");
			if(schema != NULL){
					sql = append_string(sql, schema);
					sql = append_string(sql, "\".\"");
			}
			sql = append_string(sql, table->name);
			sql = append_string(sql, "\"\n");	
			sql = append_string(sql, "  IS \'");
			sql = append_string(sql, rule->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	
	return sql;
} // create_new_table_SQL_query ()

/*
 * Creates SQL query to create new trigger
 */
gchar *
create_new_trigger_SQL_query(T_Trigger * trigger, gchar * parent_name, gboolean with_comment, gboolean with_header){
	gchar * sql = NULL;
	
	// parse parent_name
	gint k = 0;
	while(k < strlen(parent_name) && parent_name[k] != '.') k++;
	
	gchar * parent = NULL;
	if(k < (strlen(parent_name) - 1) && k > 0){	 // found dot
		if(parent_name[k-1] != '\"' || parent_name[k+1] != '\"'){ // have to add "."
			parent_name[k]= '\0';
			parent = append_string(parent, parent_name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, (parent_name+k+1));
			parent_name[k]='.';			
		}
		else parent = copy_string(parent_name);
	}
	if(parent == NULL) return NULL;
	
	// create header
	if(with_header){
		sql = append_string(sql, "-- Trigger: ");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, ".");
		sql = append_string(sql, trigger->name);
	
		sql = append_string(sql, "\n\n/*\nDROP TRIGGER \"");
		sql = append_string(sql, trigger->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent);
		sql = append_string(sql, "\";\n*/\n\n");
	}
	
	// create trigger
	sql = append_string(sql, "CREATE TRIGGER \"");
	sql = append_string(sql, trigger->name);
	sql = append_string(sql, "\"\n");
	
	if(strcmp(trigger->type, "After") == 0) sql = append_string(sql, "  AFTER ");
	else sql = append_string(sql, "  BEFORE ");
	
	gchar * events = copy_string(trigger->events);
	gint len = strlen(events);
	
	for(int i = 0; i < len; i++){
		events[i] = toupper(events[i]);
	}
	sql = append_string(sql, events);
	free(events);
	
	sql = append_string(sql, "\n");
	
	sql = append_string(sql, "  ON \"");
	sql = append_string(sql, parent);
	sql = append_string(sql, "\"\n");	
	
	if(strcmp(trigger->for_each,"Row")==0) sql = append_string(sql, "  FOR EACH ROW\n");
	else sql = append_string(sql, "  FOR EACH STATEMENT\n");
	
	// parse function
	k = 0;
	while(k < strlen(trigger->function) && trigger->function[k] != '.') k++;
	
	gchar * function = NULL;
	if(k < (strlen(trigger->function) - 1) && k > 0){	 // found dot
		if(trigger->function[k-1] != '\"' || trigger->function[k+1] != '\"'){ // have to add "."
			trigger->function[k]= '\0';
			function = append_string(function, trigger->function);
			function = append_string(function, "\".\"");
			function = append_string(function, (trigger->function+k+1));
			trigger->function[k]='.';			
		}
		else function = copy_string(trigger->function);
	}
	if(function == NULL) return NULL;
	
	sql = append_string(sql, "  EXECUTE PROCEDURE \"");
	sql = append_string(sql, function);
	sql = append_string(sql, "\"(");
	sql = append_string(sql, trigger->arguments);
	sql = append_string(sql, ");\n");
	
	// comment on trigger
	if(with_comment){
	if(strcmp(trigger->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON TRIGGER \"");
		sql = append_string(sql, trigger->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, trigger->comment);
		sql = append_string(sql, "\';\n");
	}
	}
	return sql;
} // create_new_trigger_SQL_query ()

/*
 * Creates SQL query to create new rule
 */
gchar *
create_new_rule_SQL_query(T_Rule * rule, gchar * parent_name, gboolean with_comment, gboolean with_header){
	gchar * sql = NULL;
	
	// parse parent_name
	gint k = 0;
	while(k < strlen(parent_name) && parent_name[k] != '.') k++;
	
	gchar * parent = NULL;
	if(k < (strlen(parent_name) - 1) && k > 0){	 // found dot
		if(parent_name[k-1] != '\"' || parent_name[k+1] != '\"'){ // have to add "."
			parent_name[k]= '\0';
			parent = append_string(parent, parent_name);
			parent = append_string(parent, "\".\"");
			parent = append_string(parent, (parent_name+k+1));
			parent_name[k]='.';			
		}
		else parent = copy_string(parent_name);
	}
	if(parent == NULL) return NULL;
	
	// create header
	if(with_header){
		sql = append_string(sql, "-- Rule: ");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, ".");
		sql = append_string(sql, rule->name);
	
		sql = append_string(sql, "\n\n/*\nDROP RULE \"");
		sql = append_string(sql, rule->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent);
		sql = append_string(sql, "\";\n*/\n\n");
	}
	
	// create rule
	sql = append_string(sql, "CREATE OR REPLACE RULE \"");
	sql = append_string(sql, rule->name);
	sql = append_string(sql, "\" AS\n");
	sql = append_string(sql, "  ON ");
	
	gchar * event = copy_string(rule->event);
	gint len = strlen(event);
	
	for(int i = 0; i < len; i++){
		event[i] = toupper(event[i]);
	}
	sql = append_string(sql, event);
	free(event);
	
	sql = append_string(sql, " TO \"");
	sql = append_string(sql, parent);
	sql = append_string(sql, "\"\n  DO ");
	
	if(rule->instead) sql = append_string(sql, "INSTEAD ");
	
	sql = append_string(sql, "");
	sql = append_string(sql, rule->definition);
	sql = append_string(sql, ";\n");
	
	// comment on rule
	if(with_comment){
	if(strcmp(rule->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON RULE \"");
		sql = append_string(sql, rule->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, rule->comment);
		sql = append_string(sql, "\';\n");
	}
	}
	free(parent);
	return sql;
} // create_new_rule_SQL_query ()

/*
 * Creates SQL query to create new field
 */
gchar *
create_new_field_SQL_query(T_Column * field, gchar * parent_name, gboolean with_comment){
	gchar * sql = NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n  ADD COLUMN \"");
	sql = append_string(sql, field->name);
	sql = append_string(sql, "\" ");
	
	gchar * type = copy_string(field->type);
	gint len = strlen(type);
	gboolean array = FALSE;
	if(len > 1 && type[len-2]=='[' && type[len-1]==']'){
			type[len-2] = '\0';
			array = TRUE;
	}
	sql = append_string(sql, type);
	
	if(field->resizeable && atoi(field->size) > 1){
			sql = append_string(sql, "(");
			sql = append_string(sql, field->size);
			sql = append_string(sql, ")");
	}
	if(array){
		for(int j=0; j<field->dimensions; j++) sql = append_string(sql, "[]");
		type[len-2] = '[';
	}
	free(type);
		
	if(field->not_null) sql = append_string(sql, " NOT NULL");
	if(strcmp(field->default_value, "") != 0){
		sql = append_string(sql, " DEFAULT ");
		sql = append_string(sql, field->default_value);
	}
	
	sql = append_string(sql, ";\n");


	if(field->pkey){ // new field is primary key, have to create primary key constraint
		sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
	  sql = append_string(sql, parent_name);
	  sql = append_string(sql, "\"\n  ADD CONSTRAINT \"pk_");
	  sql = append_string(sql, field->name);
	  sql = append_string(sql, "\" PRIMARY KEY (\"");
	  sql = append_string(sql, field->name);
	  sql = append_string(sql, "\");\n");
	}
	
	if(field->unique && !field->pkey){ // new field is unique, have to create unique constraint
		 	sql = append_string(sql, "\n");
			sql = append_string(sql, "ALTER TABLE \"");
		  sql = append_string(sql, parent_name);
		  sql = append_string(sql, "\"\n  ADD CONSTRAINT \"uk_");
		  sql = append_string(sql, field->name);
		  sql = append_string(sql, "\" UNIQUE (\"");
		  sql = append_string(sql, field->name);
		  sql = append_string(sql, "\");\n");
	}
	
	// comment on field	
	if(strcmp(field->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON COLUMN \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, field->name);
		sql = append_string(sql, "\"\n  IS \'");
		sql = append_string(sql, field->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_new_field_SQL_query ()

/*
 * Creates SQL query to create new index
 */
gchar *
create_new_index_SQL_query(T_Index * index, gchar * parent_name, gboolean with_comment){
	gchar * sql = NULL;
	
	if(index->pkey_constraint || index->unique_constraint){
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  ADD CONSTRAINT \"");
		sql = append_string(sql, index->name);
		
		if(index->pkey_constraint) sql = append_string(sql, "\"\n  PRIMARY KEY (\"");
		else sql = append_string(sql, "\"\n  UNIQUE (\"");
	
		//columns
		for(int i = 0; i < index->n_columns; i++){
			sql = append_string(sql, index->column[i]->name);
			if(i != (index->n_columns-1)) sql = append_string(sql, "\", \"");
		}
		sql = append_string(sql, "\")");
		
		if(strcmp(index->tablespace, "") != 0){	
			sql = append_string(sql, "\n    USING INDEX TABLESPACE \"");
			sql = append_string(sql, index->tablespace);	
			sql = append_string(sql, "\"");			
		}
		
		sql = append_string(sql, ";\n");
	}
	else{
		if(index->unique_index)	sql = append_string(sql, "CREATE UNIQUE INDEX \"");
		else sql = append_string(sql, "CREATE INDEX \"");
		sql = append_string(sql, index->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"");
		
		if(strcmp(index->access_method, "") != 0){
			sql = append_string(sql, "\n  USING ");
			sql = append_string(sql, index->access_method);
		}
		
		//columns
		sql = append_string(sql, "\n  (\"");
		for(int i = 0; i < index->n_columns; i++){
			sql = append_string(sql, index->column[i]->name);
			if(i != (index->n_columns-1)) sql = append_string(sql, "\", \"");
		}
		sql = append_string(sql, "\")");
		
		if(strcmp(index->tablespace, "") != 0){
			sql = append_string(sql, "\n  TABLESPACE \"");
			sql = append_string(sql, index->tablespace);
			sql = append_string(sql, "\"");
		}
		
		if(strcmp(index->predicate, "") != 0){
			sql = append_string(sql, "\n  WHERE ");
			sql = append_string(sql, index->predicate);
		}
		
		sql = append_string(sql, ";\n");
	}
	
	if(index->clustered){ // create cluster
		sql = append_string(sql, "\nALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  CLUSTER ON \"");
		sql = append_string(sql, index->name);
		sql = append_string(sql, "\";\n");
	}
	
	// comment on index
	if(with_comment){
		if(strcmp(index->comment, "") != 0){
			sql = append_string(sql, "\nCOMMENT ON INDEX \"");
			sql = append_string(sql, index->name);
			sql = append_string(sql, "\"\n  IS \'");
			sql = append_string(sql, index->comment);
			sql = append_string(sql, "\';\n");
		}
	}
	return sql;
} // create_new_index_SQL_query ()

/*
 * Creates SQL query to create new foreign key
 */
gchar *
create_new_fkey_SQL_query(T_FKey * fkey, gchar * parent_name, gboolean with_comment){
	gchar * sql = NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n  ADD CONSTRAINT \"");
	sql = append_string(sql, fkey->name);
	
	sql = append_string(sql, "\"\n  FOREIGN KEY (\"");
	
	//columns
	for(int i = 0; i < fkey->n_columns; i++){
		sql = append_string(sql, fkey->column[i]->name);
		if(i != (fkey->n_columns-1)) sql = append_string(sql, "\", \"");
	}
	
	
	sql = append_string(sql, "\")\n");
	
	sql = append_string(sql, "    REFERENCES \"");
	sql = append_string(sql, fkey->ref_schema);
	sql = append_string(sql, "\".\"");
	sql = append_string(sql, fkey->ref_table);
	sql = append_string(sql, "\" (\"");
	// referenced columns
	for(int i = 0; i < fkey->n_ref_columns; i++){
		sql = append_string(sql, fkey->ref_column[i]->name);
		if(i != (fkey->n_ref_columns-1)) sql = append_string(sql, "\", \"");
	}
	
	sql = append_string(sql, "\")\n");
	sql = append_string(sql, "    ON DELETE ");
	gchar * s = copy_string(fkey->on_delete_rule);
	gint len = strlen(s);
	
	for(int i = 0; i < len; i++){
		s[i] = toupper(s[i]);
	}
	sql = append_string(sql, s);
	free(s);
	sql = append_string(sql, "\n");
	
	sql = append_string(sql, "    ON UPDATE ");
	
	s = copy_string(fkey->on_update_rule);
	len = strlen(s);
	
	for(int i = 0; i < len; i++){
		s[i] = toupper(s[i]);
	}
	sql = append_string(sql, s);
	free(s);
	
	if(fkey->deferrable){
		sql = append_string(sql, "\n    DEFERRABLE\n");
		if(strcmp(fkey->check_time, "At the end of the transaction") == 0) sql = append_string(sql, "    INITIALLY DEFERRED");
		else sql = append_string(sql, "    INITIALLY IMMEDIATE");
	}
	
	sql = append_string(sql, ";\n");
	
	// comment on foreign key
	if(strcmp(fkey->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON CONSTRAINT \"");
		sql = append_string(sql, fkey->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, fkey->comment);
		sql = append_string(sql, "\';\n");
	}
		
	return sql;
} // create_new_fkey_SQL_query ()

/*
 * Creates SQL query to create new check
 */
gchar *
create_new_check_SQL_query(T_Check * check, gchar * parent_name, gboolean with_comment){
	gchar * sql = NULL;
	
	sql = append_string(sql, "ALTER TABLE \"");
	sql = append_string(sql, parent_name);
	sql = append_string(sql, "\"\n  ADD CONSTRAINT \"");
	sql = append_string(sql, check->name);
	
	sql = append_string(sql, "\"\n  CHECK (");
	sql = append_string(sql, check->condition);
	sql = append_string(sql, ");\n");
	
	if(strcmp(check->comment, "") != 0){
		sql = append_string(sql, "\nCOMMENT ON CONSTRAINT \"");
		sql = append_string(sql, check->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, check->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_new_check_SQL_query ()
