/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "db_connection.h"
#include "../functions.h"

/*
 * Makes from given strings information needed for connecting to PostgreSQL database
 */
char *
make_conninfo            (const char * host,
																			const char * port,
                                      const char * dbname,
                                      const char * user,
                                      const char * password,
																			guint ssl_int)
{
	int n = 0, k;
	gboolean hname = FALSE;
	
	gchar * SSL=NULL;
	gchar ssl_disable[] = "disable";
	gchar ssl_allow[] = "allow";
	gchar ssl_prefer[] = "prefer";
	gchar ssl_require[] = "require";
	
	switch(ssl_int){
		case 0: SSL =ssl_disable; break;
		case 1: SSL =ssl_allow; break;
		case 2: SSL =ssl_prefer; break;
		case 3: SSL =ssl_require; break;
	}
				
	if(host != NULL && (k = strlen(host)) != 0){
		if((hname = parse_host(host))){ // host is NOT IP addres
			 n += strlen("host=") + strlen(host);
		}
		else {
			n += strlen("hostaddr=") + strlen(host);
		}
	}
	
	if(port != NULL && (k = strlen(port)) != 0) n += strlen(" port=") + strlen(port);
	if(dbname != NULL && (k = strlen(dbname)) != 0) n += strlen(" dbname=") + strlen(dbname);
	if(user != NULL && (k = strlen(user)) != 0) n += strlen(" user=") + strlen(user);
	if(password != NULL && (k = strlen(password)) != 0) n += strlen(" password=") + strlen(password);
	if(SSL != NULL && (k = strlen(SSL)) != 0) n += strlen(" sslmode=") + strlen(SSL);

	char *s = alloc_string(n+1);
	s[0]='\0';

	if(host != NULL && (k = strlen(host)) != 0){
		if(hname){ // host is NOT IP addres
			 strcpy(s, "host="); strcat(s, host);
		}
		else {
			 strcpy(s, "hostaddr="); strcat(s, host);
		}
	}

	if(port != NULL && (k = strlen(port)) != 0) {strcat(s, " port=");	strcat(s, port);}
	if(dbname != NULL  && (k = strlen(dbname)) != 0)  {strcat(s, " dbname=");	strcat(s, dbname);}
	if(user != NULL && (k = strlen(user)) != 0) {strcat(s, " user=");	strcat(s, user);}
	if(password != NULL && (k = strlen(password)) != 0) {strcat(s, " password=");	strcat(s, password);}
	if(SSL != NULL && (k = strlen(SSL)) != 0) {strcat(s, " sslmode=");	strcat(s, SSL);}
	
	return s;
} // make_conninfo()

/*
 * Connects to PostgreSQL database and check connections
 */
PGconn *
connect_db (const char * conninfo)
{
	PGconn * connection;
	GtkWidget * mess;
	
	
	// Trying to connect to database
	connection = PQconnectdb(conninfo);

	// Check connection status
	if (PQstatus(connection) != CONNECTION_OK)
	{
		// Error occured during connection
		mess = gtk_message_dialog_new(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")),
                                                                             GTK_DIALOG_MODAL,
                                                                             GTK_MESSAGE_WARNING,
                                                                             GTK_BUTTONS_OK,
                                                                             "Connection failed.");
		gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG (mess),"%s", PQerrorMessage(connection));
		gint res = gtk_dialog_run(GTK_DIALOG(mess));
		if(res == GTK_RESPONSE_OK);
		gtk_widget_destroy (mess);	
		
		return NULL;
	}
		
	// Connection OK
	return connection;

} // connect_db()

/*
 * Disconnects from PostgreSQL database.
 */
void
disconnect_db             (PGconn * conn)
{
	PQfinish(conn);
} // disconnect_db()
