/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "edit_db_objects.h"
#include "create_db_objects.h"
#include "drop_db_objects.h"
#include "get_db_objects.h"

/*
 * Creates SQL query to edit database
 */
gchar *
create_edit_database_SQL_query(T_Profile * old_profile, T_Profile * new_profile){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	if(old_profile == NULL || new_profile == NULL) return NULL;
	
	// owner
	if(strcmp(old_profile->database.owner, new_profile->database.owner) != 0){
		sql = append_string(sql, "ALTER DATABASE \"");
		sql = append_string(sql, old_profile->dbname);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "  OWNER TO \"");
		sql = append_string(sql, new_profile->database.owner);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
	
	// comment
	if(strcmp(old_profile->database.comment, new_profile->database.comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON DATABASE \"");
		sql = append_string(sql, old_profile->dbname);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_profile->database.comment);
		sql = append_string(sql, "\';\n");
		pre = TRUE;
	}
	
	// connection limit
	if(old_profile->database.connection_limit != new_profile->database.connection_limit){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER DATABASE \"");
		sql = append_string(sql, old_profile->dbname);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "CONNECTION LIMIT ");
		gchar * limit = itoa_n(new_profile->database.connection_limit, 0);
		sql = append_string(sql, limit);
		free(limit);
		sql = append_string(sql, ";\n");
		pre = TRUE;
	}
	
	return sql;
} // create_edit_database_SQL_query ()

/*
 * Creates SQL query to edit schema
 */
gchar *
create_edit_schema_SQL_query(T_Schema * old_schema, T_Schema * new_schema){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	gboolean name_changed = FALSE;
	
	if(old_schema == NULL || new_schema == NULL) return NULL;
	
	// name
	if(strcmp(old_schema->name, new_schema->name) != 0){
		sql = append_string(sql, "ALTER SCHEMA \"");
		sql = append_string(sql, old_schema->name);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "  RENAME TO \"");
		sql = append_string(sql, new_schema->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
		name_changed = TRUE;
	}

	// owner
	if(strcmp(old_schema->owner, new_schema->owner) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER SCHEMA ");
		sql = append_string(sql, "\"");
		sql = append_string(sql, new_schema->name);
		sql = append_string(sql, "\"");
		sql = append_string(sql, "\n");
		sql = append_string(sql, "  OWNER TO \"");
		sql = append_string(sql, new_schema->owner);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
		
	// comment
	if(strcmp(old_schema->comment, new_schema->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON SCHEMA \"");
		sql = append_string(sql, "\"");
		sql = append_string(sql, new_schema->name);
		sql = append_string(sql, "\"");
		sql = append_string(sql, "\n");
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_schema->comment);
		sql = append_string(sql, "\';\n");
		pre = TRUE;
	}
	
	return sql;
} // create_edit_schema_SQL_query ()

/*
 * Creates SQL query to edit table
 */
gchar *
create_edit_table_SQL_query(T_Table * old_table, T_Table * new_table, gchar * schema){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	if(old_table == NULL || new_table == NULL) return NULL;
	
	// name
	if(strcmp(old_table->name, new_table->name) != 0){
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, old_table->name);
		sql = append_string(sql, "\"\n");
		sql = append_string(sql, "  RENAME TO \"");
		sql = append_string(sql, new_table->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}

	// owner
	if(strcmp(old_table->owner, new_table->owner) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, new_table->name);
		sql = append_string(sql, "\"");
		sql = append_string(sql, "\n");
		sql = append_string(sql, "  OWNER TO ");
		sql = append_string(sql, new_table->owner);
		sql = append_string(sql, ";\n");
		pre = TRUE;
	}
		
	// comment
	if(strcmp(old_table->comment, new_table->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON TABLE \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, new_table->name);
		sql = append_string(sql, "\"");
		sql = append_string(sql, "\n");
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_table->comment);
		sql = append_string(sql, "\';\n");
		pre = TRUE;
	}
	
	// tablespace
	if(strcmp(old_table->tablespace, new_table->tablespace) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, new_table->name);
		sql = append_string(sql, "\"");
		sql = append_string(sql, "\n");
		sql = append_string(sql, "  SET TABLESPACE \"");
		sql = append_string(sql, new_table->tablespace);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
	
	return sql;
} // create_edit_table_SQL_query ()

/*
 *Creates SQL query to edit trigger
 */
gchar *
create_edit_trigger_SQL_query(T_Trigger * old_trigger, T_Trigger * new_trigger, gchar * parent_name){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	// change name
	if(strcmp(old_trigger->name, new_trigger->name) != 0){
		sql = append_string(sql, "ALTER TRIGGER \"");
		sql = append_string(sql, old_trigger->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  RENAME TO \"");
		sql = append_string(sql, new_trigger->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
	
	// change type, for each, events, function or arguments
	if(strcmp(old_trigger->type, new_trigger->type) != 0 || strcmp(old_trigger->for_each, new_trigger->for_each) != 0 || 
	    strcmp(old_trigger->events, new_trigger->events) != 0 || strcmp(old_trigger->function, new_trigger->function) != 0 ||
	    strcmp(old_trigger->arguments, new_trigger->arguments) != 0){
			if(pre) sql = append_string(sql, "\n");
			// drop old
			gchar * sql1 = create_drop_trigger_SQL_query(old_trigger, parent_name, FALSE);
			sql = append_string(sql, sql1);
			free(sql1);
			
			sql = append_string(sql, "\n");
			// create new
			gchar * sql2 = create_new_trigger_SQL_query(new_trigger, parent_name, FALSE, FALSE);
			sql = append_string(sql, sql2);
			free(sql2);
			pre = TRUE;
	}
	
	// change enabled
	if( old_trigger->enabled != new_trigger->enabled){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		if(new_trigger->enabled)	sql = append_string(sql, "\"n  ENABLE TRIGGER \"");
		else sql = append_string(sql, "\"\n  DISABLE TRIGGER \"");
		sql = append_string(sql, new_trigger->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
	
	// change comment
	if(strcmp(old_trigger->comment, new_trigger->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "\nCOMMENT ON TRIGGER \"");
		sql = append_string(sql, new_trigger->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_trigger->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_edit_trigger_SQL_query()

/*
 * Creates SQL query to edit rule
 */
gchar *
create_edit_rule_SQL_query(T_Rule * old_rule, T_Rule * new_rule, gchar * parent_name){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	if(strcmp(old_rule->event, new_rule->event) != 0 || strcmp(old_rule->definition, new_rule->definition) != 0 || old_rule->instead != new_rule->instead || strcmp(old_rule->condition, new_rule->condition) != 0){
		gchar * sql1 = create_new_rule_SQL_query(new_rule, parent_name, FALSE, FALSE);
		sql = append_string(sql, sql1);
		free(sql1);
		pre = TRUE;
	}
	
	if(strcmp(old_rule->comment, new_rule->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON RULE \"");
		sql = append_string(sql, new_rule->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_rule->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_edit_rule_SQL_query()

/*
 * Creates SQL query to edit field
 */
gchar *
create_edit_field_SQL_query(T_Table * table, T_Column * old_field, T_Column * new_field, gchar * parent_name){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	// change name
	if(strcmp(old_field->name, new_field->name) != 0){
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  RENAME COLUMN \"");
		sql = append_string(sql, old_field->name);
		sql = append_string(sql, "\" TO \"");
		sql = append_string(sql, new_field->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
	
	// type or size or dimensions
	gboolean array = FALSE;
	gchar * type = copy_string(new_field->type);
	gint len = strlen(type);
	if(len > 1 && new_field->type[len-2]=='[' && new_field->type[len-1]==']'){
		type[len-2] = '\0';
		array = TRUE;
	}
	if(strcmp(old_field->type, new_field->type) != 0 || (new_field->resizeable && strcmp(old_field->size, new_field->size)) != 0 || (array && old_field->dimensions != new_field->dimensions)){
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  ALTER COLUMN \"");
		sql = append_string(sql, old_field->name);
		sql = append_string(sql, "\"\n  TYPE ");
		
		
		sql = append_string(sql, type);
	
		if(new_field->resizeable && atoi(new_field->size) > 1){
			sql = append_string(sql, "(");
			sql = append_string(sql, new_field->size);
			sql = append_string(sql, ")");
		}
		if(array){
			for(int j=0; j< new_field->dimensions; j++) sql = append_string(sql, "[]");
			type[len-2] = '[';
		}
		sql = append_string(sql, ";\n");
		pre = TRUE;
	}
	free(type);
	
	// change primary key, uniqe or not null
	if(old_field->pkey != new_field->pkey){
		if(new_field->pkey){ // new field is primary key, have to create primary key constraint
			if(pre) sql = append_string(sql, "\n");
			sql = append_string(sql, "ALTER TABLE \"");
		  sql = append_string(sql, parent_name);
		  sql = append_string(sql, "\"\n  ADD CONSTRAINT \"pk_");
		  sql = append_string(sql, new_field->name);
		  sql = append_string(sql, "\" PRIMARY KEY (\"");
		  sql = append_string(sql, new_field->name);
		  sql = append_string(sql, "\");\n");
		  pre = TRUE;
		}
		else{ // drop existing primary key constraint
				for(int i = 0; i < table->n_indexes; i++){
					if(table->index[i]->pkey_constraint){
						for(int j = 0; j < table->index[i]->n_columns; j++){
							if(strcmp(table->index[i]->column[j]->name, old_field->name) == 0){ // found index for this column
								if(pre) sql = append_string(sql, "\n");
								sql = append_string(sql, "ALTER TABLE \"");
		            sql = append_string(sql, parent_name);
		            sql = append_string(sql, "\"\n  DROP CONSTRAINT \"");
		            sql = append_string(sql, table->index[i]->name);
		            sql = append_string(sql, "\";\n");
		            pre = TRUE;
		            break;
							}
						}
					}
				}
		}
	}
	
	if(old_field->unique != new_field->unique && !new_field->pkey){
		if(new_field->unique){
			if(pre) sql = append_string(sql, "\n");
			sql = append_string(sql, "ALTER TABLE \"");
		  sql = append_string(sql, parent_name);
		  sql = append_string(sql, "\"\n  ADD CONSTRAINT \"uk_");
		  sql = append_string(sql, new_field->name);
		  sql = append_string(sql, "\" UNIQUE (\"");
		  sql = append_string(sql, new_field->name);
		  sql = append_string(sql, "\");\n");
		  pre = TRUE;
		}
		else{
				for(int i = 0; i < table->n_indexes; i++){
					if(table->index[i]->unique_constraint){
						for(int j = 0; j < table->index[i]->n_columns; j++){
							if(strcmp(table->index[i]->column[j]->name, old_field->name) == 0){ // found index for this column
								if(pre) sql = append_string(sql, "\n");
								sql = append_string(sql, "ALTER TABLE \"");
		            sql = append_string(sql, parent_name);
		            sql = append_string(sql, "\"\n  DROP CONSTRAINT \"");
		            sql = append_string(sql, table->index[i]->name);
		            sql = append_string(sql, "\";\n");
		            pre = TRUE;
		            break;
							}
						}
					}
				}
		}
	}
	
	if(old_field->not_null != new_field->not_null && !new_field->pkey){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  ALTER COLUMN \"");
		sql = append_string(sql, old_field->name);
		if(new_field->not_null)	sql = append_string(sql, "\"\n  SET NOT NULL;\n");
		else sql = append_string(sql, "\"\n  DROP NOT NULL;\n");

		pre = TRUE;
	}
	
	// change default
	if(strcmp(old_field->default_value, new_field->default_value) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n  ALTER COLUMN \"");
		sql = append_string(sql, old_field->name);
		sql = append_string(sql, "\"\n  SET DEFAULT '");
		sql = append_string(sql, new_field->default_value);
		sql = append_string(sql, "';\n");
		pre = TRUE;
	}
	
	// change comment
	if(strcmp(old_field->comment, new_field->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "\nCOMMENT ON COLUMN \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, new_field->name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_field->comment);
		sql = append_string(sql, "\';\n");
	}
	
	return sql;
} // create_edit_field_SQL_query()

/*
 * Creates SQL query to edit index
 */
gchar *
create_edit_index_SQL_query(T_Index * old_index, T_Index * new_index, gchar * parent_name){
	gchar * sql = NULL;
	gchar * schema = NULL;
	gboolean pre = FALSE;
	
	//get schema name
	gint i = 0;
	while(i < strlen(parent_name) && parent_name[i] != '.') i++;
	parent_name[i] = '\0';
	schema = copy_string(parent_name);
	parent_name[i] = '.';
	
	// change name, columns, primary key contraint, unique constraint, unique index, predicate, tablespace, access_method
	gchar * old_columns = NULL;
	old_columns = append_string(old_columns, "\"");
	for(int i = 0; i < old_index->n_columns; i++){
		old_columns = append_string(old_columns, old_index->column[i]->name);
		if(i != (old_index->n_columns-1)) old_columns = append_string(old_columns, "\", \"");
	}
	old_columns = append_string(old_columns, "\"");

	gchar * new_columns = NULL;
	new_columns = append_string(new_columns, "\"");
	for(int i = 0; i < new_index->n_columns; i++){
		new_columns = append_string(new_columns, new_index->column[i]->name);
		if(i != (new_index->n_columns-1)) new_columns = append_string(new_columns, "\", \"");
	}
	new_columns = append_string(new_columns, "\"");
	
	if(strcmp(old_columns, new_columns) != 0 || old_index->pkey_constraint != new_index->pkey_constraint ||
	    old_index->unique_constraint != new_index->unique_constraint || old_index->unique_index != new_index->unique_index ||
	    strcmp(old_index->predicate, new_index->predicate) || strcmp(old_index->tablespace, new_index->tablespace) || 
	    strcmp(old_index->access_method, new_index->access_method) != 0 ){
		// drop old
		gchar * sql1 = create_drop_index_SQL_query(old_index, parent_name, FALSE);
		sql = append_string(sql, sql1);
		free(sql1);
			
		sql = append_string(sql, "\n");
		// create new
		gchar * sql2 = create_new_index_SQL_query(new_index, parent_name, FALSE);
		sql = append_string(sql, sql2);
		free(sql2);
		pre = TRUE;
	}
	else if(strcmp(old_index->name, new_index->name) != 0){
		sql = append_string(sql, "ALTER INDEX \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, old_index->name);
		sql = append_string(sql, "\"\n  RENAME TO \"");
		sql = append_string(sql, new_index->name);
		sql = append_string(sql, "\";\n");
		pre = TRUE;
	}
  free(old_columns); free(new_columns); 
	
	// change clustered
	if( old_index->clustered != new_index->clustered ){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "ALTER TABLE \"");
		sql = append_string(sql, parent_name);
		if(new_index->clustered)	sql = append_string(sql, "\"\n  SET WITHOUT CLUSTER;\n");
		else{
			sql = append_string(sql, "\"\n  CLUSTER ON \"");
			sql = append_string(sql, new_index->name);
			sql = append_string(sql, "\";\n");
		}
		pre = TRUE;
	}
	
	// change comment
	if(strcmp(old_index->comment, new_index->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "\nCOMMENT ON INDEX \"");
		sql = append_string(sql, schema);
		sql = append_string(sql, "\".\"");
		sql = append_string(sql, new_index->name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_index->comment);
		sql = append_string(sql, "\';\n");
	}
	
	free(schema);
	return sql;
} // create_edit_index_SQL_query()

/*
 * Creates SQL query to edit foreign key
 */
gchar *
create_edit_fkey_SQL_query(T_FKey * old_fkey, T_FKey * new_fkey, gchar * parent_name){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	// change columns, foreign table, foreign columns, on update rule, on delete rule, match type, deferrable, check time
	gchar * old_columns = NULL;
	old_columns = append_string(old_columns, "\"");
	for(int i = 0; i < old_fkey->n_columns; i++){
		old_columns = append_string(old_columns, old_fkey->column[i]->name);
		if(i != (old_fkey->n_columns-1)) old_columns = append_string(old_columns, "\", \"");
	}
	old_columns = append_string(old_columns, "\"");
	
	gchar * new_columns = NULL;
	new_columns = append_string(new_columns, "\"");
	for(int i = 0; i < new_fkey->n_columns; i++){
		new_columns = append_string(new_columns, new_fkey->column[i]->name);
		if(i != (new_fkey->n_columns-1)) new_columns = append_string(new_columns, "\", \"");
	}
	new_columns = append_string(new_columns, "\"");
	
	gchar * old_ref_columns = NULL;
	old_ref_columns = append_string(old_ref_columns, "\"");
	for(int i = 0; i < old_fkey->n_ref_columns; i++){
		old_ref_columns = append_string(old_ref_columns, old_fkey->ref_column[i]->name);
		if(i != (old_fkey->n_ref_columns-1)) old_ref_columns = append_string(old_ref_columns, "\", \"");
	}
	old_ref_columns = append_string(old_ref_columns, "\"");
	
	gchar * new_ref_columns = NULL;
	new_ref_columns = append_string(new_ref_columns, "\"");
	for(int i = 0; i < new_fkey->n_ref_columns; i++){
		new_ref_columns = append_string(new_ref_columns, new_fkey->ref_column[i]->name);
		if(i != (new_fkey->n_ref_columns-1)) new_ref_columns = append_string(new_ref_columns, "\", \"");
	}
	new_ref_columns = append_string(new_ref_columns, "\"");
	
	if(strcmp(old_fkey->name, new_fkey->name) != 0 || strcmp(old_columns, new_columns) != 0 ||
	    strcmp(old_fkey->ref_schema, new_fkey->ref_schema) || strcmp(old_fkey->ref_table, new_fkey->ref_table) || 
	    strcmp(old_ref_columns, new_ref_columns) != 0 || strcmp(old_fkey->on_update_rule, new_fkey->on_update_rule) ||
	    strcmp(old_fkey->on_delete_rule, new_fkey->on_delete_rule) || strcmp(old_fkey->match_type, new_fkey->match_type) ||
	    old_fkey->deferrable != new_fkey->deferrable || strcmp(old_fkey->check_time, new_fkey->check_time) ){
		// drop old
			gchar * sql1 = create_drop_fkey_SQL_query(old_fkey, parent_name, FALSE);
			sql = append_string(sql, sql1);
			free(sql1);
			
			sql = append_string(sql, "\n");
			// create new
			gchar * sql2 = create_new_fkey_SQL_query(new_fkey, parent_name, FALSE);
			sql = append_string(sql, sql2);
			free(sql2);
			pre = TRUE;
	}
  free(old_columns); free(new_columns); free(old_ref_columns); free(new_ref_columns);
  
	// change comment
	if(strcmp(old_fkey->comment, new_fkey->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON CONSTRAINT \"");
		sql = append_string(sql, new_fkey->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_fkey->comment);
		sql = append_string(sql, "\';\n");
	}
		
	return sql;
} // create_edit_fkey_SQL_query()

/*
 * Creates SQL query to edit check
 */
gchar *
create_edit_check_SQL_query(T_Check * old_check, T_Check * new_check, gchar * parent_name){
	gchar * sql = NULL;
	gboolean pre = FALSE;
	
	// change name or condition
	if(strcmp(old_check->name, new_check->name) != 0 || strcmp(old_check->condition, new_check->condition) != 0){
		// drop old
			gchar * sql1 = create_drop_check_SQL_query(old_check, parent_name, FALSE);
			sql = append_string(sql, sql1);
			free(sql1);
			
			sql = append_string(sql, "\n");
			// create new
			gchar * sql2 = create_new_check_SQL_query(new_check, parent_name, FALSE);
			sql = append_string(sql, sql2);
			free(sql2);
			pre = TRUE;
	}

	// change comment
	if(strcmp(old_check->comment, new_check->comment) != 0){
		if(pre) sql = append_string(sql, "\n");
		sql = append_string(sql, "COMMENT ON CONSTRAINT \"");
		sql = append_string(sql, new_check->name);
		sql = append_string(sql, "\"\n  ON \"");
		sql = append_string(sql, parent_name);
		sql = append_string(sql, "\"\n");	
		sql = append_string(sql, "  IS \'");
		sql = append_string(sql, new_check->comment);
		sql = append_string(sql, "\';\n");
	}
		
	return sql;
} // create_edit_check_SQL_query()
