/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "functions.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*  Error functions               																																														 */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * Prints error message and exits the program
 */
void
error (gchar * header, gchar * message)
{
	
	message_dialog(header, message, GTK_MESSAGE_ERROR, NULL);
	
	fprintf(stderr, "%s%s%s \n", (header != NULL) ? header : "", (header != NULL) ? " - " : "", message);
	exit(0);
} // error()

/*
 * Shows message dialog
 */
void
message_dialog(gchar * markup, gchar * message, GtkMessageType type, GtkWidget * parent)
{
  GtkWidget * dialog = NULL;
  dialog = gtk_message_dialog_new_with_markup(GTK_WINDOW(parent), 
                                                          GTK_DIALOG_MODAL, 
                                                          type,
                                                          GTK_BUTTONS_OK,  
                                                          "<span font_desc='Tahoma 9 Bold' foreground='#000000'>%s</span>", markup);
                                                          
    // if(markup != NULL) gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), "<span font_desc='Tahoma 9 Bold' foreground='#000000'>%s</span>", markup);                                                     
    if(message != NULL) gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), message);
    
    gtk_window_set_icon(GTK_WINDOW(dialog), get_icon(PIXMAP_DIRECTORY "/pglogo.png"));
		
		gint r = gtk_dialog_run(GTK_DIALOG(dialog));
		if(r == GTK_RESPONSE_OK);
		gtk_widget_destroy (dialog);
} // message_dialog ()

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*  Functions to work with strings																																										 */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * Alloc new string with specified length
 */
char *
alloc_string         (int n)
{
	char *new_str = (char*) malloc (sizeof(char)*n);
	if(new_str == NULL) error("Error: Memory allocation", "Not enough memory");
	
	return new_str;
} // alloc_string()


/*
 * Create a copy of given string
 */
char *
copy_string        (const char* old_str)
{
	char * new_str = alloc_string(strlen(old_str)+1);
	strcpy(new_str, old_str);
	return new_str;
} // copy_string()


gchar *
append_string(gchar * s1, gchar * s2){
	gint len1 = 0;
	gint len2 = 0;
	
	if(s1 == NULL){
		if(s2 != NULL) len2 = strlen(s2);
		s1 = (gchar *) malloc(sizeof(char) * (len2 + 1));
		if(s1 == NULL) error("Error: Memory allocation", "Not enough memory");
		s1[0] = '\0';
	}
	else{
		if(s1 != NULL) len1 = strlen(s1);
		if(s2 != NULL) len2 = strlen(s2);
		s1 = (gchar *) realloc(s1, sizeof(char) * (len1 + len2 + 1));
		if(s1 == NULL) error("Error: Memory allocation", "Not enough memory");
	}
	strcat(s1, s2);
	return s1;	
} // append_string ()

gchar **
append_string_to_array(gchar ** list, gint * n_strings, gchar * s){
	gint n = *n_strings;
	
	if(list == NULL && n == 0){
		list = (gchar **) malloc(sizeof(gchar *));
		if(list == NULL) error("Error: Memory allocation", "Not enough memory");
		list[0] = s;
		(*n_strings)++;
	}
	else if(list != NULL){
		list = (gchar **) realloc(list, sizeof(gchar *)*(n+1));
		if(list == NULL) error("Error: Memory allocation", "Not enough memory");
		list[n] = s;
		(*n_strings)++;
	}
	else return NULL;
	
	return list;
} // append_string_to_array ()

/*
 * Make string from integer with specified number of digits.
 */
char *
itoa_n                 (int number, int n)
{
  char * s;
  int num=number;
	gboolean neg = FALSE;
	
	if(number < 0){
	   neg = TRUE;
	   number *= -1;
	 }
	
	if(n == 0){ // must count digits
    while((num /= 10) != 0){
	    n++;
	  }
		n++;
    num = number; // renew
	}

  // Create string
  if(neg) n++;
  s = alloc_string(n+1);

  // Convert the number
  for(int i = n-1; i >= 0; i--){
     s[i] = 48 + (num%10); 
     num = num/10;
  }
  if(neg) s[0]='-';
  s[n] = '\0';
  
  return s;
} // itoa_n()

/*
 * Check if the host is IP or name
 */
gboolean
parse_host(const char * host)
{
	int n = strlen(host);
	int j=0, k = 0;

	if(n > 15) return TRUE;
	
	for(int i = 0; i < strlen(host); i++){
		if(host[i] >= '0' && host[i] <= '9'){
			j++;
			if(j > 3) return TRUE;
		}
		else if(host[i] == '.'){
			if(j == 0) return TRUE;
			else{
				 j = 0;
				 k++;
				if(k > 3) return TRUE;
			}
		}
		else return TRUE;
	}
	if(k<3) return TRUE;
	if(j == 0) return TRUE; // Ended with dot

	return FALSE;
} // parse_host ()


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*  Functions to make labels																																														 */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * Changes text color
 */
gchar * 
change_text_color(gchar * text, gchar * color)
{
	gchar * t = parse_markup_text(text);
	
	int n = strlen("<span foreground='") + strlen(color) + strlen("'>") + strlen(t) + strlen("</span>");
	gchar * new_text = alloc_string(n+1);
	
	strcpy(new_text, "<span foreground='");
	strcat(new_text, color);
	strcat(new_text, "'>");
	strcat(new_text, t);
	strcat(new_text, "</span>");
	
	free(t);
	return new_text;
} // change_text_color ()

/*
 * Makes normal black text
 */
gchar * 
normal_text(gchar * text)
{
	gchar * t = parse_markup_text(text);
	
	int n = strlen("<span font_desc='Tahoma 8.25' foreground='#000000'>") + strlen(t) + strlen("</span>");
	gchar * new_text = alloc_string(n+1);
	
	strcpy(new_text, "<span font_desc='Tahoma 8.25' foreground='#000000'>");
	strcat(new_text, t);
	strcat(new_text, "</span>");
	
	free(t);
	return new_text;
} // normal_text ()

/*
 * Makes normal text with specified color
 */
gchar * 
normal_text_with_color(gchar * text, gchar * color)
{
	gchar * t = parse_markup_text(text);
	
	int n = strlen("<span font_desc='Tahoma 8.25' foreground='") + strlen(color) + strlen("'>") + strlen(t) + strlen("</span>");
	gchar * new_text = alloc_string(n+1);
	
	strcpy(new_text, "<span font_desc='Tahoma 8.25' foreground='");
	strcat(new_text, color);
	strcat(new_text, "'>");
	strcat(new_text, t);
	strcat(new_text, "</span>");
	
	free(t);
	return new_text;
} // normal_text_with_color ()

/*
 * Makes bold text with specified color
 */
gchar * 
bold_text(gchar * text, gchar * color)
{
	gchar * t = parse_markup_text(text);
	int n = strlen("<span font_desc='Tahoma bold 8.25' foreground='") + strlen(color) + strlen("'>") + strlen(t) + strlen("</span>");
	gchar * new_text = alloc_string(n+1);
	
	strcpy(new_text, "<span font_desc='Tahoma bold 8.25' foreground='");
	strcat(new_text, color);
	strcat(new_text, "'>");
	strcat(new_text, t);
	strcat(new_text, "</span>");
	
	free(t);
	return new_text;
} // bold_text ()

/*
 * Makes bold text with number in parenthesis behind text
 */
gchar * 
bold_text_with_count(gchar * text, gint count)
{
	gchar * s_count = itoa_n(count,0);
	gchar * t = parse_markup_text(text);
	
	int n = strlen("<span font_desc='Tahoma bold 8.25' foreground='#000000'>") + strlen(t) + strlen("</span>") +
	            strlen(" <span font_desc='Tahoma 8.25' foreground='#980F0F'>(") + strlen(s_count) + strlen(")</span>");
	gchar * new_text = alloc_string(n+1);
	
	strcpy(new_text, "<span font_desc='Tahoma bold 8.25' foreground='#000000'>");
	strcat(new_text, t);
	strcat(new_text, "</span>");
	strcat(new_text, " <span font_desc='Tahoma 8.25' foreground='#980F0F'>(");
	strcat(new_text, s_count);
	strcat(new_text, ")</span>");
	
	free(s_count);
	free(t);
	return new_text;
} // bold_text_with_count ()

/*
 * Checks text if there are only allowed chars
 */
gchar *
parse_markup_text(gchar * text)
{
	gint n = strlen(text);
	
	gchar temp[5*n + 1];

	int i,j;
	for(i=0, j=0; i<n; i++, j++){
		if(text[i] == '<'){
			temp[j] = '&';
			j++;
			temp[j] = 'l';
			j++;
			temp[j] = 't';
			j++;
			temp[j] = ';';			
		}
		else if(text[i] == '>'){
			temp[j] = '&';			
			j++;
			temp[j] = 'g';
			j++;
			temp[j] = 't';
			j++;
			temp[j] = ';';			
		}
		else if(text[i] == '&'){
			temp[j] = '&';
			j++;
			temp[j] = 'a';
			j++;
			temp[j] = 'm';
			j++;
			temp[j] = 'p';
			j++;
			temp[j] = ';';			
		}
		else temp[j] = text[i];
	}
	
	temp[j]='\0';
	
	return copy_string(temp);
} // parse_markup_text

/*
 * Checks text if there are no underlines, if yes, replace them
 */
gchar * 
parse_underline(gchar * text)
{
	gint n = strlen(text);
	
	gchar temp[3*n + 1];

	int i,j;
	for(i=0, j=0; i<n; i++, j++){
		if(text[i] == '_'){
			temp[j] = '_';
			j++;
			temp[j] = '_';
		}
		else temp[j] = text[i];
	}
	
	temp[j]='\0';
	
	return copy_string(temp);
} // parse_underline()

/*
 * Gets text from markup string
 */
gchar *
get_text_from_markup(gchar * markup){
	gint i=0, j = 0;
	gchar temp[strlen(markup)];
	
	while( markup[i] != '>') i++;
	i++;
	
	while( markup[i] != '<'){
		temp[j] = markup[i];
		 i++;
		 j++;
	}
	temp[j] = '\0';
	
	return copy_string(temp);
} // get_text_from_markup ()

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*  Functions for getting pictures																																											 */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * Gets icon from filename
 */
GtkWidget*
create_pixmap (const gchar *filename)
{
  GtkWidget *pixmap;

  if (!filename || !filename[0])  return gtk_image_new ();

  pixmap = gtk_image_new_from_file (filename);
  
  return pixmap;
} // create_pixmap ()

/*
 * Gets pixbuf from file
 */
GdkPixbuf * 
get_icon(gchar * file)
{
	GdkPixbuf * icon;
  GError          * error = NULL;
  
  icon = gdk_pixbuf_new_from_file(file, &error);
  if (error)
  {
   	g_warning ("Could not load icon: %s\n", error->message);
   	g_error_free(error);
   	error = NULL;
  }
	
	return icon;
} // get_icon ()
