/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */
 
#include "popup_menu_callbacks.h"
#include "../GUI/GUI_functions.h"
#include "../GUI_db_objects/database.h"
#include "../GUI_db_objects/schema.h"
#include "../GUI_db_objects/language.h"
#include "../GUI_db_objects/cast.h"
#include "../GUI_db_objects/table.h"
#include "../GUI_db_objects/view.h"
#include "../GUI_db_objects/function.h"
#include "../GUI_db_objects/domain.h"
#include "../GUI_db_objects/trigger.h"
#include "../GUI_db_objects/rule.h"
#include "../GUI_db_objects/sequence.h"
#include "../GUI_db_objects/aggregate.h"
#include "../GUI_db_objects/type.h"
#include "../GUI_db_objects/operator.h"
#include "../GUI_db_objects/other_objects.h"
#include "../tools/db_profile_manager.h"
#include "../tools/sql_editor.h"
#include "../otree/show_otree.h"
#include "../GUI_wizards/db_profile_wizard.h"
#include "../GUI_wizards/db_wizard.h"
#include "../GUI_wizards/schema_wizard.h"
#include "../GUI_wizards/table_wizard.h"
#include "../GUI_wizards/trigger_wizard.h"
#include "../GUI_wizards/rule_wizard.h"
#include "../database/create_db_objects.h"
#include "../database/drop_db_objects.h"

 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/* Callbacks for popup menus                                                                                         */
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void
connect_popup_menu_signals(GladeXML * source)
{
	GtkWidget * widget;
	
	// popup_menu_databases
	widget = glade_xml_get_widget(source, "pop_db_create_db_profile");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_create_db_profile_activate), NULL);
  widget = glade_xml_get_widget(source, "pop_db_edit_db_profile");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_edit_db_profile_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_db_rm_db_profile");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_rm_db_profile_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_db_conn_db");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_conn_db_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_db_disconn_db");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_disconn_db_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_db_create_db");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_create_db_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_db_drop_db");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_db_drop_db_activate), NULL);
	
	// popup_menu_schemas
	widget = glade_xml_get_widget(source, "pop_schs_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_schs_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_schs_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_drop_activate), NULL);
	
	// popup_menu_schema
	widget = glade_xml_get_widget(source, "pop_sch_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_sch_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_sch_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_sch_drop_activate), NULL);
	
	// popup_new
	widget = glade_xml_get_widget(source, "pop_new_table");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_new_view");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_view_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_function");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_func_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_domain");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_domain_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_aggregate");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_aggreg_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_sequence");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_seq_create_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_new_type");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_type_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_operator");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_oper_create_activate), NULL);   
		
	// popup_menu_tables
	widget = glade_xml_get_widget(source, "pop_tabs_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_tabs_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_tabs_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_drop_activate), NULL);
	
	// popup_menu_table
	widget = glade_xml_get_widget(source, "pop_tab_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_tab_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_tab_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_tab_drop_activate), NULL);
		
	// popup_new
	widget = glade_xml_get_widget(source, "pop_new_field");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_field_add_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_new_index");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_index_add_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_fkey");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_fkey_add_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_check");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_check_add_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_trigger");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_trigg_create_activate), NULL);   
	widget = glade_xml_get_widget(source, "pop_new_rule");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_rule_create_activate), NULL);   
		
	// popup_menu_views
	widget = glade_xml_get_widget(source, "pop_view_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_view_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_view_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_view_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_view_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_view_drop_activate), NULL);
		
	// popup_menu_functions   
	widget = glade_xml_get_widget(source, "pop_func_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_func_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_func_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_func_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_func_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_func_drop_activate), NULL);
	
	// popup_menu_domains
	widget = glade_xml_get_widget(source, "pop_domain_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_domain_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_domain_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_domain_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_domain_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_domain_drop_activate), NULL);
	
	// popup_menu_aggregates
	widget = glade_xml_get_widget(source, "pop_aggreg_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_aggreg_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_aggreg_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_aggreg_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_aggreg_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_aggreg_drop_activate), NULL);
	
	// popup_menu_sequences
	widget = glade_xml_get_widget(source, "pop_seq_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_seq_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_seq_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_seq_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_seq_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_seq_drop_activate), NULL);
	
	// popup_menu_types
	widget = glade_xml_get_widget(source, "pop_type_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_type_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_type_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_type_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_type_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_type_drop_activate), NULL);
	
	// popup_menu_operators
	widget = glade_xml_get_widget(source, "pop_oper_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_oper_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_oper_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_oper_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_oper_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_oper_drop_activate), NULL);
	
	// popup_menu_triggers
	widget = glade_xml_get_widget(source, "pop_trigg_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_trigg_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_trigg_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_trigg_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_trigg_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_trigg_drop_activate), NULL);
	
	// popup_menu_rules
	widget = glade_xml_get_widget(source, "pop_rule_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_rule_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_rule_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_rule_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_rule_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_rule_drop_activate), NULL);
	
	// popup_menu_languages
	widget = glade_xml_get_widget(source, "pop_lang_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_lang_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_lang_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_lang_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_lang_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_lang_drop_activate), NULL);
	
	// popup_menu_casts
	widget = glade_xml_get_widget(source, "pop_cast_create");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_cast_create_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_cast_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_cast_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_cast_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_cast_drop_activate), NULL);
	
	// popup_menu_fields
	widget = glade_xml_get_widget(source, "pop_field_add");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_field_add_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_field_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_field_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_field_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_field_drop_activate), NULL);
	
	// popup_menu_indexes
	widget = glade_xml_get_widget(source, "pop_index_add");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_index_add_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_index_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_index_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_index_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_index_drop_activate), NULL);
	
	// popup_menu_fkeys
	widget = glade_xml_get_widget(source, "pop_fkey_add");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_fkey_add_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_fkey_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_fkey_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_fkey_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_fkey_drop_activate), NULL);
	
	// popup_menu_checks
	widget = glade_xml_get_widget(source, "pop_check_add");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_check_add_activate), NULL);                                        
	widget = glade_xml_get_widget(source, "pop_check_edit");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_check_edit_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_check_drop");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_check_drop_activate), NULL);
	
	// popup_menu_refresh
	widget = glade_xml_get_widget(source, "pop_db_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL);
	widget = glade_xml_get_widget(source, "pop_schs_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_sch_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_tabs_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_tab_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_view_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_func_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_domain_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_aggreg_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_seq_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_type_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_oper_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_trigg_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_rule_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_lang_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_cast_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_field_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_index_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_fkey_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
	widget = glade_xml_get_widget(source, "pop_check_refresh");
	g_signal_connect ((gpointer) widget, "activate", G_CALLBACK (on_pop_refresh_activate), NULL); 
}

/* --------------------------------- popup_menu_databases -----------------------------------------*/
void
on_pop_db_create_db_profile_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	run_c_db_profile_wizard();
} // on_pop_db_create_db_profile_activate ()

void
on_pop_db_edit_db_profile_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	run_db_profile_manager();
} // on_pop_db_edit_db_profile_activate () 

void
on_pop_db_rm_db_profile_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_remove_profile(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path);
  gtk_tree_selection_select_path(selection, path);
} // on_pop_db_rm_db_profile_activate ()

void
on_pop_db_conn_db_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_connect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
	gtk_tree_view_expand_to_path (view, path);
} // on_pop_db_conn_db_activate ()

void
on_pop_db_disconn_db_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_disconnect_db(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
} // on_pop_db_disconn_db_activate ()

void
on_pop_db_create_db_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	run_c_db_wizard();
} // on_pop_db_create_db_activate ()

void
on_pop_db_drop_db_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 3){
		// get object name and database profile
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		
		// run dialog to confirm delete
		gchar * label = append_string(NULL, "Do you really want to drop database '");
		label = append_string(label, profile->dbname);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
		if(res == GTK_RESPONSE_YES){
			gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_database_SQL_query(profile);
		
		// execute sql
		if(sql != NULL){
			execute_sql(sql, NULL, NULL, profile, TRUE, TRUE);				
			free(sql);
		}
		
	}
} // on_pop_db_drop_db_activate ()


/* ----------------------------- popup_menu_schemas ----------------------------------------------*/
void
on_pop_sch_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		run_c_schema_wizard(o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
} // on_pop_sch_create_activate ()

void
on_pop_sch_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 5){
		create_schema_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]], gtk_tree_path_to_string(path));
	}	
} // on_pop_sch_edit_activate ()

void
on_pop_sch_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 5){
		// get object name and database profile
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop schema '");
		label = append_string(label, profile->database.schema[intpath[4]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_schema_SQL_query(profile->database.schema[intpath[4]], cascade);
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_sch_drop_activate ()

/* ----------------------------------- popup_menu_tables --------------------------------------------*/
void
on_pop_tab_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		run_c_table_wizard(o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
} // on_pop_tab_create_activate ()

void
on_pop_tab_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_table_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]], gtk_tree_path_to_string(path));
	}
} // on_pop_tab_edit_activate ()

void
on_pop_tab_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		// get object name and database profile
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop table '");
		label = append_string(label, profile->database.schema[intpath[4]]->name);
		label = append_string(label, ".");		
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_table_SQL_query(profile->database.schema[intpath[4]]->table[intpath[6]], profile->database.schema[intpath[4]]->name, cascade);
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_tab_drop_activate ()

// popup_menu_views
void
on_pop_view_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_view_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_view_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->view[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_view_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_functions
void
on_pop_func_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_func_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_function_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->function[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_func_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_domains
void
on_pop_domain_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_domain_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_domain_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->domain[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_domain_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_aggregates
void
on_pop_aggreg_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_aggreg_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_aggregate_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->aggregate[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_aggreg_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_sequences
void
on_pop_seq_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_seq_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_sequence_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->sequence[intpath[6]], gtk_tree_path_to_string(path));
	}	
}

void
on_pop_seq_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_types
void
on_pop_type_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_type_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_type_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->type[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_type_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_operators
void
on_pop_oper_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_oper_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7){
		create_operator_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->operator[intpath[6]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_oper_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

/* ----------------------------------- popup_menu_triggers ------------------------------------------*/
void
on_pop_trigg_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		run_c_trigger_wizard(o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
} // on_pop_trigg_create_activate ()

void
on_pop_trigg_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7 || depth == 9){
		if(depth == 7)	create_trigger_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->trigger[intpath[6]], gtk_tree_path_to_string(path));
		else create_trigger_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->trigger[intpath[8]], gtk_tree_path_to_string(path));
	}
} // on_pop_trigg_edit_activate ()

void
on_pop_trigg_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7 || depth == 9){
		// get object name and database profile
		gchar * parent_name = NULL;
		if(depth == 7){
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
			parent_name = append_string(parent_name, "\".\"");
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->trigger[intpath[6]]->table_name);
		}
		else{
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
			parent_name = append_string(parent_name, "\".\"");
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
		}
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop trigger '");
		if(depth == 7){
			label = append_string(label, profile->database.schema[intpath[4]]->trigger[intpath[6]]->table_name);
			label = append_string(label, ".");		
			label = append_string(label, profile->database.schema[intpath[4]]->trigger[intpath[6]]->name);	
		}
		else{
			label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
			label = append_string(label, ".");		
			label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->trigger[intpath[8]]->name);	
		}
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_trigger_SQL_query((depth == 7) ? profile->database.schema[intpath[4]]->trigger[intpath[6]] : profile->database.schema[intpath[4]]->table[intpath[6]]->trigger[intpath[8]], parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_trigg_drop_activate ()

/* ------------------------------------ popup_menu_rules -------------------------------------------- */
void
on_pop_rule_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		run_c_rule_wizard(o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
} // on_pop_rule_create_activate ()

void
on_pop_rule_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7 || depth == 9){
		if(depth == 7) create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->rule[intpath[6]], gtk_tree_path_to_string(path));
		else{
			if(intpath[5] == 0){ // table rule
				create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->rule[intpath[8]], gtk_tree_path_to_string(path));
			}
			else{ // view rule
				create_rule_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->view[intpath[6]]->rule[intpath[8]], gtk_tree_path_to_string(path));
			}
		}
	}
} // on_pop_rule_edit_activate ()

void
on_pop_rule_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 7 || depth == 9){
		// get object name and database profile
		gchar * parent_name = NULL;
		if(depth == 7){
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
			parent_name = append_string(parent_name, "\".\"");
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->rule[intpath[6]]->parent_name);
		}
		else{
			parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
			parent_name = append_string(parent_name, "\".\"");
			if(intpath[5] == 0) parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
			else parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->view[intpath[6]]->name);
		}
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop rule '");
		if(depth == 7){
			label = append_string(label, profile->database.schema[intpath[4]]->rule[intpath[6]]->parent_name);
			label = append_string(label, ".");		
			label = append_string(label, profile->database.schema[intpath[4]]->rule[intpath[6]]->name);	
		}
		else{
			if(intpath[5] == 0){
				label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
				label = append_string(label, ".");		
				label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->rule[intpath[8]]->name);	
			}
			else{
				label = append_string(label, profile->database.schema[intpath[4]]->view[intpath[6]]->name);
				label = append_string(label, ".");		
				label = append_string(label, profile->database.schema[intpath[4]]->view[intpath[6]]->rule[intpath[8]]->name);	
			}
		}
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_rule_SQL_query((depth == 7) ? profile->database.schema[intpath[4]]->rule[intpath[6]] : ((intpath[5] == 0) ? profile->database.schema[intpath[4]]->table[intpath[6]]->rule[intpath[8]] : profile->database.schema[intpath[4]]->view[intpath[6]]->rule[intpath[8]]), parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_rule_drop_activate ()

// popup_menu_languages
void
on_pop_lang_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_lang_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 5){
		create_language_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.language[intpath[4]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_lang_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

// popup_menu_casts
void
on_pop_cast_create_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

void
on_pop_cast_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 5){
		create_cast_info(glade_xml_get_widget(xml,"main_notebook"), o_tree.server[intpath[0]]->profile[intpath[2]]->database.cast[intpath[4]], gtk_tree_path_to_string(path));
	}
}

void
on_pop_cast_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{}

/* ------------------------------ popup_menu_fields ------------------------------------------------- */
/*
 * Callback function to create field
 */
void
on_pop_field_add_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to create new field
	if(depth >= 7){
		run_field_dialog(o_tree.server[intpath[0]]->profile[intpath[2]],
		                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
									             NULL, TRUE);
	}
} // on_pop_field_add_activate()

/*
 * Callback function to edit field
 */
void
on_pop_field_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to edit field
	if(depth == 9){
		run_field_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
															o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->column[intpath[8]], 
															FALSE);
	}
} // on_pop_field_edit_activate()

/*
 * Callback function to remove field
 */
void
on_pop_field_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 9){
		// get object name and database profile
		gchar * parent_name = append_string(NULL, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
		parent_name = append_string(parent_name, "\".\"");
		parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop field '");
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
		label = append_string(label, ".");		
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->column[intpath[8]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_field_SQL_query(profile->database.schema[intpath[4]]->table[intpath[6]]->column[intpath[8]], parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_field_drop_activate()

/* -------------------------------- popup_menu_indexes ---------------------------------------------*/
/*
 * Callback function to create index
 */
void
on_pop_index_add_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to create new field
	if(depth >= 7){
		run_index_dialog(o_tree.server[intpath[0]]->profile[intpath[2]],
		                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
									             NULL, TRUE);
	}
} // on_pop_index_add_activate()

/*
 * Callback function to edit index
 */
void
on_pop_index_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to edit index
	if(depth == 9){
		run_index_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
															o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->index[intpath[8]], 
															FALSE);
	}
} // on_pop_index_edit_activate()

/*
 * Callback function to remove index
 */
void
on_pop_index_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 9){
		// get object name and database profile
		gchar * parent_name = append_string(NULL, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
		parent_name = append_string(parent_name, "\".\"");
		parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop index '");
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
		label = append_string(label, ".");		
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->index[intpath[8]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_index_SQL_query(profile->database.schema[intpath[4]]->table[intpath[6]]->index[intpath[8]], parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_index_drop_activate()

/*---------------------------------  popup_menu_fkeys -----------------------------------------------*/
/*
 * Callback function to create foreign key
 */
void
on_pop_fkey_add_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to create new foreign key
	if(depth >= 7){
		run_fkey_dialog(o_tree.server[intpath[0]]->profile[intpath[2]],
		                          o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
									             NULL, TRUE);
	}
} // on_pop_fkey_add_activate()

/*
 * Callback function to edit foreign key
 */
void
on_pop_fkey_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to edit foreign key
	if(depth == 9){
		run_fkey_dialog(o_tree.server[intpath[0]]->profile[intpath[2]], 
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									            o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]],
															o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->fkey[intpath[8]], 
															FALSE);
	}
} // on_pop_fkey_edit_activate()

/*
 * Callback function to remove foreign key
 */
void
on_pop_fkey_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 9){
		// get object name and database profile
		gchar * parent_name = append_string(NULL, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
		parent_name = append_string(parent_name, "\".\"");
		parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop foreign key '");
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
		label = append_string(label, ".");		
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->fkey[intpath[8]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_fkey_SQL_query(profile->database.schema[intpath[4]]->table[intpath[6]]->fkey[intpath[8]], parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_fkey_drop_activate()

/* ------------------------- popup_menu_checks -----------------------------------------------------*/
/*
 * Callback function to create check
 */
void
on_pop_check_add_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	// run dialog to create new check
	if(depth >= 7){
		run_check_dialog(o_tree.server[intpath[0]]->profile[intpath[2]],
		                             o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									               o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]], 
                                 NULL, 
									               TRUE);
	}
} // on_pop_check_add_activate()

/*
 * Callback function to edit check
 */
void
on_pop_check_edit_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 9){
		// run dialog to edit check
		run_check_dialog(o_tree.server[intpath[0]]->profile[intpath[2]],
		                             o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]],
									               o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]], 
									               o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->check[intpath[8]], 
									               FALSE);
	}
} // on_pop_check_edit_activate()

/*
 * Callback function to remove check
 */
void
on_pop_check_drop_activate (GtkMenuItem *menuitem,  gpointer user_data)
{
	gboolean cascade = FALSE;
	
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth == 9){
		// get object name and database profile
		gchar * parent_name = append_string(NULL, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->name);
		parent_name = append_string(parent_name, "\".\"");
		parent_name = append_string(parent_name, o_tree.server[intpath[0]]->profile[intpath[2]]->database.schema[intpath[4]]->table[intpath[6]]->name);
		T_Profile * profile = o_tree.server[intpath[0]]->profile[intpath[2]];
		T_Server * server = o_tree.server[intpath[0]];
		
		// run dialog to confirm delete
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")), FALSE);
		gchar * label = append_string(NULL, "Do you really want to drop check '");
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->name);
		label = append_string(label, ".");		
		label = append_string(label, profile->database.schema[intpath[4]]->table[intpath[6]]->check[intpath[8]]->name);	
		label = append_string(label, "'?");
		gchar * l = normal_text(label);
		free(label);
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"drop_dialog_label")), l);
		free(l);
		gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "drop_dialog")));
		if(res == GTK_RESPONSE_YES){
			cascade = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "drop_dialog_cascade")));
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));
		}
		else{
			gtk_widget_hide(glade_xml_get_widget(xml, "drop_dialog"));	
			return;
		}
		
		// create sql code
		gchar * sql = create_drop_check_SQL_query(profile->database.schema[intpath[4]]->table[intpath[6]]->check[intpath[8]], parent_name, cascade);
		free(parent_name);
		
		
		// execute sql
		if(sql != NULL){
			if(execute_sql(sql, NULL, NULL, profile, TRUE, TRUE)) gui_refresh(server, profile);
			free(sql);
		}
		
	}
} // on_pop_check_drop_activate()

/* ------------------------------------------- Refresh ----------------------------------------------------*/
/*
 * Callback function to refresh selected profile
 */
void
on_pop_refresh_activate (GtkMenuItem *menuitem,  gpointer user_data){
	// get selected object
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "object_tree"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	
	if(depth >= 3){
		gui_refresh(o_tree.server[intpath[0]], o_tree.server[intpath[0]]->profile[intpath[2]]);
	}
} // on_pop_refresh_activate()
