/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "db_profile_manager.h"
#include "../otree/show_otree.h"
#include "../database/create_db_objects.h"
#include "../database/drop_db_objects.h"
#include "../GUI/GUI_functions.h"
#include "../tools/sql_editor.h"
#include "../GUI_wizards/db_profile_wizard.h"
#include "../GUI_wizards/db_wizard.h"

/*
 * Inits database profile manager
 */
void 
init_db_profile_manager(void){
	// Color settings
	GdkColor color;
	
	color.red = 132*257;
  color.green = 5*257;
  color.blue = 5*257;
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_menu_bg"), GTK_STATE_NORMAL, &color);
  
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_host_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_general_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_db_mgmt_bg"), GTK_STATE_NORMAL, &color);

	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
  
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_profiles_bg"), GTK_STATE_NORMAL, &color);  
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_properties_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_host_hr"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_general_hr"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "dbpm_db_mgmt_hr"), GTK_STATE_NORMAL, &color);
	
	// Init listviews ...
	create_profiles_treeview(GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles")));
	gtk_widget_hide(glade_xml_get_widget(xml, "database_profile_manager"));
	
	gtk_entry_set_editable(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "dbpm_host"))->child), FALSE);
	
	GtkTreeModel      *model;
  
  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")), model);
	gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(glade_xml_get_widget(xml, "dbpm_host")), 0);
} // init_db_profile_manager ()

/*
 * Runs database profile manager
 */
void 
run_db_profile_manager(void){
	gtk_widget_show(glade_xml_get_widget(xml, "database_profile_manager"));
	gtk_notebook_set_current_page(
			GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")), 
			gtk_notebook_page_num (
																GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")), 
																glade_xml_get_widget(xml, "database_profile_manager")
			)
	);
		
	refresh_profile_manager();
} // run_db_profile_manager ()

/*
 * Refreshes database profile manager
 */
void
refresh_profile_manager(void){
	gint n = o_tree.n_servers;
	
	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host"))));
	gtk_list_store_clear(store);
	
	GtkTreeStore * tree = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"))));
	gtk_tree_store_clear(tree);
	
	for(int i = 0; i < n; i++){
 		gtk_combo_box_append_text(GTK_COMBO_BOX (glade_xml_get_widget(xml, "dbpm_host")), o_tree.server[i]->name);
	}
	// set active first
	if(n > 0) gtk_combo_box_set_active(GTK_COMBO_BOX (glade_xml_get_widget(xml, "dbpm_host")) , 0);
	else{
		gtk_entry_set_text(GTK_ENTRY(GTK_BIN(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")))->child), "");
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_database_alias")), "");
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_database_name")), "");
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_host_name")), "");
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "dbpm_port")), (gdouble)5432);
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_user_name")), "");
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_password")), "");
		gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_SSL")), 2);
		
		gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db_profile"));
		gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_rm_selected_profile"));
		gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_apply_changes"));
		gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_connect_to_db"));
		gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_disconnect_from_db"));
		gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db"));
		gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_drop_selected_db"));
	}
} // refresh_profile_manager ()

/*
 * Creates treeview for existing profiles
 */
void
create_profiles_treeview(GtkTreeView * view){
    GtkTreeModel      *model;
    GtkTreeViewColumn *col;
    GtkCellRenderer   *renderer;
 
    model = GTK_TREE_MODEL(gtk_tree_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING));
    
    gtk_tree_view_set_model(GTK_TREE_VIEW(view), model);
				
    col = gtk_tree_view_column_new();
    
    renderer = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_start(col, renderer, FALSE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "pixbuf", 0,
                                        NULL);
                                        
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer,
                                        "markup", 1,
                                        NULL);
                                            
    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
} // create_profiles_treeview ()

/*
 * Adds profile to treeview
 */
GtkTreeIter
add_profile(GtkTreeStore * store, GtkTreeIter * iter, GdkPixbuf *icon, gchar * profile, gboolean bold)
{
		GtkTreeIter child;
		gchar * text;
 		gtk_tree_store_append(store, &child, iter);
 		
 		if(bold)	text = bold_text(profile, "#000000");
 		else text = normal_text(profile);
 		
		gtk_tree_store_set(store, &child, 0, icon, 1, text, -1);
		free(text);
		return child;
} // add_profile ()

/*
 * Connects database profile manager callbacks
 */
void
connect_dbpm_signals (GladeXML * source){
	GtkWidget * widget;
	
	// Create new database profile
  widget = glade_xml_get_widget(xml, "dbpm_create_db_profile");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_create_db_profile_clicked),
                    NULL);
	
	// Remove selected profile
  widget = glade_xml_get_widget(xml, "dbpm_rm_selected_profile");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_rm_selected_profile_clicked),
                    NULL);
	
	// Apply changes
  widget = glade_xml_get_widget(xml, "dbpm_apply_changes");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_apply_changes_clicked),
                    NULL);
	
	// Connect from database
  widget = glade_xml_get_widget(xml, "dbpm_connect_to_db");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_connect_to_db_clicked),
                    NULL);

	// Disconnect from database
  widget = glade_xml_get_widget(xml, "dbpm_disconnect_from_db");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_disconnect_from_db_clicked),
                    NULL);

	// Create new database
  widget = glade_xml_get_widget(xml, "dbpm_create_db");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_create_db_clicked),
                    NULL);
	
	// Drop selected database
  widget = glade_xml_get_widget(xml, "dbpm_drop_selected_db");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_drop_selected_db_clicked),
                    NULL);                                                                                                    

	// Host changed
	widget = glade_xml_get_widget(xml, "dbpm_host");	
	g_signal_connect_after((gpointer) widget, "changed",
                    G_CALLBACK (on_dbpm_host_changed),
                    NULL);
	
	// Alias changed
	widget = glade_xml_get_widget(xml, "dbpm_database_alias");	
	g_signal_connect_after((gpointer) widget, "changed",
                    G_CALLBACK (on_dbpm_alias_changed),
                    NULL);
  // Close                  
  widget = glade_xml_get_widget(xml, "dbpm_close");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_dbpm_close_clicked),
                    NULL);   
	
  widget = glade_xml_get_widget(xml, "dbpm_profiles");	                 
	GtkTreeSelection  *selection;
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
  gtk_tree_selection_set_select_function(selection, dbpm_profile_selected, NULL, NULL); 
} // connect_dbpm_signals ()

////////////////////////////////////// Database Profile Manager callbacks ///////////////////////////////////////////
void
on_dbpm_create_db_profile_clicked(GtkButton *button, gpointer user_data){
	run_c_db_profile_wizard();	
}

void
on_dbpm_rm_selected_profile_clicked(GtkButton *button, gpointer user_data){
	// get server
	gint i_server = -1;
	i_server = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(i_server == -1) return; // no server selected
	
	// get profile
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	if(!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
	
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	if(depth < 1 && depth > 1) return; //unexpected error
	gint i_profile = intpath[0];
	
	// remove selected profile
	gui_remove_profile(o_tree.server[i_server], o_tree.server[i_server]->profile[i_profile]);
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
}

void
on_dbpm_apply_changes_clicked(GtkButton *button, gpointer user_data){
	gint i = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	GtkTreeSelection  *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles")));
  GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles")));
  GtkTreeIter iter;
  if(!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
  GtkTreePath * path = gtk_tree_model_get_path(model, &iter);
  gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	if(depth == 1){
		o_tree.server[i]->profile[intpath[0]]->modified = FALSE;
		free(o_tree.server[i]->profile[intpath[0]]->alias);
		o_tree.server[i]->profile[intpath[0]]->alias = copy_string(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_database_alias"))));
		show_object_tree();
	}
}

void
on_dbpm_connect_to_db_clicked(GtkButton *button, gpointer user_data){
	// get server
	gint i_server = -1;
	i_server = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(i_server == -1) return; // no server selected
	
	// get profile
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	if(!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
	
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	if(depth < 1 && depth > 1) return; //unexpected error
	gint i_profile = intpath[0];
	
	// connect to database
	gui_connect_db(o_tree.server[i_server], o_tree.server[i_server]->profile[i_profile]);
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
}

void
on_dbpm_disconnect_from_db_clicked(GtkButton *button, gpointer user_data){
	// get server
	gint i_server = -1;
	i_server = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(i_server == -1) return; // no server selected
	
	// get profile
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	if(!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
	
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	if(depth < 1 && depth > 1) return; //unexpected error
	gint i_profile = intpath[0];
	
	// disconnect from database
	gui_disconnect_db(o_tree.server[i_server], o_tree.server[i_server]->profile[i_profile]);
	refresh_profile_manager();
	refresh_sql_editor();
	show_object_tree();
}

void
on_dbpm_create_db_clicked(GtkButton *button, gpointer user_data){
	run_c_db_wizard();
}

void
on_dbpm_drop_selected_db_clicked(GtkButton *button, gpointer user_data){
	// get server
	gint i_server = -1;
	i_server = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(i_server == -1) return; // no server selected
	
	// get profile
	GtkTreeSelection * selection;
	GtkTreeView * view = GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"));
	GtkTreeModel * model = gtk_tree_view_get_model(view);
	GtkTreeIter iter;
	GtkTreePath * path;
	selection = gtk_tree_view_get_selection(view);
	if(!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
	
	path = gtk_tree_model_get_path(model, &iter);
 	gint depth = gtk_tree_path_get_depth(path);
	gint * intpath = gtk_tree_path_get_indices(path);
	if(depth < 1 && depth > 1) return; //unexpected error
	gint i_profile = intpath[0];	
	
	T_Profile * profile  = o_tree.server[i_server]->profile[i_profile];
	// run dialog to confirm delete	
	gchar * label = append_string(NULL, "Do you really want to drop database '");
	label = append_string(label, profile->dbname);	
	label = append_string(label, "'?");
	gchar * l = normal_text(label);
	free(label);
	gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml,"yes_no_dialog_label")), l);
	free(l);
	gint res = gtk_dialog_run(GTK_DIALOG(glade_xml_get_widget(xml, "yes_no_dialog")));
	if(res == GTK_RESPONSE_YES){
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));
	}
	else{
		gtk_widget_hide(glade_xml_get_widget(xml, "yes_no_dialog"));	
		return;
	}
	
	// create sql code
	gchar * sql = create_drop_database_SQL_query(profile);
	
	// execute sql
	if(sql != NULL){
		execute_sql(sql, NULL, NULL, profile, TRUE, TRUE);				
		free(sql);
	}
}

void
on_dbpm_alias_changed(GtkEditable *editable, gpointer user_data){
}

gboolean
dbpm_profile_selected (GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean path_currently_selected, gpointer data){
	gint i = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(!path_currently_selected){
		gint depth = gtk_tree_path_get_depth(path);
		gint * intpath = gtk_tree_path_get_indices(path);
		if(depth == 1){
			// Set menu buttons sensitivity
			if(o_tree.server[i]->profile[intpath[0]]->connected){
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db_profile"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_rm_selected_profile"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_apply_changes"));
				gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_connect_to_db"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_disconnect_from_db"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_drop_selected_db"));
			}
			else{
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db_profile"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_rm_selected_profile"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_apply_changes"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_connect_to_db"));
				gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_disconnect_from_db"));
				gtk_widget_show(glade_xml_get_widget(xml, "dbpm_create_db"));
				gtk_widget_hide(glade_xml_get_widget(xml, "dbpm_drop_selected_db"));
			}
			
			// Set profile properties
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_database_alias")), o_tree.server[i]->profile[intpath[0]]->alias);
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_database_name")), o_tree.server[i]->profile[intpath[0]]->dbname);
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_host_name")), o_tree.server[i]->host);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "dbpm_port")), (gdouble)o_tree.server[i]->port);
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_user_name")), o_tree.server[i]->uname);
			gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "dbpm_password")), o_tree.server[i]->passwd);
			gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_SSL")), o_tree.server[i]->SSL);
		}						 
	}
	
	return TRUE;
}

void
on_dbpm_close_clicked(GtkButton *button, gpointer user_data){
	gtk_widget_hide(glade_xml_get_widget(xml, "database_profile_manager"));
}

void
on_dbpm_host_changed(GtkEditable *editable, gpointer user_data){
	gint i = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "dbpm_host")));
	if(i == -1) return;
	GtkTreeView * treeview = GTK_TREE_VIEW(glade_xml_get_widget(xml, "dbpm_profiles"));
	GtkTreeStore * store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
	gtk_tree_store_clear(store);
	GtkTreeSelection * selection = gtk_tree_view_get_selection(treeview);
	GtkTreeIter iter;
	
	for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
			if(o_tree.server[i]->profile[j]->connected) iter = add_profile(store,NULL,get_icon(PIXMAP_DIRECTORY "/Database.png"), o_tree.server[i]->profile[j]->alias, TRUE);
			else iter = add_profile(store,NULL,get_icon(PIXMAP_DIRECTORY "/Database_not_conn.png"), o_tree.server[i]->profile[j]->alias, FALSE);
			gtk_tree_selection_select_iter(selection, &iter);
	}
}
