/*
 * C source file for program pguiman - PostgreSQL Manager
 *
 * Copyright (C) 2007 Dvoracek Ondrej
 *
 * Dvoracek Ondrej <odvorace@redhat.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 *
 */

#include "sql_editor.h"
#include "../GUI/GUI_functions.h"
#include "../GUI/textview_functions.h"
#include "../database/create_db_objects.h"

/*
 * Inits SQL editor
 */
void 
init_sql_editor(void){
	// Color settings
	GdkColor color;

	color.red = 132*257;
  color.green = 5*257;
  color.blue = 5*257;
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_menu_bg"), GTK_STATE_NORMAL, &color);
  
	color.red = 255*257;
  color.green = 255*257;
  color.blue = 255*257;  
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_host_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_general_bg"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_menubottom_bg"), GTK_STATE_NORMAL, &color);
  
	color.red = 128*257;
  color.green = 128*257;
  color.blue = 128*257;  
  
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_code_bg"), GTK_STATE_NORMAL, &color);  
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_messages_bg"), GTK_STATE_NORMAL, &color);
	gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_host_hr"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_general_hr"), GTK_STATE_NORMAL, &color);
  gtk_widget_modify_bg (glade_xml_get_widget(xml, "sqle_database_hr"), GTK_STATE_NORMAL, &color);
    	
	// Init textviews ...
	init_text_view(glade_xml_get_widget(xml, "sqle_code"), TRUE,"-- Your SQL code");
	init_message_view(glade_xml_get_widget(xml, "sqle_messages"), FALSE,"");
		
	gtk_widget_hide(glade_xml_get_widget(xml, "sql_editor"));
	
	gtk_entry_set_editable(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "sqle_host"))->child), FALSE);
	gtk_entry_set_editable(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "sqle_database"))->child), FALSE);
	
	GtkTreeModel      *model;
  
  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_host")), model);
	gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(glade_xml_get_widget(xml, "sqle_host")), 0);

	model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
	
	gtk_combo_box_set_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_database")), model);
	gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(glade_xml_get_widget(xml, "sqle_database")), 0);
} // init_sql_editor ()

/*
 * Runs SQL Editor
 */
void 
run_sql_editor(void){
	gtk_widget_show(glade_xml_get_widget(xml, "sql_editor"));
	gtk_notebook_set_current_page(
			GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")), 
			gtk_notebook_page_num (
																GTK_NOTEBOOK(glade_xml_get_widget(xml, "main_notebook")), 
																glade_xml_get_widget(xml, "sql_editor")
			)
	);
		
	refresh_sql_editor();
} // run_sql_editor ()

/*
 * Refreshes SQL Editor
 */
void
refresh_sql_editor(void){
	gint n = o_tree.n_servers;
	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_host"))));
	gtk_list_store_clear(store);
	//gtk_combo_box_set_active(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_host")) , -1);
	
	for(int i = 0; i < n; i++){
 		gtk_combo_box_append_text(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_host")), o_tree.server[i]->name);
	}
	// set active first
	if(n > 0) gtk_combo_box_set_active(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_host")) , 0);
	else gtk_widget_hide(glade_xml_get_widget(xml, "sqle_execute"));
	
	
	GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")));
	if(model != NULL){
			gtk_list_store_clear(GTK_LIST_STORE(model));
			GList * list = gtk_tree_view_get_columns(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")));
			for(; list != NULL; list = list->next) gtk_tree_view_remove_column(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")), GTK_TREE_VIEW_COLUMN(list->data));
	}
	
	GtkTextBuffer * buffer;
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "sqle_code")));
	GtkTextIter start, end;
 	gtk_text_buffer_get_start_iter(buffer, &start);
 	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
	gtk_text_buffer_insert (buffer, &start, "-- Your SQL code", -1);
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "sqle_messages")));
	gtk_text_buffer_get_start_iter(buffer, &start);
 	gtk_text_buffer_get_end_iter(buffer, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
	
	if(GTK_IS_NOTEBOOK(GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")))){
		gtk_notebook_set_current_page( GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")), 1);
	}
} // refresh_sql_editor ()

/*
 * Connects SQL editor callbacks
 */
void
connect_sqle_signals (GladeXML * source){
	GtkWidget * widget;
	
	//Execute defined SQL code
  widget = glade_xml_get_widget(xml, "sqle_execute");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_sqle_execute_clicked),
                    NULL);
                    
	// Treeview changed
	widget = glade_xml_get_widget(xml, "sqle_data_view");	
	g_signal_connect_after((gpointer) widget, "columns-changed",
                    G_CALLBACK (on_tree_view_changed),
                    NULL);
	
	// Host changed
	widget = glade_xml_get_widget(xml, "sqle_host");	
	g_signal_connect_after((gpointer) widget, "changed",
                    G_CALLBACK (on_sqle_host_changed),
                    NULL);
                    
	//Close SQL Editor
  widget = glade_xml_get_widget(xml, "sqle_close");	
	g_signal_connect ((gpointer) widget, "clicked",
                    G_CALLBACK (on_sqle_close_clicked),
                    NULL);   
   
  widget = glade_xml_get_widget(xml, "notebook1");	                  
  g_signal_connect ((gpointer) widget, "switch-page",
                    G_CALLBACK (on_sqle_notebook_change_page),
                    NULL);
} // connect_sqle_signals ()

///////////////////////////////////////// SQL Editor callbacks ////////////////////////////////////////////////////////////////
void
on_sqle_notebook_change_page (GtkNotebook *nb, GtkNotebookPage *page, guint page_num, gpointer user_data)
{
	switch(page_num){
	  case 0: gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "label184")), "<span font_desc='9 Bold' foreground='#FFFFFF'>Data</span>");
									break;
		case 1: gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "label184")), "<span font_desc='9 Bold' foreground='#FFFFFF'>Messages</span>");
									break;
	}
}

void 
on_tree_view_changed(GtkTreeView *tree_view, gpointer user_data){
	if(GTK_IS_NOTEBOOK(GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")))){
		gtk_notebook_set_current_page( GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")), 0);
	}
}

void
on_sqle_execute_clicked(GtkButton *button, gpointer user_data){
		GtkTreeModel * model = gtk_tree_view_get_model(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")));
		if(model != NULL){
			gtk_list_store_clear(GTK_LIST_STORE(model));
			GList * list = gtk_tree_view_get_columns(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")));
			for(; list != NULL; list = list->next) gtk_tree_view_remove_column(GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")), GTK_TREE_VIEW_COLUMN(list->data));
		}
		gint s = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_host")));
		gchar * db = copy_string(gtk_entry_get_text(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "sqle_database"))->child)));
		gint d = -1;
		if(s == -1 || strcmp(db,"") == 0) return;
		
		for(int i = 0; i < o_tree.server[s]->n_profiles; i++){
			if(strcmp(o_tree.server[s]->profile[i]->dbname, db)==0){
				d = i;
				break;
			}
		}
		if(d == -1) return;
		free(db);
		
		if(GTK_IS_NOTEBOOK(GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")))){
			gtk_notebook_set_current_page( GTK_NOTEBOOK(glade_xml_get_widget(xml, "notebook1")), 1);
		}
		
		
		GtkTextBuffer * buffer;
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (glade_xml_get_widget(xml, "sqle_code")));
		GtkTextIter start, end;
  	gtk_text_buffer_get_start_iter(buffer, &start);
  	gtk_text_buffer_get_end_iter(buffer, &end);
  	gchar * sql = gtk_text_buffer_get_text(buffer, &start, &end, TRUE);
  
		execute_sql(sql, GTK_TREE_VIEW (glade_xml_get_widget(xml, "sqle_data_view")), GTK_TEXT_VIEW (glade_xml_get_widget(xml, "sqle_messages")),  o_tree.server[s]->profile[d], FALSE, FALSE);
}


void
on_sqle_close_clicked(GtkButton *button, gpointer user_data){
	gtk_widget_hide(glade_xml_get_widget(xml, "sql_editor"));
}

void
on_sqle_host_changed(GtkEditable *editable, gpointer user_data){
	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(glade_xml_get_widget(xml, "sqle_database"))->child), "");
	
	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_database"))));
	gtk_list_store_clear(store);
	//gtk_combo_box_set_active(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_database")) , -1);

	
	gint i = gtk_combo_box_get_active(GTK_COMBO_BOX(glade_xml_get_widget(xml, "sqle_host")));
	gint n=0;
		
	for(int j = 0; j < o_tree.server[i]->n_profiles; j++){
		if(o_tree.server[i]->profile[j]->connected){
			 gtk_combo_box_append_text(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_database")),o_tree.server[i]->profile[j]->dbname);
			 n++;
		}
	}
	if(n > 0){
	 gtk_combo_box_set_active(GTK_COMBO_BOX (glade_xml_get_widget(xml, "sqle_database")) , 0);
	 gtk_widget_show(glade_xml_get_widget(xml, "sqle_execute"));
	}
	else gtk_widget_hide(glade_xml_get_widget(xml, "sqle_execute"));
}
